/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "PxIconPicker.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void AFXAPI DDX_PxIconPicker(CDataExchange *pDX, int nIDC, CString& sIconFile, UINT& nIconIndex)
{
    HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
    ASSERT (hWndCtrl != NULL);                
    
    CPxIconPicker* pIconPicker = (CPxIconPicker*)CWnd::FromHandle(hWndCtrl);
    if (pDX->m_bSaveAndValidate)
    {
		sIconFile = pIconPicker->GetIconFile();
		nIconIndex = pIconPicker->GetIconIndex(); 
    }
    else // initializing
    {
		pIconPicker->SetIcon(sIconFile,nIconIndex);
    }
}

/////////////////////////////////////////////////////////////////////////////
// CPxIconPicker

CPxIconPicker::~CPxIconPicker()
{
}

CSize CPxIconPicker::GetIconSize()
{
	return CSize(
		::GetSystemMetrics(m_nIconMode == IconSmall?SM_CXSMICON:SM_CXICON),
		::GetSystemMetrics(m_nIconMode == IconSmall?SM_CYSMICON:SM_CYICON));
}

void CPxIconPicker::AdjustSize()
{
	CRect rc;
	GetWindowRect(rc);
	GetParent()->ScreenToClient(rc);
	
	CSize sz = GetIconSize();
	sz.cx += 2 * ::GetSystemMetrics(SM_CXVSCROLL) + 4 + 6; //two buttons
	rc.right  = rc.left + sz.cx;
	rc.bottom = rc.top + 4;
	MoveWindow(rc);
	SetItemHeight(-1,sz.cy+4);
}

BOOL CPxIconPicker::LoadIcons(LPCTSTR pszFileName)
{
	UINT nCount = ExtractIconEx(pszFileName,-1,NULL,NULL,0);
	if( nCount <= 0 ) return FALSE;

	this->ResetContent();
	m_aIcons.RemoveAll();
	m_aIcons.SetSize(nCount,0);
	if( m_nIconMode == IconLarge )
		nCount = ::ExtractIconEx(pszFileName,0,m_aIcons.GetData(),NULL,nCount); else
		nCount = ::ExtractIconEx(pszFileName,0,NULL,m_aIcons.GetData(),nCount);
	for(UINT i=0; i<nCount; i++)
		AddString((LPCTSTR)i);
	m_sIconFile = pszFileName;
	return TRUE;
}

void CPxIconPicker::SetIconMode(IconMode nMode)
{
	if( m_nIconMode != nMode ) {
		m_nIconMode = nMode;
		AdjustSize();
		if( !m_sIconFile.IsEmpty() )
			LoadIcons(m_sIconFile);
	}
}

BOOL CPxIconPicker::Create(DWORD dwStyle,
	const CRect &rect, CWnd *pParentWnd, UINT nID)
{

	BOOL bResult;

	m_bCreating = TRUE;
	VERIFY(bResult = CComboBox::Create(
		dwStyle|CBS_OWNERDRAWVARIABLE|CBS_DROPDOWNLIST,
		rect, pParentWnd, nID));
	m_bCreating = FALSE;

	if( !bResult )
		return FALSE;

	VERIFY(bResult = m_OpenButton.Create(NULL,
		WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		CRect(), this, IDOK));
	if( !bResult )
		return FALSE;
	AdjustSize();

	return TRUE;
}

BEGIN_MESSAGE_MAP(CPxIconPicker, CComboBox)
	//{{AFX_MSG_MAP(CPxIconPicker)
	ON_BN_CLICKED(IDOK, OnBrowse)
	ON_WM_SIZE()
	ON_WM_DRAWITEM()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPxIconPicker message handlers

void CPxIconPicker::PreSubclassWindow() 
{
	CComboBox::PreSubclassWindow();

	if( !m_bCreating ) {
		const UINT nStyle = CBS_OWNERDRAWVARIABLE|CBS_DROPDOWNLIST;
		ASSERT((GetStyle() & nStyle) == nStyle );

		VERIFY(m_OpenButton.Create(NULL, WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
			CRect(), this, IDOK));
		AdjustSize();
		CRect rc; GetClientRect(rc);
		OnSize(SIZE_RESTORED, rc.Width(), rc.Height());
	}
}

void CPxIconPicker::MeasureItem(LPMEASUREITEMSTRUCT lpMS) 
{
	CSize sz = GetIconSize();
	lpMS->itemWidth  = sz.cx;
	lpMS->itemHeight = sz.cy+4;
}

void CPxIconPicker::DrawItem(LPDRAWITEMSTRUCT lpDS) 
{
	CDC dc; dc.Attach(lpDS->hDC); int nSaveID = dc.SaveDC();

	if( lpDS->itemState & ODS_SELECTED ) {
		dc.SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
		dc.SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
	} else {
		dc.SetBkColor(::GetSysColor(COLOR_WINDOW));
		dc.SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
	}
	if( lpDS->itemState & ODS_DISABLED ) {
		dc.SetBkColor(::GetSysColor(COLOR_BTNFACE));
		dc.SetTextColor(::GetSysColor(COLOR_GRAYTEXT));
	} 
	
	CRect rc(lpDS->rcItem);
	if( lpDS->itemState & ODS_COMBOBOXEDIT )
		rc.right -= ::GetSystemMetrics(SM_CXVSCROLL);
	dc.FillRect(rc, &CBrush(dc.GetBkColor()));
	if( lpDS->itemState & ODS_SELECTED )
		dc.DrawFocusRect(rc);

	if( lpDS->itemID != -1 ) {
		CSize szIcon = GetIconSize();
		if( lpDS->itemState & ODS_DISABLED ) {
			dc.DrawState(
				CPoint(rc.left+max(rc.Width()-szIcon.cx,0)/2,
				rc.top+max(rc.Height()-szIcon.cy,0)/2),
				CSize(0,0), m_aIcons[lpDS->itemID], DSS_DISABLED, (HBRUSH)NULL);
		} else {
			::DrawIconEx(lpDS->hDC, 
				rc.left+max(rc.Width()-szIcon.cx,0)/2,
				rc.top+max(rc.Height()-szIcon.cy,0)/2,
				m_aIcons[lpDS->itemID], 0,0, NULL, NULL, DI_NORMAL);
		}
	}
	dc.RestoreDC(nSaveID); dc.Detach();

	if( lpDS->itemState & ODS_COMBOBOXEDIT 
		&& m_OpenButton.GetSafeHwnd())
		m_OpenButton.Invalidate();
}

void CPxIconPicker::OnSize(UINT nType, int cx, int cy) 
{// place "Browse" button properly - next to dropdown button
	CComboBox::OnSize(nType, cx, cy);
	if( m_OpenButton.GetSafeHwnd() ) {
		CRect rc;
		GetClientRect(rc);
		rc.InflateRect(-2,-2); // 3d border
		rc.right -= ::GetSystemMetrics(SM_CXVSCROLL);
		rc.left  = rc.right - ::GetSystemMetrics(SM_CXVSCROLL);
		m_OpenButton.MoveWindow(rc);
	}
}

void CPxIconPicker::OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDS) 
{//draw "Browse" button
	if( nIDCtl == IDOK ) {
		ASSERT(lpDS->CtlType == ODT_BUTTON);
		CRect rc(lpDS->rcItem);
		::DrawFrameControl(lpDS->hDC, &lpDS->rcItem, DFC_SCROLL, DFCS_SCROLLCOMBOBOX|
			(lpDS->itemState&ODS_DISABLED?DFCS_INACTIVE:NULL)|
			(lpDS->itemState&ODS_SELECTED?DFCS_FLAT:NULL));
		rc.InflateRect(-2,-2);
		::FillRect(lpDS->hDC, rc, (HBRUSH)(COLOR_BTNFACE+1));
		rc = lpDS->rcItem;
		rc = CRect((rc.Width()-8)/2,rc.Height()/2-1,(rc.Width()-8)/2+2,rc.Height()/2+1);
		if( lpDS->itemState&ODS_SELECTED ) rc += CSize(1,1);
		for(int i=0;i<3;i++,rc+=CSize(3,0)) {
			if( GetStyle()&WS_DISABLED ) {
				::FillRect(lpDS->hDC, rc+CSize(1,1), (HBRUSH)(COLOR_3DLIGHT+1));
				::FillRect(lpDS->hDC, rc, (HBRUSH)(COLOR_GRAYTEXT+1));
			} else 
				::FillRect(lpDS->hDC, rc, (HBRUSH)(COLOR_BTNTEXT+1));
		}
	} else
		CComboBox::OnDrawItem(nIDCtl, lpDS);
}

void CPxIconPicker::OnBrowse() 
{//handle click on "Browse" button
	CFileDialog fd(TRUE,NULL,m_sIconFile,OFN_HIDEREADONLY|OFN_FILEMUSTEXIST|OFN_NOCHANGEDIR,
		"Icon libraries (*.ico;*.exe;*.dll;*.ocx)|*.ico;*.exe;*.dll;*.ocx|All Files (*.*)|*.*||",
		this);
	if( fd.DoModal() == IDOK ) {
		if( !LoadIcons(fd.GetPathName()) )
			::MessageBeep(MB_ICONSTOP);
		else SetIcon((UINT)0);
	}
}


BOOL CPxIconPicker::SetIcon(LPCTSTR pszIconFile, UINT nIconIndex)
{
	if( LoadIcons(pszIconFile) )
		return SetIcon(nIconIndex);
	return FALSE;
}

BOOL CPxIconPicker::SetIcon(LPCTSTR pszIconFileWithIndex)
{
	LPCTSTR pszNumber;
	if( pszNumber = _tcsrchr(pszIconFileWithIndex, ',') ) {
		UINT nIndex = atoi(pszNumber+1);
		LPTSTR pszFName = _tcsdup(pszIconFileWithIndex);
		pszFName[pszNumber-pszIconFileWithIndex] = 0;
		BOOL bResult = SetIcon(pszFName, nIndex);
		free(pszFName);
		return bResult;
	} else
		return SetIcon(pszIconFileWithIndex, 0);
}

BOOL CPxIconPicker::SetIcon(UINT nIconIndex)
{
	return SetCurSel(nIconIndex);
}

