/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __PXICONPICKER_H_INCLUDED__
#define __PXICONPICKER_H_INCLUDED__
#pragma once

/////////////////////////////////////////////////////////////////////////////
// CPxIconPicker window
void AFXAPI DDX_PxIconPicker(CDataExchange *pDX, int nIDC, CString& sIconFile, UINT& nIconIndex);

inline 
void AFXAPI DestructElements(HICON* pItems, int nCount)
{
	for( ;nCount>0;nCount-- ) ::DestroyIcon(*pItems++);
}

class CPxIconPicker : public CComboBox
{
// Construction
	BOOL m_bCreating;
public:
	enum IconMode{
		IconLarge = 1,
		IconSmall = 2
	};

	CPxIconPicker(IconMode nMode=IconLarge):
	  m_nIconMode(nMode),
	  m_bCreating(FALSE) {}

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CPxIconPicker)
	public:
	virtual void MeasureItem(LPMEASUREITEMSTRUCT lpMeasureItemStruct);
	virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
	protected:
	virtual void PreSubclassWindow();
	//}}AFX_VIRTUAL

// Implementation
public:
	BOOL SetIcon(UINT nIconIndex);
	BOOL SetIcon(LPCTSTR pszIconFileWithIndex);
	BOOL SetIcon(LPCTSTR pszIconFile, UINT nIconIndex);
	BOOL Create(DWORD dwStyle, const CRect& rect, CWnd* pParentWnd, UINT nID);
	void SetIconMode(IconMode nMode);
	IconMode GetIconMode() { return m_nIconMode; }
	CSize GetIconSize();
	const CString& GetIconFile() { return m_sIconFile; }
	UINT GetIconIndex() { return GetCurSel(); }
	BOOL LoadIcons(LPCTSTR pszFileName);

	virtual ~CPxIconPicker();
	// Generated message map functions
protected:
	CArray<HICON,HICON> m_aIcons;
	CButton m_OpenButton;
	IconMode m_nIconMode;
	CString m_sIconFile;
	void AdjustSize();
	//{{AFX_MSG(CPxIconPicker)
	afx_msg void OnBrowse();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnDrawItem(int nIDCtl, LPDRAWITEMSTRUCT lpDrawItemStruct);
	//}}AFX_MSG

	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif //__PXICONPICKER_H_INCLUDED__


