#ifndef __PXMENU_H_INCLUDED__
#define __PXMENU_H_INCLUDED__
////////////////////////////////////////////////////////////////
// 1997 Microsoft Systems Journal. 
// If this code works, it was written by Paul DiLascia.
// If not, I don't know who wrote it.
//
// Various changes and improvements (c) Vassili Bourdo
// if curious what was actually changed, diff it with
// original version :-]
#include "SubClass.h"

#ifndef __AFXTEMPL_H__
	#include <afxtempl.h>
#endif

namespace PxLib {

extern void FillRect(CDC& dc, const CRect& rc, COLORREF color);
extern void FillRect(CDC& dc, const CRect& rc, CBrush& brush);
extern void DrawEmbossed(CDC& dc, CImageList& il, int i,
	CPoint p, BOOL bColor=FALSE);

extern HBITMAP LoadSysColorBitmap(LPCTSTR lpResName, BOOL bMono=FALSE);
inline HBITMAP LoadSysColorBitmap(UINT nResID, BOOL bMono=FALSE)
{
	return LoadSysColorBitmap(MAKEINTRESOURCE(nResID), bMono);
}

} // end namespace

//////////////////
// CPxMenuManager implements "cool" menus with buttons in them. To use:
//
//  *	Instantiate in your CMainFrame.
//	 * Call Install to install it
//  * Call LoadToolbars or LoadToolbar to load toolbars
//
//  Don't forget to link with CoolMenu.cpp, Subclass.cpp and DrawTool.cpp!
//
class CPxMenuManager : private CSubclassWnd {
public:
	static COLORREF m_crImageBkColor;
public:
	BOOL SetItemImage(UINT nID, CBitmap* pImage, CBitmap* pMask);
	BOOL SetItemImage(UINT nID, CBitmap* pImage, COLORREF crMask=m_crImageBkColor);
	BOOL SetItemImage(UINT nID, LPCTSTR pszResourceID, COLORREF crMask=m_crImageBkColor);
	BOOL SetItemImage(UINT nID, HICON hIcon);
	BOOL SetItemImage(UINT nID, UINT nImageID);
	BOOL SetItemOverlay(UINT nID, int nOverlayID);
	void ResetOverlay() { m_mapIDtoOverlay.RemoveAll(); }

	CPxMenuManager();
	~CPxMenuManager();

	// You can set these any time
	BOOL m_bShowButtons;				// use to control whether buttons are shown
	BOOL m_bAutoAccel;					// generate auto accelerators
	BOOL m_bUseDrawState;				// use ::DrawState for disabled buttons
	BOOL m_bDrawDisabledButtonsInColor; // draw disabled buttons in color
										// (only if m_bUseDrawState = FALSE)

	// public functions to use
	void Install(CWnd* pFrame);						// connect to main frame
	BOOL LoadToolbars(const UINT* arIDs, int n);	// load multiple toolbars
	BOOL LoadToolbar(UINT nID);						// load one toolbar

	// should never need to call:
	virtual void Destroy();					// destroys everything--to re-load new toolbars?
	virtual void Refresh();					// called when system colors, etc change
	static  HBITMAP GetMFCDotBitmap();		// get..
	static  void    FixMFCDotBitmap();		// and fix MFC's dot bitmap
protected:
	CWnd*				m_pFrame;			// frame window I belong to
	CArray<UINT,UINT>	m_arToolbarID;		// array of toolbar IDs loaded
	CImageList			m_ilButtons;		// image list for all buttons
	CMap<UINT,UINT,int,int>			m_mapIDtoImage;		// maps command ID -> image number
	CMap<UINT,UINT,int,int>			m_mapIDtoOverlay;	// maps command ID -> overlay number
	CMap<UINT,UINT,ACCEL*,ACCEL*>	m_mapIDtoAccel;		// maps command ID -> ACCEL*
	HACCEL				m_hAccel;			// current accelerators, if any
	ACCEL*				m_pAccel;			// ..and table in memory
	CList<HMENU,HMENU>	m_menuList;			// list of HMENU's initialized
	CSize				m_szBitmap;			// size of button bitmap
	CSize				m_szButton;			// size of button (including shadow)
	CFont				m_fontMenu;			// menu font
	CFont				m_fontMenuBold;		// menu font for painting default items
	CBitmap				m_bmpPushedFiller;	// bitmap brush for filling pushed down buttons

	// helpers
	void DestroyAccel();
	void DrawMenuText(CDC& dc, CRect rc, CString text, COLORREF color, BOOL bBold = FALSE);
	BOOL Draw3DCheckmark(CDC& dc, const CRect& rc, BOOL bSelected,
				HBITMAP hbmCheck=NULL);
	void ConvertMenu(CMenu* pMenu,UINT nIndex,BOOL bSysMenu,BOOL bShowButtons);
	void LoadAccel(HACCEL hAccel);
	BOOL AppendAccelName(CString& sItemName, UINT nID);
	CFont* GetMenuFont(BOOL bBold = FALSE);
	CBitmap* GetPushedFiller();

	// Get button index for given command ID, or -1 if not found
	int  GetButtonIndex(WORD nID) {
		int val;
		return m_mapIDtoImage.Lookup(nID, val) ? val : -1;
	}

	// Get ACCEL structure associated with a given command ID
	ACCEL* GetAccel(WORD nID) {
		ACCEL* val;
		return m_mapIDtoAccel.Lookup(nID, val) ? val : NULL;
	}

	// window proc to hook frame using CSubclassWnd implementation
	virtual LRESULT WindowProc(UINT msg, WPARAM wp, LPARAM lp);

	// CSubclassWnd message handlers 
	virtual void OnInitMenuPopup(CMenu* pMenu, UINT nIndex, BOOL bSysMenu);
	virtual BOOL OnMeasureItem(LPMEASUREITEMSTRUCT lpms);
	virtual BOOL OnDrawItem(LPDRAWITEMSTRUCT lpds);
	virtual LONG OnMenuChar(UINT nChar, UINT nFlags, CMenu* pMenu);
	virtual void OnMenuSelect(UINT nItemID, UINT nFlags, HMENU hSysMenu);
};

#endif //__COOLMENU_H_INCLUDED__
