/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __PXMISC_H_INCLUDED__
#define	__PXMISC_H_INCLUDED__

#ifndef __PXMODVER_H
#include "pxmodver.h"
#endif

#define countof(array)	(sizeof(array)/sizeof(array[0]))

template<class T> 
struct INIT: public T {
	INIT(BOOL bZeroMe=TRUE)
		{
			if( bZeroMe) clear();
		}
	INIT(const T& other)
		{
			*this = other;
		}
	operator =(const T& other)
		{
			CopyMemory(this,T,sizeof(T))
		}
	void clear()
		{
			ZeroMemory(this, sizeof(T));
		}
};

template<class T> 
struct INIT2: public T {
	INIT2(BOOL bZeroMe=TRUE)
		{
			if( bZeroMe ) clear();
		}
	INIT2(const T& other)
		{
			*this = other;
		}
	operator =(const T& other)
		{
			CopyMemory(this,T,sizeof(T))
		}
	void clear()
		{
			ZeroMemory(this, sizeof(T));
			cbSize=sizeof(T);
		}
};

inline
CString LoadString(UINT nResourceID)
{
	CString str; str.LoadString(nResourceID); return str;
}

inline
CString StrPrintf(LPCTSTR fmt,...)
{
	va_list args;
	va_start(args, fmt);
	CString param;
	param.FormatV(fmt,args);
	va_end(args);
	return param;
}
 
inline
CString StrPrintf(UINT nFmtResource,...)
{
	va_list args;
	va_start(args, nFmtResource);
	CString param;
	param.FormatV(::LoadString(nFmtResource), args);
	va_end(args);
	return param;
}
 
inline
BOOL PxCheckModuleVersion(LPCTSTR pszModule,
	DWORD dwVersionMajor, DWORD dwVersionMinor, LPCTSTR pszMessage)
{
	DWORD dwVersion = CPxModuleVersion::DllGetVersion(pszModule);
	if( dwVersion == 0 ) {
		CPxModuleVersion mv(pszModule);
		dwVersion = mv.dwFileVersionMS;
	}
	if( dwVersion < (DWORD)MAKELONG(dwVersionMinor, dwVersionMajor) ) {
		AfxMessageBox(
			StrPrintf(pszMessage, pszModule, dwVersionMajor, dwVersionMinor),
			MB_ICONSTOP|MB_OK);
		return FALSE;
	}
	return TRUE;
}

inline
BOOL PxCheckModuleVersion(LPCTSTR pszModule,
	DWORD dwVersionMajor, DWORD dwVersionMinor,
	UINT nMessageID)
{
	return PxCheckModuleVersion(pszModule, dwVersionMajor, dwVersionMinor,
		LoadString(nMessageID));
}

inline
void AFXAPI DDX_SpinMinMax(CDataExchange* pDX, int nIDC, int &nMin, int &nMax)
{
    HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
    ASSERT (hWndCtrl != NULL);                
    
    CSpinButtonCtrl* pSpin= (CSpinButtonCtrl*) CWnd::FromHandle(hWndCtrl);
    if (pDX->m_bSaveAndValidate) {
        pSpin->GetRange(nMin, nMax);
	} else {
        pSpin->SetRange(nMin, nMax);
    }
}

#endif	//__PXMISC_H_INCLUDED__
