////////////////////////////////////////////////////////////////
// 1998 Microsoft Systems Journal
// If this code works, it was written by Paul DiLascia.
// If not, I don't know who wrote it.
//
// CPxModuleVersion provides an easy way to get version info
// for a module.(DLL or EXE).
//
#include "StdAfx.h"
#include "PxModVer.h"
#include "PxMisc.h"
CPxModuleVersion::TRANSLATION CPxModuleVersion::m_defaultTranslation = { 0x419, 1200 };

CPxModuleVersion::CPxModuleVersion(LPCTSTR modulename):
	m_pVersionInfo(NULL)
{
	if( modulename ) GetFileVersionInfo(modulename);
}

//////////////////
// Destroy: delete version info
//
CPxModuleVersion::~CPxModuleVersion()
{
	if( m_pVersionInfo ) delete [] m_pVersionInfo;
}

//////////////////
// Get file version info for a given module
// Allocates storage for all info, fills "this" with
// VS_FIXEDFILEINFO, and sets codepage.
//
BOOL CPxModuleVersion::GetFileVersionInfo(LPCTSTR modulename)
{
	if( m_pVersionInfo ) {
		delete [] m_pVersionInfo;
		m_pVersionInfo = NULL;
	}

	m_pTranslation = &m_defaultTranslation;
	m_nTranslationCount = 1;
	memset((VS_FIXEDFILEINFO*)this, 0, sizeof(VS_FIXEDFILEINFO));

	// get module handle
	TCHAR filename[_MAX_PATH];
	HMODULE hModule = ::GetModuleHandle(modulename);
	if (hModule==NULL && modulename!=NULL)
		return FALSE;

	// get module file name
	DWORD len = GetModuleFileName(hModule, filename, countof(filename));
	if( len <= 0 )
		return FALSE;

	// read file version info
	DWORD dwDummyHandle; // will always be set to zero
	len = GetFileVersionInfoSize(filename, &dwDummyHandle);
	if( len <= 0 )
		return FALSE;

	m_pVersionInfo = new BYTE[len]; // allocate version info
	if (!::GetFileVersionInfo(filename, 0, len, m_pVersionInfo))
		return FALSE;

	LPVOID lpvi;
	UINT iLen;
	if( !VerQueryValue(m_pVersionInfo, _T("\\"), &lpvi, &iLen) )
		return FALSE;

	// copy fixed info to myself, which am derived from VS_FIXEDFILEINFO
	*(VS_FIXEDFILEINFO*)this = *(VS_FIXEDFILEINFO*)lpvi;

	// Get translation info
	if( VerQueryValue(m_pVersionInfo, _T("\\VarFileInfo\\Translation"), &lpvi, &iLen) &&
		iLen >= sizeof(TRANSLATION))
	{
		m_pTranslation = (TRANSLATION*)lpvi;
		m_nTranslationCount = iLen / sizeof(TRANSLATION);
	}
	return dwSignature == VS_FFI_SIGNATURE;
}

//////////////////
// Get string file info.
// Key name is something like "CompanyName".
// returns the value as a CString.
//
CString CPxModuleVersion::GetValue(LPCTSTR lpKeyName)
{
	CString sVal;
	if( m_pVersionInfo ) {
	//--figure out available translation that match 
	//	current thread locale
		int nLang = LANGIDFROMLCID(GetThreadLocale());
		for( int i=0;i < m_nTranslationCount; i++ )
			 if( m_pTranslation[i].langID == nLang ) break;
		if( i >= m_nTranslationCount ) i = 0;

		CString query;
		query.Format(_T("\\StringFileInfo\\%04x%04x\\%s"),
			m_pTranslation[i].langID , m_pTranslation[i].charset, lpKeyName);

		LPCTSTR pVal;
		UINT iLen;
		if( VerQueryValue(m_pVersionInfo, (LPTSTR)(LPCTSTR)query, (LPVOID*)&pVal, &iLen) )
			sVal = pVal;
	}
	return sVal;
}

// typedef for DllGetVersion proc
typedef HRESULT (CALLBACK* DLLGETVERSIONPROC)(DLLVERSIONINFO *);

/////////////////
// Get DLL Version by calling DLL's DllGetVersion proc
//
BOOL CPxModuleVersion::DllGetVersion(LPCTSTR modulename, DLLVERSIONINFO& dvi)
{
	BOOL bResult = FALSE;
	HINSTANCE hinst = LoadLibrary(modulename);
	if( hinst ) {
		DLLGETVERSIONPROC pDllGetVersion =
			(DLLGETVERSIONPROC)GetProcAddress(hinst, "DllGetVersion");
		if( pDllGetVersion ) {
			memset(&dvi, 0, sizeof(dvi));
			dvi.cbSize = sizeof(dvi);
			bResult = SUCCEEDED((*pDllGetVersion)(&dvi));
		}
		FreeLibrary(hinst);
	}
	return bResult;
}

DWORD CPxModuleVersion::DllGetVersion(LPCTSTR modulename)
{
	DLLVERSIONINFO dvi;
	if( DllGetVersion(modulename,dvi) ) {
		return MAKELONG(dvi.dwMinorVersion, dvi.dwMajorVersion);
	} else
		return 0;
}
