/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __PxNotify_H_INCLUDED__
#define __PxNotify_H_INCLUDED__

//file system change notification
class CPxChangeNotify {
protected:
	HANDLE	hNotify;
public:
	enum {
		FileName    = FILE_NOTIFY_CHANGE_FILE_NAME,
		DirName     = FILE_NOTIFY_CHANGE_DIR_NAME,
		Attributes  = FILE_NOTIFY_CHANGE_ATTRIBUTES,
		Size		= FILE_NOTIFY_CHANGE_SIZE,
		Write		= FILE_NOTIFY_CHANGE_LAST_WRITE,
		Seciruty    = FILE_NOTIFY_CHANGE_SECURITY
	} FileChangeFlags;

	CPxChangeNotify():
			hNotify(NULL)
		{}
	CPxChangeNotify(
		LPCTSTR lpPathName,
		DWORD dwNotifyFilter,
		BOOL bWatchSubtree = FALSE):
			hNotify(NULL)
		{
			Create(lpPathName, dwNotifyFilter, bWatchSubtree);
		}
	~CPxChangeNotify()
		{
			Destroy();
		}
	BOOL Create(
		LPCTSTR lpPathName,
		DWORD dwNotifyFilter,
		BOOL bWatchSubtree = FALSE)
		{
			Destroy();
			if( _tcsncmp(lpPathName, "\\\\", 2) == 0 )
				return FALSE;
			hNotify = ::FindFirstChangeNotification(
				lpPathName, bWatchSubtree, dwNotifyFilter);
			if( hNotify == INVALID_HANDLE_VALUE )
				hNotify = NULL;
			return hNotify != NULL;
		}
	void Destroy()
		{
			if( hNotify ) ::FindCloseChangeNotification(hNotify);
			hNotify = NULL;
		}
	operator HANDLE() const
		{
			return hNotify;
		}
	BOOL Changed(DWORD dwWaitFor = 0) const
		{
			if( !hNotify )
				return FALSE;
			if( WaitForSingleObject(hNotify,dwWaitFor) == WAIT_OBJECT_0 ) {
				::FindNextChangeNotification(hNotify);
				return TRUE;
			}
			return FALSE;
		}
};

#endif //__PxNotify_H_INCLUDED__
