#ifndef __PXTEMPL_H_INCLUDED__
#define __PXTEMPL_INCLUDED__

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#ifndef __AFXTEMPL_H__
#include <afxtempl.h>
#endif

// tips & triks. Speed optimization and bug detecting compromise
#ifdef _DEBUG
	#define DATA_ACCESS_OPERATOR(i) ((*this)[i]) // for better bug tracking
#else
	#define DATA_ACCESS_OPERATOR(i) (m_pData[i]) // 10 times faster
#endif

/*
Compare two elements. Used for array sorting if compare
function is not assigned.
Returns:
	0	if	pElement1 == pElement2
	1	if	pElement1 >  pElement2
	-1	if	pElement1 <  pElement2
*/
template<class TYPE>
int AFXAPI PxCompareElements(const TYPE* pElement1, const TYPE* pElement2)
{
	ASSERT(AfxIsValidAddress(pElement1, sizeof(TYPE), FALSE));
	ASSERT(AfxIsValidAddress(pElement2, sizeof(TYPE), FALSE));

	if( *pElement1 == *pElement2 ) return 0; else
	if( *pElement1 > *pElement2 ) return 1; else
	return -1;
}

template<class TYPE, class ARG_TYPE>
class CPxArray:
	public CArray<TYPE,ARG_TYPE>
{
public:// Typedefs
	typedef int (AFXAPI *CMPFUNC)(const TYPE*,const TYPE*);
private:// Atributes
// variable for sort speed tunning. Recommended range [3..128]
	int m_nCutOff; 
	CMPFUNC m_pCompareFunc;
public:// Construction
	CPxArray ();
	CPxArray (const CPxArray &x);
public:// Assigment
	CPxArray &operator = (const CArray<TYPE, ARG_TYPE> &x);
	CPxArray &operator = (const CPxArray &x);
	operator CArray<TYPE, ARG_TYPE> ();
public:// Comparison
	BOOL operator < (const CPxArray &x) const;
	BOOL operator <= (const CPxArray &x) const;
	BOOL operator == (const CPxArray &x) const;
	BOOL operator != (const CPxArray &x) const;
	BOOL operator > (const CPxArray &x) const;
	BOOL operator >= (const CPxArray &x) const;
public:// Methods
	void Sort(CMPFUNC cmpFunc=NULL);
	void Sort(int nLow, int nHigh, CMPFUNC cmpFunc=NULL);
	CMPFUNC SetCompareFunc(CMPFUNC func)
		{
			CMPFUNC prev = m_pCompareFunc;
			m_pCompareFunc = func;
			return prev;
		}
protected:// internal sorting
	void InsertionSort(int nLow, int nHigh);
	void QuickSort(int nLow, int nHigh);
	int Compare(const TYPE *e1, const TYPE *e2) const
		{
			if( m_pCompareFunc ) return m_pCompareFunc(e1,e2); else
			return PxCompareElements<TYPE>(e1,e2);
		}
};

template<class TYPE, class ARG_TYPE>
class CPxSortedArray:
	public CPxArray<TYPE,ARG_TYPE>
{
public:// Construction
	CPxSortedArray();
	CPxSortedArray(const CPxSortedArray &x);
	virtual ~CPxSortedArray();
public:// Assigment
	CPxSortedArray &operator = (const CArray<TYPE, ARG_TYPE> &x);
	CPxSortedArray &operator = (const CPxArray<TYPE, ARG_TYPE>  &x);
	CPxSortedArray &operator = (const CPxSortedArray &x);
public:// Operator
	operator CPxArray<TYPE, ARG_TYPE> ();
public:// Methods
// insert new element to sorted array and return insertion index
	int AddSorted(ARG_TYPE tValue);		
// insert new element to sorted array.
	void SetAt(ARG_TYPE tValue);
// binary search if fail return -1
	int Lookup(ARG_TYPE tValue) const;
// binary search if fail return nearest item
	int Search(ARG_TYPE tValue) const;
};

//==============================================================================
// Helpful template functions

// swaps two elements
template<class T> AFX_INLINE void Swap(T& x, T& y)
{
	T temp;

	temp = x;
	x = y;
	y = temp;
}

// swaps two elements and returns value
// usefull for situation when need return old value
template<class T> AFX_INLINE T& SwapRet(T& x, T& y)
{
	T temp;

	temp = x;
	x = y;
	y = temp;

	return x;
}

// template function returns y if x < y
// To avoid conflicts with min and max in WINDEF.H
template<class T> AFX_INLINE const T& Max(const T& x, const T& y)
{
	if (x > y) {
		return x;
	}

	return y;
}


// template function returns y if x > y
// To avoid conflicts with min and max in WINDEF.H
template<class T> AFX_INLINE const T& Min(const T& x, const T& y)
{
	if (x < y) {
		return x;
	}

	return y;
}

//==============================================================================
// Implementation of the CPxArray class.
template<class TYPE, class ARG_TYPE>
AFX_INLINE
CPxArray<TYPE,ARG_TYPE>::CPxArray():
	m_nCutOff(64), m_pCompareFunc(NULL)
{
}


template<class TYPE, class ARG_TYPE>
AFX_INLINE
CPxArray<TYPE,ARG_TYPE>::CPxArray(const CPxArray &x):
	m_pCompareFunc(NULL)
{
	*this = x;
}

template<class TYPE, class ARG_TYPE>
AFX_INLINE CPxArray<TYPE,ARG_TYPE> &
CPxArray<TYPE,ARG_TYPE>::operator = (const CArray<TYPE, ARG_TYPE> &x)
{
	if(this != &x) {
		Copy(x);
	}

	return *this;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxArray<TYPE,ARG_TYPE>& CPxArray<TYPE,ARG_TYPE>::operator = (const CPxArray &x)
{
	if(this != &x) {
		Copy(x);
		m_nCutOff = x.m_nCutOff;
		m_pCompareFunc = x.m_pCompareFunc;
	}

	return *this;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator < (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) < x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator <= (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) <= x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator == (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) == x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator != (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) != x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator > (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) > x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
BOOL CPxArray<TYPE,ARG_TYPE>::operator >= (const CPxArray &x) const
{
	int i;

	if(GetSize() != x.GetSize()) {
		return FALSE;
	}

	for(i = 0; i < GetSize(); i ++) {
		if(!(DATA_ACCESS_OPERATOR(i) >= x[i])) {
			return FALSE;
		}
	}

	return TRUE;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxArray<TYPE,ARG_TYPE>::operator CArray<TYPE, ARG_TYPE > ()
{
	return *this;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
void CPxArray<TYPE,ARG_TYPE>::Sort(CMPFUNC cmpFunc/*=NULL*/)
{
	CMPFUNC prev=NULL;
	if( cmpFunc ) prev = SetCompareFunc(cmpFunc);
	QuickSort(0, GetSize() - 1);
	if( prev ) SetCompareFunc(prev);
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
void CPxArray<TYPE,ARG_TYPE>::Sort(int nLow, int nHigh, CMPFUNC cmpFunc/*=NULL*/)
{
	CMPFUNC prev=NULL;
	if( cmpFunc ) prev = SetCompareFunc(cmpFunc);
	QuickSort(nLow, nHigh);
	if( prev ) SetCompareFunc(prev);
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
void CPxArray<TYPE,ARG_TYPE>::InsertionSort(int nLow, int nHigh)
{
	int i,j;
	for(i = nLow + 1; i <= nHigh; i++ ) {
		TYPE tValue = DATA_ACCESS_OPERATOR(i);

		for(j = i;
			j > nLow && Compare(&tValue, &DATA_ACCESS_OPERATOR(j-1)) < 0;
			j--)
		{
			DATA_ACCESS_OPERATOR(j) = DATA_ACCESS_OPERATOR(j-1);
		}
		DATA_ACCESS_OPERATOR(j) = tValue;
	}
}

// Quicksort: sort first N items in array A
// TYPE: must have copy constructor, operator =, and operator <
template<class TYPE, class ARG_TYPE> AFX_INLINE
void CPxArray<TYPE,ARG_TYPE>::QuickSort(int nLow, int nHigh)
{
	if(nLow + m_nCutOff > nHigh ) {
		InsertionSort(nLow, nHigh);
	}
	else {
		//Sort Low, Middle, High
		int nMid = (nLow + nHigh) / 2; 	// middle value to partition about

		if( Compare(&DATA_ACCESS_OPERATOR(nMid), &DATA_ACCESS_OPERATOR(nLow)) < 0 ) {
			Swap(DATA_ACCESS_OPERATOR(nLow), DATA_ACCESS_OPERATOR(nMid));
		}
		if( Compare(&DATA_ACCESS_OPERATOR(nHigh), &DATA_ACCESS_OPERATOR(nLow)) < 0 ) {
			Swap(DATA_ACCESS_OPERATOR(nLow), DATA_ACCESS_OPERATOR(nHigh));
		}
		if( Compare(&DATA_ACCESS_OPERATOR(nHigh), &DATA_ACCESS_OPERATOR(nMid)) < 0 ) {
			Swap(DATA_ACCESS_OPERATOR(nMid), DATA_ACCESS_OPERATOR(nHigh));
		}

		// Place pivot at Position High-1
		TYPE tPivot = DATA_ACCESS_OPERATOR(nMid);
		Swap(DATA_ACCESS_OPERATOR(nMid), DATA_ACCESS_OPERATOR(nHigh - 1));

		// Begin partitioning
		int i, j;
		for (i = nLow, j = nHigh - 1; ; ) {
			// find first element too big to be in low partition
			while( Compare(&DATA_ACCESS_OPERATOR(++i), &tPivot) < 0 ) // Symantec may require { }
				;                      // instead of ;

			// find first element too small to be in high partition
			while( Compare(&tPivot, &DATA_ACCESS_OPERATOR(--j)) < 0 ) // for both these
				;                      // loops. It should not.
			if( i < j ) {
				Swap(DATA_ACCESS_OPERATOR(i), DATA_ACCESS_OPERATOR(j));
			} else {
				break;
			}
		}

		// Restore pivot
		Swap(DATA_ACCESS_OPERATOR(i), DATA_ACCESS_OPERATOR(nHigh - 1));

		QuickSort(nLow, i - 1);    // Sort small elements
		QuickSort(i + 1, nHigh);   // Sort large elements
	}

}

//==============================================================================
// Implementation of the CPxSortedArray class.
template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE> ::CPxSortedArray ()
{
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE> ::CPxSortedArray (const CPxSortedArray &x)
{
	*this = x;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE> ::~CPxSortedArray ()
{}

template<class TYPE, class ARG_TYPE> AFX_INLINE
int CPxSortedArray<TYPE,ARG_TYPE>::Search(ARG_TYPE tValue) const
{	// binary search if fail return nearest item
	int nLow = 0, nHigh = GetSize(), nMid;

	while(nLow < nHigh) {
		nMid = (nLow + nHigh ) / 2;
		if( Compare(&DATA_ACCESS_OPERATOR(nMid), &tValue) < 0 ) {
			nLow = nMid + 1;
		} else {
			nHigh = nMid;
		}
	}
	return nLow;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
int CPxSortedArray<TYPE,ARG_TYPE>::Lookup(ARG_TYPE tValue) const
{	// binary lookup if fail return -1
	int nRet = Search(tValue);

	if( nRet < GetSize() ) {
		if( Compare(&DATA_ACCESS_OPERATOR(nRet), &tValue) == 0 )
		{
			return nRet;
		}
	}

	return -1;
}

template<class TYPE, class ARG_TYPE>
AFX_INLINE int
CPxSortedArray<TYPE,ARG_TYPE> ::AddSorted(ARG_TYPE tValue)
{
	int nInsertIndex = Search(tValue);

	if (nInsertIndex < GetSize()) {
		InsertAt(nInsertIndex, tValue);
	} else {
		nInsertIndex = Add(tValue);
	}

	return nInsertIndex;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
void CPxSortedArray<TYPE,ARG_TYPE> ::SetAt(ARG_TYPE tValue)
{
	AddSorted(tValue);
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE>& CPxSortedArray<TYPE,ARG_TYPE>::operator = (const CPxSortedArray &x)
{
	CPxArray<TYPE,ARG_TYPE>::operator = (x);
	return *this;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE>& CPxSortedArray<TYPE,ARG_TYPE>::operator = (const CArray<TYPE,ARG_TYPE> &x)
{
	CPxArray<TYPE,ARG_TYPE>::operator = (x); Sort();
	return *this;
}

template<class TYPE, class ARG_TYPE>AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE>& CPxSortedArray<TYPE,ARG_TYPE>::operator=(const CPxArray<TYPE,ARG_TYPE> &x)
{
	CPxArray<TYPE,ARG_TYPE>::operator = (x);
	Sort();
	return *this;
}

template<class TYPE, class ARG_TYPE> AFX_INLINE
CPxSortedArray<TYPE,ARG_TYPE>::operator CPxArray<TYPE, ARG_TYPE>()
{
	return *this;
}

#endif //__PXTEMPL_H_INCLUDED__
