/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdafx.h"
#include "SubClass.h"
#include "PxTTM.h"
#include "PxMisc.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPxTooltipManager::CPxTooltipManager():
	m_iWidth(-1), m_bUseHelp(FALSE)
{

}

CPxTooltipManager::~CPxTooltipManager()
{

}

void CPxTooltipManager::Install(CWnd *pwndParent)
{
	if( pwndParent ) pwndParent->EnableToolTips(TRUE);
	HookWindow(pwndParent);
}

LRESULT CPxTooltipManager::WindowProc(UINT msg, WPARAM wp, LPARAM lp)
{
	if( msg == WM_NOTIFY ) {
		LRESULT lResult;
		switch( ((NMHDR*)lp)->code ) {
		case TTN_NEEDTEXTA:
		case TTN_NEEDTEXTW:
			if( OnToolTipText(wp, (NMHDR*)lp, &lResult) )
				return lResult;
			break;
		case TTN_SHOW:
			if( OnToolTipShow(wp, (NMHDR*)lp, &lResult) )
				return lResult;
			break;
		}
	}
	return CSubclassWnd::WindowProc(msg, wp, lp);
}

BOOL CPxTooltipManager::OnToolTipText(UINT, NMHDR* pNMHDR, LRESULT *pResult)
{
	if( AfxGetThreadState()->m_pToolTip &&
		AfxGetThreadState()->m_pToolTip->GetMaxTipWidth() != m_iWidth)
	{
		AfxGetThreadState()->m_pToolTip->SetMaxTipWidth(m_iWidth);
	}
	
	TOOLTIPTEXTA* pTTTA = (TOOLTIPTEXTA*)pNMHDR;
	TOOLTIPTEXTW* pTTTW = (TOOLTIPTEXTW*)pNMHDR;

	UINT nCtrlID = pNMHDR->idFrom;
	if (pNMHDR->code == TTN_NEEDTEXTA && (pTTTA->uFlags & TTF_IDISHWND) ||
		pNMHDR->code == TTN_NEEDTEXTW && (pTTTW->uFlags & TTF_IDISHWND))
	{
		nCtrlID = ::GetDlgCtrlID((HWND)nCtrlID);
	}

	int nResult = 0;
	#ifndef _UNICODE
		if( pNMHDR->code == TTN_NEEDTEXTA ) {
			nResult = ::LoadStringA(AfxGetInstanceHandle(),nCtrlID,m_szTooltipText,countof(m_szTooltipText));
			if( nResult ) {
				pTTTA->hinst = NULL;
				pTTTA->lpszText = m_szTooltipText;
			}
		} else {
			nResult = ::LoadStringW(AfxGetInstanceHandle(),nCtrlID,m_wszTooltipText,countof(m_wszTooltipText));
			if( nResult ) {
				pTTTW->hinst = NULL;
				pTTTW->lpszText = m_wszTooltipText;
			}
		}
		if( nResult == 0 && m_bUseHelp ) {
		//  we need to provide fake tooltip if "HelpTips" are used.
			HWND hWnd = ::GetDlgItem(m_pWndHooked->GetSafeHwnd(), nCtrlID);
			if( hWnd && ::GetWindowContextHelpId(hWnd) ) {
				if (pNMHDR->code == TTN_NEEDTEXTA)
				lstrcpyn(pTTTA->szText, "???", countof(pTTTA->szText)); else
				_mbstowcsz(pTTTW->szText, "???", countof(pTTTW->szText));			}
			}
	#else
		#error not implemented
	#endif
	return FALSE;
}

BOOL CPxTooltipManager::OnToolTipShow(UINT, NMHDR* pNMHDR, LRESULT* result)
{
	if( !m_bUseHelp ) return FALSE;
//	provide pop-up help topic for the control instead of
//	instead of tooltip that is going to be shown
	INIT2<TOOLINFO> tti;
	if( ::SendMessage(pNMHDR->hwndFrom,TTM_GETCURRENTTOOL,0,(LPARAM)&tti) )
	{
		HWND hWnd;

		if( tti.uFlags & TTF_IDISHWND ) hWnd = (HWND)tti.uId;
		else hWnd = ::GetDlgItem(m_pWndHooked->GetSafeHwnd(), tti.uId);
		if( !hWnd ) return FALSE;

		DWORD nHelpId;
		while( !(nHelpId = ::GetWindowContextHelpId(hWnd)) ) {
			hWnd = ::GetParent(hWnd) ;
			if( !hWnd ) return FALSE;
		}

		::SetWindowPos(pNMHDR->hwndFrom,0,0,0,0,0,SWP_HIDEWINDOW);
		AfxGetApp()->WinHelp(::GetWindowContextHelpId(hWnd), HELP_CONTEXTPOPUP);
		*result = TRUE;
		return TRUE;
	}
	return FALSE;
}

void CPxTooltipManager::SetTooltipWidth(int iWidth)
{
	m_iWidth = iWidth;
}

void CPxTooltipManager::SetHelpTip(BOOL bOn)
{
	m_bUseHelp = bOn;
}
