/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "GLOBALS.H"
#include "wcat.h"
#include "PPMisc.h"
#include "Password.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void CPPMisc::CDesktopColor::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) 
{
    ASSERT(lpDrawItemStruct);
    
    CRect   rect    = lpDrawItemStruct->rcItem;
    CDC*    pDC     = CDC::FromHandle(lpDrawItemStruct->hDC);
    UINT    state   = lpDrawItemStruct->itemState;
    CString m_strText;

    CSize Margins(::GetSystemMetrics(SM_CXEDGE),
		::GetSystemMetrics(SM_CYEDGE));

    // Draw arrow
    if (m_bActive) state |= ODS_SELECTED;
    pDC->DrawEdge(rect, EDGE_SUNKEN, BF_RECT);
    pDC->DrawFrameControl(&m_ArrowRect, DFC_SCROLL,
		((state & ODS_SELECTED) ? DFCS_PUSHED : 0) |
        ((state & ODS_DISABLED) ? DFCS_INACTIVE : 0) |
		DFCS_SCROLLDOWN);

    // Must reduce the size of the "client" area of the button due to edge thickness.
    rect.DeflateRect(Margins.cx, Margins.cy);
    rect.right -= m_ArrowRect.Width();
	if( (state & ODS_DISABLED) == 0 ) {
    //  Fill remaining area with Color
	/*  this code does not work on MustDie`98
		CMemDC mDC(pDC, rect);
		::PaintDesktop( mDC.GetSafeHdc() ); */
	// more portable version with one small flaw -
	// it draws part of desktop that sits right under
	// control being painted
		CRect rcDesk(rect);
		CRgn rgn; rgn.CreateRectRgnIndirect(rect);
		pDC->SelectClipRgn(&rgn);
		::PaintDesktop( pDC->GetSafeHdc() );
		pDC->SelectClipRgn(NULL);
	} else {
		::FillRect(lpDrawItemStruct->hDC, rect, (HBRUSH)(COLOR_3DFACE+1));
	}

    // Draw the window text (if any)
	GetWindowText(m_strText);
    if (m_strText.GetLength())
    {
		CRect rcText(0,0,rect.Width(),rect.Height());
		pDC->DrawText(m_strText, rcText, DT_CENTER|DT_VCENTER|DT_CALCRECT);
		rcText.OffsetRect(
			rect.left+(rect.Width()-rcText.Width())/2-1,
			rect.top+(rect.Height()-rcText.Height())/2-1);
        pDC->SetBkMode(TRANSPARENT);

        if (state & ODS_DISABLED) {
            rcText.OffsetRect(1,1);
            pDC->SetTextColor(::GetSysColor(COLOR_3DHILIGHT));
            pDC->DrawText(m_strText, rcText, DT_CENTER|DT_VCENTER);
            rcText.OffsetRect(-1,-1);
            pDC->SetTextColor(::GetSysColor(COLOR_3DSHADOW));
            pDC->DrawText(m_strText, rcText, DT_CENTER|DT_VCENTER);
        } else {
            pDC->SetTextColor((m_crColorText == CLR_DEFAULT)? 0 : m_crColorText);
            pDC->DrawText(m_strText, rcText, DT_CENTER|DT_VCENTER);
        }
    }

    // Draw focus rect
    if (state & ODS_FOCUS) 
    {
        rect.DeflateRect(1,1);
        pDC->DrawFocusRect(rect);
    }
}


/////////////////////////////////////////////////////////////////////////////
// CPPMisc property page
CPPMisc::CPPMisc() : CPropertyPage(CPPMisc::IDD)
{
	//{{AFX_DATA_INIT(CPPMisc)
	m_strPassword = _T("");
	m_strPassword2 = _T("");
	m_bFixDskText = FALSE;
	m_bRegisterService = FALSE;
	m_bPasswordMenu = FALSE;
	m_nPasswordTimeout = 0;
	m_bEjectCDAtExit = FALSE;
	//}}AFX_DATA_INIT
	m_bPasswordChanged = FALSE;
}	

void CPPMisc::SetData(CMainWnd* pWnd)
{
	m_bFixDskText      = pWnd->m_bFixDskText;
	m_crDskText        = pWnd->m_crDskText;
	m_bPasswordMenu    = pWnd->m_bPasswordMenu;
	if( readPasswordHash() ) m_strPassword = _T("          ");
	m_nPasswordTimeout = CPassword::m_nPasswordTimeout;
	m_bRegisterService = theApp.GetRegisterService();
	m_lsDisabledMenus  = pWnd->m_lsDisabledMenus;
	m_bEjectCDAtExit   = pWnd->m_bEjectCDAtExit;
}

void CPPMisc::GetData(CMainWnd* pWnd)
{
	pWnd->m_crDskText = m_crDskText;
	pWnd->m_lsDisabledMenus = m_lsDisabledMenus;
	if( pWnd->m_bFixDskText = m_bFixDskText)
		pWnd->OnFixDskText(0,0);
	else if( CWndData& hDeskList = CWndData::GetDesktopListCtrl() ) {
		ListView_SetTextBkColor(hDeskList, ::GetSysColor(COLOR_DESKTOP));
		ListView_SetTextColor(hDeskList, RGB(255,255,255));
		::InvalidateRect(hDeskList, NULL, TRUE);
	}
	if( m_bPasswordChanged ) {
		if( m_strPassword.IsEmpty() ) {
			HKEY hKey = theApp.GetSectionKey(_T(""));
			::RegDeleteValue(hKey, _T("Password"));
			RegCloseKey(hKey);
		} else 
			savePasswordHash(m_strPassword);
	}
	pWnd->m_bPasswordMenu = m_bPasswordMenu;
	pWnd->m_bEjectCDAtExit = m_bEjectCDAtExit;
	CPassword::m_nPasswordTimeout = m_nPasswordTimeout;
	theApp.SetRegisterService(m_bRegisterService);
}

void CPPMisc::DoDataExchange(CDataExchange* pDX)
{
	int i,j;
	CPropertyPage::DoDataExchange(pDX);
	m_strPassword2 = m_strPassword;
	//{{AFX_DATA_MAP(CPPMisc)
	DDX_Control(pDX, IDC_MISC_MENULIST, m_lbMenuList);
	DDX_Text(pDX, IDC_MISC_PASSWORD, m_strPassword);
	DDX_Text(pDX, IDC_MISC_PASSWORD2, m_strPassword2);
	DDX_Check(pDX, IDC_MISC_DESKTOP, m_bFixDskText);
	DDX_Check(pDX, IDC_MISC_REGSERVICE, m_bRegisterService);
	DDX_Check(pDX, IDC_MISC_PASSWORDMENU, m_bPasswordMenu);
	DDX_Text(pDX, IDC_MISC_PASSWD_TIMEOUT, m_nPasswordTimeout);
	DDV_MinMaxInt(pDX, m_nPasswordTimeout, 0, 300);
	DDX_Check(pDX, IDC_MISC_EJECTCD, m_bEjectCDAtExit);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_MISC_DESKTOPTEXT, m_txtColor);
	DDX_PxColorPicker(pDX, IDC_MISC_DESKTOPTEXT, m_crDskText);
	DDX_SpinMinMax(pDX, IDC_MISC_PASSWORD_TIMEOUT_SPIN, i=0, j=300);

	if( !pDX->m_bSaveAndValidate ) {
		while( m_lbMenuList.GetCount() )
			m_lbMenuList.DeleteString(0);

		CMenu2 theMenu;
		MENUITEMINFO mii;
		TCHAR szMenuName[0x100];
		theMenu.LoadMenu(IDR_MAINFRAME);
		for(int i=0; i < (int)theMenu.GetMenuItemCount(); i++) {
			mii.cbSize = sizeof(mii);
			mii.fMask  = MIIM_ID|MIIM_DATA|MIIM_TYPE;
			mii.dwTypeData = szMenuName;
			mii.cch	   = countof(szMenuName);
			theMenu.GetMenuItemInfo(i, &mii, TRUE);
			if( mii.fType & MFT_SEPARATOR ) 
				continue;
			if( mii.wID == ID_TIMEROP )
				break;
			m_lbMenuList.AddString(szMenuName);
			m_lbMenuList.SetItemData(m_lbMenuList.GetCount()-1,mii.wID);
			m_lbMenuList.SetCheck(m_lbMenuList.GetCount()-1, m_lsDisabledMenus.Find(mii.wID) == NULL);
		}
		theMenu.DestroyMenu();

		theMenu.LoadMenu(IDR_SHELL);
		for(i=0; i < (int)theMenu.GetMenuItemCount(); i++) {
			mii.cbSize = sizeof(mii);
			mii.fMask  = MIIM_ID|MIIM_DATA|MIIM_TYPE;
			mii.dwTypeData = szMenuName;
			mii.cch	= countof(szMenuName);
			theMenu.GetMenuItemInfo(i, &mii, TRUE);
			if( mii.fType & MFT_SEPARATOR ) 
				continue;
			m_lbMenuList.AddString(szMenuName);
			m_lbMenuList.SetItemData(m_lbMenuList.GetCount()-1,mii.wID);
			m_lbMenuList.SetCheck(m_lbMenuList.GetCount()-1, m_lsDisabledMenus.Find(mii.wID) == NULL);
		}
	} else {
		m_lsDisabledMenus.RemoveAll();
		for(int i=m_lbMenuList.GetCount()-1;i>=0;i--)
			if( m_lbMenuList.GetCheck(i) ==  0)
				m_lsDisabledMenus.AddTail((WORD)m_lbMenuList.GetItemData(i));
	}

	if( pDX->m_bSaveAndValidate ) {
		if( m_bPasswordChanged && m_strPassword != m_strPassword2 ) {
			AfxMessageBox(IDS_ERR_PASSWORD_VERIFICATION);
			GetDlgItem(IDC_MISC_PASSWORD)->SetFocus();
			AfxThrowUserException();
		}
	}
	OnDesktopChange();
}


BEGIN_MESSAGE_MAP(CPPMisc, CPropertyPage)
	//{{AFX_MSG_MAP(CPPMisc)
	ON_BN_CLICKED(IDC_MISC_DESKTOP, OnDesktopChange)
	ON_EN_CHANGE(IDC_MISC_PASSWORD, OnPasswordChange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPPMisc message handlers

BOOL CPPMisc::OnInitDialog() 
{
	m_txtColor.SetSelectionMode(CP_MODE_TEXT);
	m_txtColor.SetBkColor(::GetSysColor(COLOR_DESKTOP));
	m_txtColor.SetDefaultText(NULL);
	//m_txtColor.SubclassDlgItem(IDC_MISC_DESKTOPTEXT, this);
	
	CPropertyPage::OnInitDialog();
	ttm.Install(this);
	ttm.SetTooltipWidth(200);

	GetDlgItem(IDC_MISC_REGSERVICE)->EnableWindow(::IsWin95());
	return TRUE; 
}

void CPPMisc::OnDesktopChange() 
{
	if( GetDlgItem(IDC_MISC_DESKTOP) &&
		((CButton*)GetDlgItem(IDC_MISC_DESKTOP))->GetCheck() == 1)
	{
		m_txtColor.EnableWindow(TRUE);
	} else {
		m_txtColor.EnableWindow(FALSE);
	}
}

void CPPMisc::OnPasswordChange() 
{
	m_bPasswordChanged = TRUE;
}
