/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "wcat.h"
#include "Password.h"
#include "md5.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPassword dialog


CPassword::CPassword(CWnd* pParent /*=NULL*/)
	: CDialog(CPassword::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPassword)
	m_strPassword = _T("");
	//}}AFX_DATA_INIT
}


void CPassword::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPassword)
	DDX_Text(pDX, IDC_EDIT1, m_strPassword);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPassword, CDialog)
	//{{AFX_MSG_MAP(CPassword)
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPassword message handlers

BYTE* computeHash(LPCTSTR strIn, BYTE hash[16])
{
	MD5_CTX context;

	MD5Init(&context);
	MD5Update(&context, (BYTE*)strIn, strlen(strIn)*sizeof(TCHAR));
	MD5Final(hash, &context);
	return hash;
}

BYTE* readPasswordHash()
{
	static BYTE pHash[16];

	BYTE* pSavedHash;
	UINT bHashSize;

	if( !AfxGetApp()->GetProfileBinary(_T(""), _T("Password"), &pSavedHash, &bHashSize) )
	//	no password in registry
		return NULL;
	if( bHashSize != sizeof(pHash) ) {
	//	pasword hash has invalid size
		delete [] pSavedHash;
		return NULL;
	}

	memcpy(pHash, pSavedHash, sizeof(pHash));
	delete [] pSavedHash;
	return pHash;
}

void savePasswordHash(LPCTSTR strIn)
{
	BYTE pHash[16];
	computeHash(strIn, pHash);
	AfxGetApp()->WriteProfileBinary(_T(""), _T("Password"), pHash, sizeof(pHash));
}

time_t CPassword::m_LastPasswordTime=0;
INT CPassword::m_nPasswordTimeout=0;
INT CPassword::AskPassword(int nTries)
{
	BYTE *pOriginalHash = readPasswordHash();
	if( !pOriginalHash )
		return IDOK; //no password

	if( m_nPasswordTimeout )
	{
		time_t cur_time;
		if( (m_LastPasswordTime + m_nPasswordTimeout ) > time(&cur_time) )
		{// password timeout was not expired - update
		 // last password verification time
			m_LastPasswordTime = cur_time;
			return IDOK;
		}
	}

	CPassword dlg(AfxGetMainWnd());
	INT nResult = IDCANCEL;
	while( nTries-- ) {
		if( dlg.DoModal() == IDOK ) {
			if( dlg.m_strPassword.IsEmpty() )
			{
				WARNSOUND();
				continue;
			}

			BYTE pHash[16];
			computeHash(dlg.m_strPassword, pHash);
			if( memcmp(pHash, pOriginalHash, sizeof(pHash)) == 0 ) {
				time(&m_LastPasswordTime);
				nResult = IDOK;
				break;
			}
			WARNSOUND();
		} else {
		//  password dialog was canceled
			nResult = IDABORT;
			break;
		}
	}
	return nResult;
}
