#include "stdafx.h"
#include "globals.h"
#include "rule.h"
#include "enumproc.h"
#include "wndlist.h"
#include "wcat.h"

/*private*/ CWndData::ProcessNameMap CWndData::m_mProcessNameCache; 
/*private*/ const DWORD CWndData::RequiredStyle = (WS_OVERLAPPED);
/*private*/ const DWORD CWndData::DisabledStyle = (WS_CHILDWINDOW/*|WS_DISABLED*/);
/*public*/  LPCTSTR CWndData::propHidden		= _T("WatchCat-hidden");
/*public*/  LPCTSTR CWndData::propTray			= _T("WatchCat-has-trayicon");
/*public*/  LPCTSTR CWndData::propAutoMinimize	= _T("WatchCat-autominimize");
/*public*/  LPCTSTR CWndData::propAutoHide	    = _T("WatchCat-autohide");
/*public*/  LPCTSTR CWndData::propAutoClose		= _T("WatchCat-autoclose");

/*static*/
LRESULT CWndData::SendMainWndMessage(UINT message, WPARAM wParam/*=0*/, LPARAM lParam/*=0*/)
{
	CWnd* pWnd = AfxGetMainWnd();
	if( pWnd ) return pWnd->SendMessage(message, wParam, lParam);
	return NULL;
}

CWndData& CWndData::operator=(HWND hWnd) 
{
	m_hWnd = hWnd;
	m_sClassName.Empty();
	m_sExeName.Empty();
	Refresh();
	return *this;
}

CWndData& CWndData::operator=(const CWndData& x)
{
	m_pRuleSet = x.m_pRuleSet;
	return this->operator=(x.m_hWnd);
}

void CWndData::RefreshProcessCache()
{
	DWORD	dwPid;
	CString sProcessName;
	POSITION pos = m_mProcessNameCache.GetStartPosition();
	while( pos ) {
		m_mProcessNameCache.GetNextAssoc(pos, dwPid, sProcessName);
		HANDLE hProcess = ::OpenProcess(PROCESS_QUERY_INFORMATION,FALSE,dwPid);
		if( hProcess ) {
			::CloseHandle(hProcess);
			continue;
		}
		m_mProcessNameCache.RemoveKey(dwPid);
	}
}

LPCTSTR CWndData::GetWindowName() const
{
	if( m_sWindowName.IsEmpty() && ::IsWindow(m_hWnd) )
	{
		int nLen = ::GetWindowTextLength(m_hWnd)+1;
		if( !nLen ) return m_sWindowName;
		::GetWindowText(m_hWnd,	m_sWindowName.GetBuffer(nLen), nLen);
		m_sWindowName.ReleaseBuffer();
	}
	return m_sWindowName;
}

DWORD CWndData::GetHotKey() const
{
	if( !m_dwHotKey && ::IsWindow(m_hWnd) )
	{
		::SendMessageTimeout(m_hWnd,WM_GETHOTKEY,0,0,
			SMTO_ABORTIFHUNG|SMTO_BLOCK, 50, &m_dwHotKey);
	}
	return m_dwHotKey;
}

LPCTSTR CWndData::GetClassName() const
{
	if( m_sClassName.IsEmpty() && ::IsWindow(m_hWnd) )
	{
		::GetClassName(m_hWnd,	m_sClassName.GetBuffer(0x100), 0x100);
		m_sClassName.ReleaseBuffer();
	}
	return m_sClassName;
}

BOOL CALLBACK CWndData::ProcessNameCallback(
	DWORD	nProcessId,		
	WORD,      
	LPSTR	pszProcessName, 
	LPARAM	lParam)
{
	m_mProcessNameCache[nProcessId] = pszProcessName;
	return TRUE;
}

LPCTSTR CWndData::GetExeName() const
{
	if( m_sExeName.IsEmpty() && ::IsWindow(m_hWnd) )
	{
		DWORD nProcessId;
		::GetWindowThreadProcessId(m_hWnd, &nProcessId);
		if( !m_mProcessNameCache.Lookup(nProcessId, m_sExeName) )
		{
			TRACE("<WCAT> CWndData::GetExeName(): process enumeration engaged\n");
			::EnumProcesses(CWndData::ProcessNameCallback, NULL);
		}
		m_mProcessNameCache.Lookup(nProcessId, m_sExeName);
	}
	return m_sExeName;
}

HICON CWndData::GetIcon() const
{
	if( m_hIcon == NULL && ::IsWindow(m_hWnd) )
	{
		HICON hIcon = NULL;
		if( ::SendMessageTimeout(m_hWnd,WM_GETICON,ICON_SMALL,0,
			SMTO_ABORTIFHUNG|SMTO_BLOCK, 50, (LPDWORD)&hIcon) && 
			hIcon)
			return hIcon;
		if( ::SendMessageTimeout(m_hWnd,WM_GETICON,ICON_BIG,0,
			SMTO_ABORTIFHUNG|SMTO_BLOCK, 50, (LPDWORD)&hIcon) &&
			hIcon)
			return hIcon;
		if( !hIcon ) hIcon = (HICON)GetClassLong(m_hWnd, GCL_HICONSM);
		if( !hIcon ) hIcon = (HICON)GetClassLong(m_hWnd, GCL_HICON);
		if( !hIcon ) hIcon = ::LoadIcon(NULL, IDI_APPLICATION);
		return hIcon;
	}
	return m_hIcon;
}

/*static*/ BOOL CWndData::IsVisible(HWND hWnd)
{
	return (::GetWindowLong(hWnd, GWL_STYLE)&WS_VISIBLE) != 0;
}

BOOL CWndData::IsVisible() const
{
	return IsVisible(m_hWnd);
}

/*static*/ BOOL CWndData::IsChanged(HWND hWnd)
{
	return CWndData(hWnd).property[propHidden] != NULL;
}

BOOL CWndData::IsChanged() const
{
	return property[propHidden] != NULL;
}

BOOL CWndData::IsHideable() const
{
	if( !::IsWindow(m_hWnd) )
		return FALSE;

//	always report "changed" windows as hideables 
	if( IsChanged() )
		return TRUE;

	DWORD dwStyle   = ::GetWindowLong(m_hWnd, GWL_STYLE);
	DWORD dwExStyle = ::GetWindowLong(m_hWnd, GWL_EXSTYLE);
//	Window should look like application window	
	if( (dwStyle & RequiredStyle) != RequiredStyle ) 
		return FALSE;
//	Window should not have be "forbidden" style
	if( dwStyle & DisabledStyle ) 
		return FALSE;
//	It should not be a "Tool Window"
	if( dwExStyle & WS_EX_TOOLWINDOW ) 
		return FALSE;

//	Window should not have owner - it concerns
	if( (MAINWND && MAINWND->m_bTopOnly) &&
		::GetWindow(m_hWnd, GW_OWNER) )
		return FALSE;

//	Window initially must be visible or,
//	if it's not visible, hidden by us
	if( !IsVisible() )
		return FALSE;

	if( m_pRuleSet && m_pRuleSet->IsIgnore(*this) )
		return FALSE;
	
//	prevent hiding own windows
	DWORD dwPid;
	if( ::GetWindowThreadProcessId(m_hWnd, &dwPid) &&
		dwPid == ::GetCurrentProcessId() )
	{
		return FALSE;
	}

	return TRUE;
}

BOOL CWndData::IsTrayable() const
{
	return m_pRuleSet && m_pRuleSet->IsTray(*this);
}


/*static*/ BOOL CWndData::IsInActiveThread(HWND hWnd)
{
	return ::GetWindowThreadProcessId(hWnd, NULL) ==
		   ::GetWindowThreadProcessId(::GetForegroundWindow(), NULL);
}

BOOL CWndData::IsInActiveThread() const
{
	return IsInActiveThread(m_hWnd);
}

void CWndData::Refresh() const
{
	m_sWindowName.Empty();
	m_hIcon = NULL;
	m_dwHotKey = NULL;
}

void CWndData::Show(BOOL bShow, BOOL bTray/*=-1*/) 
{
	bShow = bShow?1:0;
	if( IsVisible() != bShow ) {
		if( bShow )	{
			if( (m_pRuleSet && m_pRuleSet->IsPassword(*this)) &&
				!VerifyPassword() )
			{
				return;
			}
			property[propHidden] = NULL;
			property[propTray]   = NULL;
		} else {
			property[propHidden] = TRUE;
			SendMainWndMessage(WM_SETJUSTHIDDEN, NULL, (LPARAM)m_hWnd);
		}
		::ShowWindow(m_hWnd, bShow?SW_SHOW:SW_HIDE);
		::ShowOwnedPopups(m_hWnd, bShow);

		if( bShow && (::GetWindowLong(m_hWnd, GWL_STYLE)&WS_MINIMIZE) )
		//	if window was shown and if it was minimized, restore it
			::ShowWindow(m_hWnd,
				(::GetWindowLong(m_hWnd, GWL_STYLE)&WS_MAXIMIZE)?
					SW_MAXIMIZE:SW_RESTORE);

		if( !bShow ) {
			bTray = bTray==-1?IsTrayable():bTray;
			if( bTray ) CreateTrayIcon();
		}
		::PlaySound(NULL,NULL,NULL);
		::PlaySound(bShow?_T("RestoreUp"):_T("Minimize"), NULL, SND_ALIAS|SND_ASYNC|SND_NODEFAULT);
	}
	if( !bShow ) Compact();
}

/*stati*/ void CWndData::Compact(HWND hWnd)
{
	if( !SendMainWndMessage(WM_GETCONFIG, NULL, (LPARAM)"Compact") )
		return;

	HANDLE hProcess;
	DWORD hProcessID;
	if( !GetWindowThreadProcessId(hWnd, &hProcessID) ) return;
	hProcess = OpenProcess(PROCESS_ALL_ACCESS, FALSE, hProcessID);
	if( hProcess ) {
		HINSTANCE hLib = GetModuleHandle(_T("KERNEL32.DLL"));
		if( hLib ) {
			BOOL (FAR WINAPI *spwss)(HANDLE,DWORD,DWORD) = 
				(BOOL (FAR WINAPI *)(HANDLE,DWORD,DWORD))GetProcAddress(hLib,"SetProcessWorkingSetSize");
			if( spwss )	spwss(hProcess, 0xffffffff, 0xffffffff);
		}
	}
	CloseHandle(hProcess);
}

void CWndData::Compact()
{
	Compact(m_hWnd);
}

BOOL CWndData::VerifyPassword()
{
	return SendMainWndMessage(WM_VERIFYPASSWORD,NULL,(LPARAM)this);
}

void CWndData::CreateTrayIcon()
{
	SendMainWndMessage(WM_CREATETRAYICON,NULL,(LPARAM)this);
}

CWndData& CWndData::GetDesktopWnd()
{
	static CWndData wndData;
	if( !wndData.m_hWnd || !::IsWindow(wndData.m_hWnd) )
	{
		wndData = ::FindWindow(_T("Progman"),_T("Program Manager")); 
	}
	return wndData;
}

CWndData& CWndData::GetDesktopListCtrl()
{
	static CWndData wndData;
	if( !wndData.m_hWnd || !::IsWindow(wndData.m_hWnd) )
	while(1){
		wndData = (HWND)NULL;
		HWND hDesktop = CWndData::GetDesktopWnd();
		if( !hDesktop ) break;
		HWND hShellDefV = ::FindWindowEx(hDesktop, NULL, "SHELLDLL_DefView", NULL);
		if( !hShellDefV ) break;
		wndData = ::FindWindowEx(hShellDefV, NULL, "SysListView32", NULL);
		break;
	}
	return wndData;
}

CWndData& CWndData::GetTaskbarWnd()
{
	static CWndData wndData;
	if( !wndData.m_hWnd || !::IsWindow(wndData.m_hWnd) )
	{
		wndData = ::FindWindow(_T("Shell_TrayWnd"),_T(""));
	}
	return wndData;
}

void CWndData::AutoMinimize()
{
	const CRule *pRule;
	DWORD dwTimeout;
do {
	if( !IsVisible() || !m_pRuleSet )
		break;
	if( m_pRuleSet->_FindRule(*this, CRule::rsfMinimizeWindow(2)) )
		break; //check for "never" rule
	BOOL bIsActive = IsInActiveThread();
	if( bIsActive && (pRule = m_pRuleSet->_FindRule(*this, CRule::rsfMinimizeWindow(1), TRUE)) )
		goto DOIT;
	if( bIsActive )
		break;
	if( pRule = m_pRuleSet->_FindRule(*this, CRule::rsfMinimizeWindow(1)) )
	{
DOIT:
		if( !(dwTimeout = property[propAutoMinimize]) )
			dwTimeout = pRule->minimize.wTimeout?SEC2TIMESLICE(pRule->minimize.wTimeout):1;
		if( dwTimeout <= 1 )
			CloseWindow(m_hWnd);
		else {
			property[propAutoMinimize] = --dwTimeout;
			return;
		}
	}
} while(0);
	property[propAutoMinimize] = NULL;
}

void CWndData::AutoHide()
{
	const CRule *pRule;
	DWORD dwTimeout;
do {	
	if( !IsVisible() || !m_pRuleSet )
		break;
	if( m_pRuleSet->_FindRule(*this, CRule::rsfHideWindow(2)) )
		break; //check for "never" rule
	BOOL bIsActive = IsInActiveThread();
	if( bIsActive && (pRule = m_pRuleSet->_FindRule(*this, CRule::rsfHideWindow(1), TRUE)) )
		goto DOIT;
	if( bIsActive )
		break;
	if( pRule = m_pRuleSet->_FindRule(*this, CRule::rsfHideWindow(1)) )
	{
DOIT:
		if( !(dwTimeout = property[propAutoHide]) )
			dwTimeout = pRule->hide.wTimeout?SEC2TIMESLICE(pRule->hide.wTimeout):1;
		if( dwTimeout <= 1 )
			Show(FALSE);
		else {
			property[propAutoHide] = --dwTimeout;
			return;
		}
	}
} while(0);
	property[propAutoHide] = NULL;
}

void CWndData::AutoClose()
{
	const CRule *pRule;
	DWORD dwTimeout;
do {
	if( !IsVisible() || !m_pRuleSet )
		break;
	if( m_pRuleSet->_FindRule(*this, CRule::rsfCloseWindow(2)) )
		break; //check for "never" rule
	BOOL bIsActive = IsInActiveThread();
	if( bIsActive && (pRule = m_pRuleSet->_FindRule(*this, CRule::rsfCloseWindow(1), TRUE)) )
		goto DOIT;
	if( bIsActive )
		break;
	if( pRule = m_pRuleSet->_FindRule(*this, CRule::rsfCloseWindow(1)) )
	{
DOIT:
		if( !(dwTimeout = property[propAutoClose]) )
			dwTimeout = pRule->close.wTimeout?SEC2TIMESLICE(pRule->close.wTimeout):1;
		if( dwTimeout <= 1 )
			::PostMessage(m_hWnd, WM_CLOSE, NULL, NULL);
		else {
			property[propAutoClose] = --dwTimeout;
			return;
		}
	}
} while(0);
	property[propAutoClose] = NULL;
}

/////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK
CWndList::RefreshCallback(HWND hWnd, LPARAM lParam)
{
	CWndList& rList = *(CWndList*)lParam;
	CWndData wndData(hWnd, rList.m_pRuleSet);
	int nPos = rList.Search(wndData);
	if( nPos >= rList.GetSize() ||
		rList[nPos] != wndData )
	{
		if( !wndData.IsHideable() )
			return TRUE;
		rList.InsertAt(nPos, wndData);
	}
	return TRUE;
};

void CWndList::Refresh()
{
	for(int i=GetSize()-1;i>=0;i--)
		if( !(*this)[i].IsHideable() ) RemoveAt(i);
		else (*this)[i].Refresh();
	::EnumWindows(CWndList::RefreshCallback, (LPARAM)this);
}

CWndData& CWndList::operator[](HWND hWnd)
{
	CWndData wndData(hWnd, m_pRuleSet);
	if( !::IsWindow(hWnd) ||
		!wndData.IsHideable() ) 
	{
		AfxThrowUserException();
	}

	int nPos = Search(wndData);
	if( nPos < GetSize() ) {
		if( (*this)[nPos] != wndData )
			InsertAt(nPos, wndData);
	} else 
		nPos = Add(wndData);
	return (*this)[nPos];
}

void CWndList::SetRuleSet(CRuleSet* x)
{
	m_pRuleSet = x;
	for(int i=GetSize()-1;i>=0;i--)
		(*this)[i].m_pRuleSet = m_pRuleSet;
}

void CWndList::ShowAll()
{
	Refresh();
	for(int i=GetSize()-1;i>=0;i--) {
		CWndData& wndData = (*this)[i];
		if( wndData.IsHideable() && !wndData.IsVisible() )
			wndData.Show(TRUE);
	}
}

void CWndList::HideAll()
{
	Refresh();
	for(int i=GetSize()-1;i>=0;i--) {
		CWndData& wndData = (*this)[i];
		if( wndData.IsHideable() && wndData.IsVisible() )
			wndData.Show(FALSE);
	}
}

BOOL CALLBACK CWndList::GetVisibleWndCallback(HWND hWnd, LPARAM lParam)
{
	CWndData& wndData = *(CWndData*)lParam;
	wndData = hWnd;
	if( wndData.IsVisible() && 
		wndData.IsHideable() )
	{
		return FALSE;
	}
	wndData = (HWND)NULL;
	return TRUE;
}

CWndData& CWndList::GetVisibleWnd()
{
	static CWndData wndData;
	wndData.m_pRuleSet = m_pRuleSet;
	::EnumWindows(CWndList::GetVisibleWndCallback, (LPARAM)&wndData);
	return wndData;
}

static
int AFXAPI CompareWndByName(CWndData* const* e1, CWndData* const* e2)
{
	return _tcsicmp((*e1)->GetWindowName(),(*e2)->GetWindowName());
}

CWndList::ByName CWndList::GetWndByName()
{
	Refresh();
	ByName byName;
	byName.SetCompareFunc(CompareWndByName);
	
	for(int i=GetSize()-1;i>=0;i--) byName.Add(&(*this)[i]);
	byName.Sort();
	
	return byName;
}
