/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __WNDLIST_H_INCLUDED__
#define __WNDLIST_H_INCLUDED__

class CRuleSet;
class CWndList;
class CWndData;

template void AFXAPI ConstructElements<CWndData>(CWndData*,int);

class CWndData {
	friend class CWndList;
	friend void AFXAPI ConstructElements(CWndData*,int);
//	process names
	typedef CMap<DWORD,DWORD&,CString,CString&> ProcessNameMap;
	static ProcessNameMap m_mProcessNameCache;
	static BOOL CALLBACK ProcessNameCallback(DWORD,WORD,LPSTR,LPARAM);
//	window style tests for IsHideable()
	static const DWORD RequiredStyle;
	static const DWORD DisabledStyle;
//  user interface 
	BOOL VerifyPassword();
	void CreateTrayIcon();
public:
	static LRESULT SendMainWndMessage(UINT message, WPARAM wParam=0, LPARAM lParam=0);
	static void RefreshProcessCache();
//	special window access
	static CWndData& GetDesktopWnd();
	static CWndData& GetDesktopListCtrl();
	static CWndData& GetTaskbarWnd();
//	window properties
	static LPCTSTR propHidden;
	static LPCTSTR propTray;
	static LPCTSTR propAutoMinimize;
	static LPCTSTR propAutoHide;
	static LPCTSTR propAutoClose;

	DWORD GetProp(LPCTSTR n) const
		{ return (DWORD)::GetProp(m_hWnd, n); }
	void SetProp(LPCTSTR n, DWORD p)
		{ if(p) ::SetProp(m_hWnd, n, (HANDLE)p); else ::RemoveProp(m_hWnd, n); }
	__declspec(property(get=GetProp, put=SetProp)) DWORD property;
private:
	CRuleSet* m_pRuleSet;
	HWND	  m_hWnd;
	mutable CString m_sWindowName;
	mutable CString m_sClassName;
	mutable CString m_sExeName;
	mutable HICON   m_hIcon;
	mutable DWORD   m_dwHotKey;
protected:
	CWndData(HWND hWnd=NULL,CRuleSet* pRuleSet=NULL):
		m_hWnd(hWnd), m_pRuleSet(pRuleSet),
		m_hIcon(NULL), m_dwHotKey(NULL)
		{}
public:
	LPCTSTR GetWindowName() const;
	LPCTSTR GetClassName() const;
	LPCTSTR GetExeName() const;
	DWORD GetHotKey() const;
	HICON GetIcon() const;
	
	CWndData& operator=(HWND hWnd);
	CWndData& operator=(const CWndData& x);
	BOOL operator==(const CWndData& x) const { return m_hWnd == x.m_hWnd; }
	BOOL operator!=(const CWndData& x) const { return m_hWnd != x.m_hWnd; }
	BOOL operator>(const CWndData& x) const  { return m_hWnd > x.m_hWnd; }
	operator HWND() const {	return m_hWnd;	}

	void Refresh() const;
	static BOOL IsVisible(HWND hWnd);
	BOOL IsVisible() const;
	static BOOL IsChanged(HWND hWnd);
	BOOL IsChanged() const;
	static BOOL IsInActiveThread(HWND hWnd);
	BOOL IsInActiveThread() const;
	static void Compact(HWND hWnd);
	void Compact();
	BOOL IsHideable() const;
	BOOL IsTrayable() const;
	void Show(BOOL bShow, BOOL bTray=-1); // bTray=-1 -> follow rules;TRUE -> hide to tray; FALSE -> don't hide to tray
	void AutoMinimize();
	void AutoHide();
	void AutoClose();
};
/*
template<CWndData>
void AFXAPI ConstructElements(CWndData* pElements, int nCount)
{
	ASSERT(nCount == 0 || AfxIsValidAddress(pElements, nCount*sizeof(CWndData)));
	memset((void*)pElements, 0, nCount * sizeof(CWndData));
	for(; nCount--; pElements++) ::new((void*)pElements) CWndData;
}*/

#ifndef YYBISON
class CWndList:
	protected CPxSortedArray<CWndData, CWndData&>
{
	static BOOL CALLBACK RefreshCallback(HWND, LPARAM lParam);
	static BOOL CALLBACK GetVisibleWndCallback(HWND, LPARAM);
	CRuleSet* m_pRuleSet; // ruleset
public:
	typedef CPxArray<CWndData*, CWndData*> ByName;
public:
	CWndList(CRuleSet* x=NULL): m_pRuleSet(x)
		{}
public: //publics from parent
	int Lookup(CWndData& x)
		{ return CPxSortedArray<CWndData, CWndData&>::Lookup(x); }
	int GetSize()
		{ return CPxSortedArray<CWndData, CWndData&>::GetSize(); }
	CWndData& operator[](int i)
		{ return CPxSortedArray<CWndData, CWndData&>::operator[](i); }
	CWndData& operator[](HWND hWnd);
public: //
	CRuleSet* GetRuleSet() { return m_pRuleSet; }
	void SetRuleSet(CRuleSet* x);
	__declspec(property(get=GetRuleSet, put=SetRuleSet)) CRuleSet* RuleSet;
	void Refresh();
	void ShowAll();
	void HideAll();
	CWndData& GetVisibleWnd();
	ByName GetWndByName();
};
#endif //YYBISON

#endif //__WNDLIST_H_INCLUDED__
