/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "dlgRuleEdit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CdlgRuleEdit dialog


CdlgRuleEdit::CdlgRuleEdit(CRule& rule, CWnd* pParent /*=NULL*/)
	: CDialog(CdlgRuleEdit::IDD, pParent),
	m_rule(rule)
{
	//{{AFX_DATA_INIT(CdlgRuleEdit)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void AFXAPI
DDX_Text(CDataExchange* pDX, int nIDC, CRuleExp& value)
{
    HWND hWndCtrl = pDX->PrepareCtrl(nIDC);
    ASSERT (hWndCtrl != NULL);                
    
    CEdit* pEdit=(CEdit*)CWnd::FromHandle(hWndCtrl);
    if(pDX->m_bSaveAndValidate) {
		CString str;
		pEdit->GetWindowText(str);
		value = str;
		if( value.bad() ) {
			AfxMessageBox(value.error());
			pDX->Fail();
		}
	} else {
		pEdit->SetWindowText(value.expression());
    }
}

void CdlgRuleEdit::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CdlgRuleEdit)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
	int _x,_y;
	BOOL bOn,bSave = pDX->m_bSaveAndValidate;
	DDX_SpinMinMax(pDX, IDC_RE_MINIMIZE_SPIN,	_x=0, _y=3600);
	DDX_SpinMinMax(pDX, IDC_RE_HIDE_SPIN,		_x=0, _y=3600);
	DDX_SpinMinMax(pDX, IDC_RE_CLOSE_SPIN,		_x=0, _y=3600);

	DDX_Text (pDX, IDC_RE_EXPR,			   m_rule.expr);
	DDX_Text (pDX, IDC_RE_NAME,			   m_rule.name);

	DDX_Check(pDX, IDC_RE_CBDISABLE,       _x=m_rule.stmt.rsDisable != 0);
	if( bSave ) m_rule.stmt.rsDisable=_x;

	DDX_Check(pDX, IDC_RE_CBIGNORE,        bOn=m_rule.stmt.rsIgnoreWindow != 0);
	DDX_Radio(pDX, IDC_RE_IGNORE_ON,	   _x=m_rule.stmt.rsIgnoreWindow-1);
	if( bSave ) m_rule.stmt.rsIgnoreWindow = bOn?_x+1:0;

	DDX_Check(pDX, IDC_RE_CBMINIMIZE,	   bOn=m_rule.stmt.rsMinimizeWindow != 0);
	DDX_Radio(pDX, IDC_RE_MINIMIZE_ON,	   _x=m_rule.stmt.rsMinimizeWindow-1);
	if( bSave ) m_rule.stmt.rsMinimizeWindow = bOn?_x+1:0;
	DDX_Check(pDX, IDC_RE_MINIMIZE_ACTIVE, m_rule.minimize.bActive);
	DDX_Text (pDX, IDC_RE_MINIMIZE_TIME,   m_rule.minimize.wTimeout);

	DDX_Check(pDX, IDC_RE_CBHIDE,		   bOn=m_rule.stmt.rsHideWindow != 0);
	DDX_Radio(pDX, IDC_RE_HIDE_ON,		   _x=m_rule.stmt.rsHideWindow-1);
	if( bSave ) m_rule.stmt.rsHideWindow = bOn?_x+1:0;
	DDX_Check(pDX, IDC_RE_HIDE_ACTIVE,	   m_rule.hide.bActive);
	DDX_Text (pDX, IDC_RE_HIDE_TIME,	   m_rule.hide.wTimeout);

	DDX_Check(pDX, IDC_RE_CBCLOSE,		   bOn=m_rule.stmt.rsCloseWindow != 0);
	DDX_Radio(pDX, IDC_RE_CLOSE_ON,		   _x=m_rule.stmt.rsCloseWindow-1);
	if( bSave ) m_rule.stmt.rsCloseWindow = bOn?_x+1:0;
	DDX_Check(pDX, IDC_RE_CLOSE_ACTIVE,	   m_rule.close.bActive);
	DDX_Text (pDX, IDC_RE_CLOSE_TIME,	   m_rule.close.wTimeout);

	DDX_Check(pDX, IDC_RE_CBTRAY,		   bOn=m_rule.stmt.rsTrayWindow != 0);
	DDX_Radio(pDX, IDC_RE_TRAY_ON,		   _x=m_rule.stmt.rsTrayWindow-1);
	if( bSave ) m_rule.stmt.rsTrayWindow = bOn?_x+1:0;

	DDX_Check(pDX, IDC_RE_CBPASSWORD,	   bOn=m_rule.stmt.rsPasswordWindow != 0);
	DDX_Radio(pDX, IDC_RE_PASSWORD_ON,	   _x=m_rule.stmt.rsPasswordWindow-1);
	if( bSave ) m_rule.stmt.rsPasswordWindow = bOn?_x+1:0;

	if( !pDX->m_bSaveAndValidate )
		UpdateControlState();
}

void CdlgRuleEdit::UpdateControlState()
{try {
	BOOL bOn, bOn2;
	BOOL bEnable = !((CButton*)(*this)[IDC_RE_CBDISABLE])->GetCheck() == 1;

	(*this)[IDC_RE_NAME]->EnableWindow(bEnable);
	(*this)[IDC_RE_EXPR]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBIGNORE]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBMINIMIZE]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBHIDE]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBCLOSE]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBTRAY]->EnableWindow(bEnable);
	(*this)[IDC_RE_CBPASSWORD]->EnableWindow(bEnable);

	bOn = ((CButton*)(*this)[IDC_RE_CBIGNORE])->GetCheck() == 1;
	(*this)[IDC_RE_IGNORE_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_IGNORE_OFF]->EnableWindow(bEnable && bOn);

	bOn  = ((CButton*)(*this)[IDC_RE_CBMINIMIZE])->GetCheck() == 1;
	bOn2 = ((CButton*)(*this)[IDC_RE_MINIMIZE_ON])->GetCheck() == 1;
	(*this)[IDC_RE_MINIMIZE_TIME]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_MINIMIZE_SPIN]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_MINIMIZE_ACTIVE]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_MINIMIZE_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_MINIMIZE_OFF]->EnableWindow(bEnable && bOn);

	bOn  = ((CButton*)(*this)[IDC_RE_CBHIDE])->GetCheck() == 1;
	bOn2 = ((CButton*)(*this)[IDC_RE_HIDE_ON])->GetCheck() == 1;
	(*this)[IDC_RE_HIDE_TIME]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_HIDE_SPIN]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_HIDE_ACTIVE]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_HIDE_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_HIDE_OFF]->EnableWindow(bEnable && bOn);

	bOn =  ((CButton*)(*this)[IDC_RE_CBCLOSE])->GetCheck() == 1;
	bOn2 = ((CButton*)(*this)[IDC_RE_CLOSE_ON])->GetCheck() == 1;
	(*this)[IDC_RE_CLOSE_TIME]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_CLOSE_SPIN]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_CLOSE_ACTIVE]->EnableWindow(bEnable && bOn && bOn2);
	(*this)[IDC_RE_CLOSE_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_CLOSE_OFF]->EnableWindow(bEnable && bOn);

	bOn = ((CButton*)(*this)[IDC_RE_CBTRAY])->GetCheck() == 1;
	(*this)[IDC_RE_TRAY_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_TRAY_OFF]->EnableWindow(bEnable && bOn);

	bOn = ((CButton*)(*this)[IDC_RE_CBPASSWORD])->GetCheck() == 1;
	(*this)[IDC_RE_PASSWORD_ON]->EnableWindow(bEnable && bOn);
	(*this)[IDC_RE_PASSWORD_OFF]->EnableWindow(bEnable && bOn);
} catch( CResourceException*) {}
}

BEGIN_MESSAGE_MAP(CdlgRuleEdit, CDialog)
	//{{AFX_MSG_MAP(CdlgRuleEdit)
	ON_BN_CLICKED(IDC_RE_CBDISABLE, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBIGNORE, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBMINIMIZE, OnCbClick)
	ON_BN_CLICKED(IDC_RE_MINIMIZE_ON, OnCbClick)
	ON_BN_CLICKED(IDC_RE_MINIMIZE_OFF, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBHIDE, OnCbClick)
	ON_BN_CLICKED(IDC_RE_HIDE_ON, OnCbClick)
	ON_BN_CLICKED(IDC_RE_HIDE_OFF, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBCLOSE, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CLOSE_ON, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CLOSE_OFF, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBTRAY, OnCbClick)
	ON_BN_CLICKED(IDC_RE_CBPASSWORD, OnCbClick)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CdlgRuleEdit message handlers

void CdlgRuleEdit::OnCbClick() 
{
	UpdateControlState();
}
