/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "dlgRuleTest.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CdlgRuleTest dialog
CdlgRuleTest::CdlgRuleTest(CRuleSet& aRuleSet, CWnd* pParent /*=NULL*/):
	CDialog(CdlgRuleTest::IDD, pParent),
	m_aRuleSet(aRuleSet)
{
	//{{AFX_DATA_INIT(CdlgRuleTest)
	//}}AFX_DATA_INIT
}

void CdlgRuleTest::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CdlgRuleTest)
	DDX_Control(pDX, IDC_RT_WINDOWLIST, m_lvList);
	//}}AFX_DATA_MAP
	if( !pDX->m_bSaveAndValidate ) {
		CString sTemp;
		m_lvList.DeleteAllItems();
		while( m_lvList.GetHeaderCtrl()->GetItemCount() )
			m_lvList.DeleteColumn(0);
		CRect rc; m_lvList.GetClientRect(rc);
		rc.right -= ::GetSystemMetrics(SM_CXVSCROLL);
		m_lvList.InsertColumn(0, ((*this)[IDC_RT_WINDOW_TITLE]->GetWindowText(sTemp),sTemp), LVCFMT_LEFT, rc.Width()/3, 0);
		m_lvList.InsertColumn(1, ((*this)[IDC_RT_CLASS_TITLE]->GetWindowText(sTemp),sTemp),  LVCFMT_LEFT, rc.Width()/3, 0);
		m_lvList.InsertColumn(2, ((*this)[IDC_RT_EXE_TITLE]->GetWindowText(sTemp),sTemp),    LVCFMT_LEFT, rc.Width()/3, 0);

		m_aWindows.Refresh();
		m_ilWndIcons.DeleteImageList();
		m_ilWndIcons.Create(16,16,ILC_COLOR8|ILC_MASK,m_aWindows.GetSize(),0);
		for(int i=0; i < m_aWindows.GetSize(); i++)
			m_ilWndIcons.Add(m_aWindows[i].GetIcon());
		m_lvList.SetImageList(&m_ilWndIcons,LVSIL_SMALL);

		for(i=m_aWindows.GetSize()-1; i>=0; i--)
			m_lvList.InsertItem(0, LPSTR_TEXTCALLBACK,i);
		
		m_lvList.SetExtendedStyle(m_lvList.GetExtendedStyle() |
			LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT );
	}
}

BEGIN_MESSAGE_MAP(CdlgRuleTest, CDialog)
	//{{AFX_MSG_MAP(CdlgRuleTest)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_RT_WINDOWLIST, OnListGetdispInfo)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_RT_WINDOWLIST, OnListItemChanged)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CdlgRuleTest message handlers

void CdlgRuleTest::OnListGetdispInfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLVDISPINFO* pDI = (NMLVDISPINFO*)pNMHDR;
	pDI->item.mask = LVIF_TEXT;
	if( pDI->item.iSubItem == 0 ) {
		pDI->item.pszText = (LPTSTR)m_aWindows[pDI->item.iItem].GetWindowName();
		pDI->item.iImage = pDI->item.iItem;
	} else
	if( pDI->item.iSubItem == 1 ) {
		pDI->item.pszText = (LPTSTR)m_aWindows[pDI->item.iItem].GetClassName();
	} else
	if( pDI->item.iSubItem == 2 ) {
		pDI->item.pszText = (LPTSTR)m_aWindows[pDI->item.iItem].GetExeName();
	}
	
	*pResult = 0;
}

void CdlgRuleTest::OnListItemChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	*pResult = 0;
	NMLISTVIEW* pNM = (NMLISTVIEW*)pNMHDR;
	if( (pNM->uOldState ^ pNM->uNewState) & LVIS_FOCUSED &&
		pNM->uNewState & LVIS_FOCUSED )
	{
		CString sReport;
		const CRule* rule;
		CWndData &wndData = m_aWindows[pNM->iItem];
		do {
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfIgnoreWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_IGNORE_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfIgnoreWindow(1)) ) {
				sReport += StrPrintf(IDS_RT_REPORT_IGNORE, (LPCTSTR)rule->name);
				continue;
			}
			
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfTrayWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_TRAY_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfTrayWindow(1)) )
				sReport += StrPrintf(IDS_RT_REPORT_TRAY, (LPCTSTR)rule->name);

			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfPasswordWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_PASSWORD_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfPasswordWindow(1)) )
				sReport += StrPrintf(IDS_RT_REPORT_PASSWORD, (LPCTSTR)rule->name);

			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfMinimizeWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_MINIMIZE_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfMinimizeWindow(1)) )
				sReport += StrPrintf(IDS_RT_REPORT_MINIMIZE, rule->minimize.wTimeout, (LPCTSTR)rule->name);
			
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfHideWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_HIDE_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfHideWindow(1)) )
				sReport += StrPrintf(IDS_RT_REPORT_HIDE, rule->hide.wTimeout, (LPCTSTR)rule->name);
			
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfCloseWindow(2)) )
				sReport += StrPrintf(IDS_RT_REPORT_CLOSE_OFF, (LPCTSTR)rule->name); else
			if( rule=m_aRuleSet._FindRule(wndData, CRule::rsfCloseWindow(1)) )
				sReport += StrPrintf(IDS_RT_REPORT_CLOSE, rule->close.wTimeout, (LPCTSTR)rule->name);
		} while(0);
		if( sReport.IsEmpty() )
			sReport = StrPrintf(IDS_RT_REPORT_NORULES);
		(*this)[IDC_RT_REPORT_TITLE]->SetWindowText(
			StrPrintf(IDS_RT_REPORT_TITLE, wndData.GetWindowName()));
		(*this)[IDC_RT_REPORT]->SetWindowText(sReport);
	}
}
