/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "globals.h"
#include "wcat.h"
#include "ppRuleSet.h"
#include "dlgRuleEdit.h"
#include "dlgRuleTest.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CppRuleSet dialog


CppRuleSet::CppRuleSet(CWnd* pParent /*=NULL*/)
	: CPropertyPage(CppRuleSet::IDD)
{
	//{{AFX_DATA_INIT(CppRuleSet)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CppRuleSet::SetData(CMainWnd* pWnd)
{
	m_aRuleSet.RemoveAll();
	for(int i=0;i<pWnd->m_aRuleSet.GetSize();i++)
		m_aRuleSet.Add(pWnd->m_aRuleSet[i]);
}

void CppRuleSet::GetData(CMainWnd* pWnd)
{
	pWnd->m_aRuleSet.RemoveAll();
	for(int i=0;i<m_aRuleSet.GetSize();i++)
		pWnd->m_aRuleSet.Add(m_aRuleSet[i]);
}

void CppRuleSet::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CppRuleSet)
	DDX_Control(pDX, IDC_RS_LIST, m_lvList);
	//}}AFX_DATA_MAP
	if( !pDX->m_bSaveAndValidate ) {
		CString sTemp;

		m_lvList.DeleteAllItems();
		while( m_lvList.GetHeaderCtrl()->GetItemCount() )
			m_lvList.DeleteColumn(0);

		CRect rc; m_lvList.GetClientRect(rc);
		rc.right -= ::GetSystemMetrics(SM_CXVSCROLL);
		m_lvList.InsertColumn(0, ((*this)[IDC_RS_RULE_TITLE]->GetWindowText(sTemp),sTemp), LVCFMT_LEFT, rc.Width()/2, 0);
		m_lvList.InsertColumn(1, ((*this)[IDC_RS_EXPR_TITLE]->GetWindowText(sTemp),sTemp), LVCFMT_LEFT, rc.Width()/2, 0);
		for(int i=0;i<m_aRuleSet.GetSize();i++)
			m_lvList.InsertItem(0, LPSTR_TEXTCALLBACK);
	}
}

void CppRuleSet::UpdateItem(UINT nItem)
{
	m_lvList.SetItemText(nItem,0,LPSTR_TEXTCALLBACK);
	m_lvList.SetItemText(nItem,1,LPSTR_TEXTCALLBACK);
	m_lvList.Update(nItem);
}

BEGIN_MESSAGE_MAP(CppRuleSet, CPropertyPage)
	//{{AFX_MSG_MAP(CppRuleSet)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_RS_LIST, OnListGetdispInfo)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_RS_LIST, OnListItemChanged)
	ON_NOTIFY(LVN_KEYDOWN, IDC_RS_LIST, OnListKeyDown)
	ON_BN_CLICKED(IDC_RS_BTNADD, OnAdd)
	ON_BN_CLICKED(IDC_RS_BTNEDIT, OnEdit)
	ON_BN_CLICKED(IDC_RS_BTNDEL, OnDelete)
	ON_BN_CLICKED(IDC_RS_BTNMOVEDN, OnMoveDn)
	ON_BN_CLICKED(IDC_RS_BTNMOVEUP, OnMoveUp)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_RS_BTNTEST, OnTest)
	ON_NOTIFY(NM_DBLCLK, IDC_RS_LIST, OnListDblClick)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CppRuleSet message handlers

BOOL CppRuleSet::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	ttm.Install(this);
	ttm.SetTooltipWidth(200);

	((CButton*)(*this)[IDC_RS_BTNADD])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNADD)));
	((CButton*)(*this)[IDC_RS_BTNDEL])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNDEL)));
	((CButton*)(*this)[IDC_RS_BTNEDIT])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNOPEN)));
	((CButton*)(*this)[IDC_RS_BTNMOVEUP])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNMOVEUP)));
	((CButton*)(*this)[IDC_RS_BTNMOVEDN])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNMOVEDN)));
	((CButton*)(*this)[IDC_RS_BTNTEST])->SetBitmap(
		::LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNRULETEST)));
	m_ilRuleState.Create(IDD,16,0,RGB(255,0,255));
	m_lvList.SetImageList(&m_ilRuleState, LVSIL_SMALL);
	m_lvList.SetExtendedStyle(m_lvList.GetExtendedStyle() |
		LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT );


	return TRUE;
}

void CppRuleSet::OnDestroy() 
{
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNADD])->GetBitmap());
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNDEL])->GetBitmap());
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNEDIT])->GetBitmap());
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNMOVEUP])->GetBitmap());
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNMOVEDN])->GetBitmap());
	::DeleteObject(((CButton*)(*this)[IDC_RS_BTNTEST])->GetBitmap());

	CPropertyPage::OnDestroy();
}

void CppRuleSet::OnListGetdispInfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLVDISPINFO* pDI = (NMLVDISPINFO*)pNMHDR;
	pDI->item.mask = LVIF_TEXT;
	if( pDI->item.iSubItem == 0 ) {
		pDI->item.mask |= LVIF_IMAGE;
		pDI->item.iImage  = m_aRuleSet[pDI->item.iItem].stmt.rsDisable;
		pDI->item.pszText = (LPTSTR)(LPCTSTR)m_aRuleSet[pDI->item.iItem].name;
	} else
	if( pDI->item.iSubItem == 1 ) {
		pDI->item.pszText = (LPTSTR)(LPCTSTR)m_aRuleSet[pDI->item.iItem].expr;
	}
	*pResult = 0;
}

void CppRuleSet::OnListItemChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNM = (NM_LISTVIEW*)pNMHDR;
	// TODO: Add your control notification handler code here
	*pResult = 0;
}

void CppRuleSet::OnListKeyDown(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_KEYDOWN* pNM = (LV_KEYDOWN*)pNMHDR;
	if( pNM->wVKey == VK_INSERT) OnAdd(); else
	if( pNM->wVKey == VK_DELETE) OnDelete();
	*pResult = 0;
}

void CppRuleSet::OnListDblClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnEdit();
	*pResult = 0;
}

void CppRuleSet::OnTest() 
{
	CdlgRuleTest tester(m_aRuleSet, this);
	tester.DoModal();
}

void CppRuleSet::OnAdd() 
{
	CPxMenuManager mManager;
	mManager.Install(this);
	mManager.LoadToolbar(IDR_OVERLAY);

	CMenu2 cMenu; cMenu.CreatePopupMenu();
	cMenu.AppendMenu(MF_STRING, ID_WND_FIRST-1, "New rule");
	cMenu.AppendMenu(MF_SEPARATOR);
	::SetMenuDefaultItem(cMenu,ID_WND_FIRST-1,FALSE);

	CWndList _aWndList;
	CWndList::ByName aWndList = _aWndList.GetWndByName();
	for( int i=0;i<aWndList.GetSize();i++ )
	{
		cMenu.AppendMenu(MF_STRING, ID_WND_FIRST+i, 
			MAINWND->TruncateWndName(aWndList[i]->GetWindowName()));
		mManager.SetItemImage(ID_WND_FIRST+i, aWndList[i]->GetIcon());
	}
	CRect rcBtn; (*this)[IDC_RS_BTNADD]->GetWindowRect(rcBtn);
	UINT nResult = cMenu.TrackPopupMenu(MenuFlags, rcBtn.right, rcBtn.top,
		this);

	CRule rule;
	if( nResult >= ID_WND_FIRST && nResult <= ID_WND_LAST ) 
	{
		rule.name = StrPrintf("The Rule for \"%s\"", cMenu.GetMenuString(nResult) );
		CString sExe = aWndList[nResult-ID_WND_FIRST]->GetExeName();
		if( sExe.ReverseFind('\\') >= 0 )
			sExe = sExe.Mid(sExe.ReverseFind('\\')+1);
		rule.expr = StrPrintf("c/%s/ci & e/%s$/ri",
			aWndList[nResult-ID_WND_FIRST]->GetClassName(), (LPCTSTR)sExe);
	} else
	if ( nResult == ID_WND_FIRST-1 ) {
		rule.name = _T("New Rule");
	} else {
		m_lvList.SetFocus();
		return;
	}

	CdlgRuleEdit editor(rule, this);
	if( editor.DoModal() == IDOK ) {
		m_aRuleSet.Add(rule);
		m_lvList.InsertItem(m_aRuleSet.GetSize()-1, LPSTR_TEXTCALLBACK);
	}
	m_lvList.SetFocus();
}

void CppRuleSet::OnEdit() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused >= 0 ) {
		CdlgRuleEdit editor(m_aRuleSet[nFocused], this);
		if( editor.DoModal() == IDOK ) UpdateItem(nFocused);
	}
}

void CppRuleSet::OnDelete() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused >= 0 ) {
		m_lvList.DeleteItem(nFocused);
		m_aRuleSet.RemoveAt(nFocused);
	}
}

void CppRuleSet::OnMoveDn() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;
	if( nFocused >= (m_lvList.GetItemCount()-1)) return;

	CRule tmp = m_aRuleSet[nFocused];
	m_aRuleSet[nFocused]   = m_aRuleSet[nFocused+1];
	m_aRuleSet[nFocused+1] = tmp;
	UpdateItem(nFocused);
	UpdateItem(nFocused+1);
	m_lvList.SetItemState(nFocused+1, LVIS_FOCUSED|LVIS_SELECTED,-1);
}

void CppRuleSet::OnMoveUp() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;
	if( nFocused == 0 ) return;

	CRule tmp = m_aRuleSet[nFocused-1];
	m_aRuleSet[nFocused-1] = m_aRuleSet[nFocused];
	m_aRuleSet[nFocused]   = tmp;
	UpdateItem(nFocused);
	UpdateItem(nFocused-1);
	m_lvList.SetItemState(nFocused-1, LVIS_FOCUSED|LVIS_SELECTED,-1);
}


