/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "globals.h"
#include "ppfolders.h"
#include "shlcache.h"
#include "shlmenu.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPPFolders dialog

CPPFolders::CPPFolders()
	: CPropertyPage(CPPFolders::IDD)
{
	//{{AFX_DATA_INIT(CPPFolders)
	//}}AFX_DATA_INIT
}

void CPPFolders::SetData(CMainWnd* pWnd)
{
	m_aItemData.RemoveAll();
	for(int i=0; i < pWnd->m_aShellMenus.GetSize(); i++) {
		m_aItemData.Add(new CShlRoot());
		*m_aItemData[m_aItemData.GetSize()-1] = *pWnd->m_aShellMenus[i];
	}
	pWnd->m_CwdMenuTemplate.SetDisplayName(LoadString(ID_SHL_CWD));
	m_aItemData.InsertAt(0, new CShlRoot());
	*m_aItemData[0] = pWnd->m_CwdMenuTemplate;
}

void CPPFolders::GetData(CMainWnd* pWnd)
{
	pWnd->m_aShellTrayIcons.RemoveAll();
	pWnd->m_aShellMenus.RemoveAll();
	for(int i=1;i<m_aItemData.GetSize();i++) {
		pWnd->m_aShellMenus.Add(new CShlRoot());
		*pWnd->m_aShellMenus[pWnd->m_aShellMenus.GetSize()-1] = *m_aItemData[i];
	}
	pWnd->m_CwdMenuTemplate = *m_aItemData[0];
}

void CPPFolders::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPPFolders)
	DDX_Control(pDX, IDC_FLDR_TRAYICON, m_cbTrayIcon);
	DDX_Control(pDX, IDC_FLDR_SORTBYTYPE, m_cbSortByType);
	DDX_Control(pDX, IDC_FLDR_NESTLEVEL, m_edNestLevel);
	DDX_Control(pDX, IDC_FLDR_FILEMASK, m_edFileMask);
	DDX_Control(pDX, IDC_FLDR_FLDRONTOP, m_cbFoldersOnTop);
	DDX_Control(pDX, IDC_FLDR_SHOWHIDDEN, m_cbShowHidden);
	DDX_Control(pDX, IDC_FLDR_EXPRM, m_cbExpandRemovable);
	DDX_Control(pDX, IDC_FLDR_EXPLINK, m_cbExpandLinks);
	DDX_Control(pDX, IDC_FLDR_LIST, m_lvList);
	DDX_Control(pDX, IDC_FLDR_HOTKEY, m_hkMenuKey);
	DDX_Control(pDX, IDC_FLDR_MENUNAME, m_edMenuName);
	//}}AFX_DATA_MAP
	int _aa, _bb;
	DDX_SpinMinMax(pDX, IDC_FLDR_NESTLEVEL_SPIN, _aa=1, _bb=10);

	if( !pDX->m_bSaveAndValidate ) {
		m_lvList.DeleteAllItems();
		while( m_lvList.GetHeaderCtrl()->GetItemCount() )
			m_lvList.DeleteColumn(0);

		CString sTemp;
		int iTemp;
		
		CRect rc; m_lvList.GetClientRect(&rc);
		(*this)[IDC_FLDR_MENULBL]->GetWindowText(sTemp);
		if( (iTemp = sTemp.Find('&')) >= 0 ) sTemp.Delete(iTemp);
		m_lvList.InsertColumn(0, sTemp, LVCFMT_LEFT, rc.Width()/2, 0); rc.right-= rc.Width()/2;
		
		(*this)[IDC_FLDR_HOTKEYLBL]->GetWindowText(sTemp);
		if( (iTemp = sTemp.Find('&')) >= 0 ) sTemp.Delete(iTemp);
		m_lvList.InsertColumn(1, sTemp, LVCFMT_LEFT, 70, 0); rc.right-= 70;

		(*this)[IDC_FLDR_FLDR_TITLE]->GetWindowText(sTemp);
		if( (iTemp = sTemp.Find('&')) >= 0 ) sTemp.Delete(iTemp);
		m_lvList.InsertColumn(2, sTemp, LVCFMT_LEFT, rc.Width(), 0);

		for(iTemp=m_aItemData.GetSize()-1;iTemp>=0;iTemp--)
			m_lvList.InsertItem(0, LPSTR_TEXTCALLBACK);
	}
}

BEGIN_MESSAGE_MAP(CPPFolders, CPropertyPage)
	//{{AFX_MSG_MAP(CPPFolders)
	ON_BN_CLICKED(IDC_FLDR_BTNBROWSE, OnBrowseClick)
	ON_NOTIFY(LVN_GETDISPINFO, IDC_FLDR_LIST, OnListGetdispinfo)
	ON_NOTIFY(LVN_ITEMCHANGED, IDC_FLDR_LIST, OnListItemChanged)
	ON_BN_CLICKED(IDC_FLDR_BTNHKSAVE, OnHkSaveClick)
	ON_EN_UPDATE(IDC_FLDR_MENUNAME, OnMenunameChange)
	ON_BN_CLICKED(IDC_FLDR_BTNADD, OnAddClick)
	ON_BN_CLICKED(IDC_FLDR_BTNDEL, OnDelClick)
	ON_BN_CLICKED(IDC_FLDR_BTNMOVEDN, OnMovednClick)
	ON_BN_CLICKED(IDC_FLDR_BTNMOVEUP, OnMoveupClick)
	ON_BN_CLICKED(IDC_FLDR_TESTMENU, OnTestMenu)
	ON_EN_UPDATE(IDC_FLDR_NESTLEVEL, OnNestLevelChange)
	ON_EN_UPDATE(IDC_FLDR_FILEMASK, OnFilemaskChange)
	ON_BN_CLICKED(IDC_FLDR_EXPLINK, OnExpandLinksChange)
	ON_BN_CLICKED(IDC_FLDR_EXPRM, OnExpandRemovableChange)
	ON_BN_CLICKED(IDC_FLDR_FLDRONTOP, OnFoldersOnTopChange)
	ON_BN_CLICKED(IDC_FLDR_SHOWHIDDEN, OnrShowHiddenChange)
	ON_WM_DESTROY()
	ON_BN_CLICKED(IDC_FLDR_SORTBYTYPE, OnSortByTypeChange)
	ON_BN_CLICKED(IDC_FLDR_TRAYICON, OnTrayIconChange)
	ON_NOTIFY(LVN_KEYDOWN, IDC_FLDR_LIST, OnListKeyDown)
	ON_NOTIFY(NM_DBLCLK, IDC_FLDR_LIST, OnListDblClick)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPPFolders message handlers

BOOL CPPFolders::OnInitDialog()
{
	CPropertyPage::OnInitDialog();
	ttm.Install(this);
	ttm.SetTooltipWidth(200);

	((CButton*)GetDlgItem(IDC_FLDR_BTNADD))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNADD)));
	((CButton*)GetDlgItem(IDC_FLDR_BTNDEL))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNDEL)));
	((CButton*)GetDlgItem(IDC_FLDR_BTNMOVEUP))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNMOVEUP)));
	((CButton*)GetDlgItem(IDC_FLDR_BTNMOVEDN))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNMOVEDN)));
	((CButton*)GetDlgItem(IDC_FLDR_BTNBROWSE))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNOPEN)));
	((CButton*)GetDlgItem(IDC_FLDR_BTNHKSAVE))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNSAVE)));
	((CButton*)GetDlgItem(IDC_FLDR_TESTMENU))->
		SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_BTNMENUTEST)));

	m_lvList.SetExtendedStyle(m_lvList.GetExtendedStyle() |
		LVS_EX_INFOTIP | LVS_EX_FULLROWSELECT );

	return TRUE;
}

void CPPFolders::OnDestroy() 
{
	CPropertyPage::OnDestroy();
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNADD))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNDEL))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNMOVEUP))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNMOVEDN))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNBROWSE))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_BTNHKSAVE))->GetBitmap());
	DeleteObject(((CButton*)GetDlgItem(IDC_FLDR_TESTMENU))->GetBitmap());
}

void CPPFolders::OnBrowseClick() 
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	LPITEMIDLIST pidl = BrowseForFolder(*(CPidl*)(m_aItemData[nFocused]));
	if( pidl ) {
		m_aItemData[nFocused]->Attach(pidl);
		UpdateItem(nFocused);
	}
}

void CPPFolders::OnAddClick() 
{
	m_lvList.SetFocus();	
	LPITEMIDLIST pidl = BrowseForFolder();
	if( pidl ) {
		m_aItemData.Add(new CShlRoot(pidl));
		m_lvList.InsertItem(m_lvList.GetItemCount(), LPSTR_TEXTCALLBACK);
		m_lvList.SetItemState(m_lvList.GetItemCount()-1, LVIS_FOCUSED|LVIS_SELECTED,-1);
	}
}

void CPPFolders::OnDelClick() 
{
	m_lvList.SetFocus();	
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;
	m_lvList.DeleteItem(nFocused);
	m_aItemData.RemoveAt(nFocused);
	if(--nFocused < 0) nFocused = 0;
	m_lvList.SetItemState(nFocused, LVIS_FOCUSED|LVIS_SELECTED,-1);
}

void CPPFolders::OnMovednClick() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;
	if( nFocused >= (m_lvList.GetItemCount()-1)) return;

	CShlRoot* tmp = m_aItemData[nFocused];
	m_aItemData[nFocused]   = m_aItemData[nFocused+1];
	m_aItemData[nFocused+1] = tmp;
	UpdateItem(nFocused);
	UpdateItem(nFocused+1);
	m_lvList.SetItemState(nFocused+1, LVIS_FOCUSED|LVIS_SELECTED,-1);
}

void CPPFolders::OnMoveupClick() 
{
	m_lvList.SetFocus();	
	int nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;
	if( nFocused == 0 ) return;

	CShlRoot* tmp = m_aItemData[nFocused-1];
	m_aItemData[nFocused-1] = m_aItemData[nFocused];
	m_aItemData[nFocused]   = tmp;
	UpdateItem(nFocused);
	UpdateItem(nFocused-1);
	m_lvList.SetItemState(nFocused-1, LVIS_FOCUSED|LVIS_SELECTED,-1);
}

void CPPFolders::OnHkSaveClick() 
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	m_aItemData[nFocused]->SetHotKey((WORD)m_hkMenuKey.GetHotKey());
	UpdateItem(nFocused);
}

BOOL GetShellItemName(
	/*in*/  LPCITEMIDLIST pidl,
	/*in*/  DWORD dwFlags,
	/*out*/ CString& sDisplayName);

void CPPFolders::OnListGetdispinfo(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLVDISPINFO* pDI = (NMLVDISPINFO*)pNMHDR;
	pDI->item.mask = LVIF_TEXT;
	if( pDI->item.iSubItem == 0 ) {
		pDI->item.pszText = (LPTSTR)m_aItemData[pDI->item.iItem]->GetDisplayName();
	} else
	if( pDI->item.iSubItem == 1 ) {
		CString sTemp = ::GetKeyName(m_aItemData[pDI->item.iItem]->GetHotKey());
		m_lvList.SetItemText(pDI->item.iItem, pDI->item.iSubItem, sTemp);
	} else
	if( pDI->item.iSubItem == 2 ) {
		CString sTemp;
		if( ::GetShellItemName((*(CPidl*)m_aItemData[pDI->item.iItem]),
			  SHGDN_NORMAL|SHGDN_FORPARSING, sTemp) )
		{
			m_lvList.SetItemText(pDI->item.iItem, pDI->item.iSubItem, sTemp);
		} else
			pDI->item.pszText = _T("<<not applicable>>");
	}
}

void CPPFolders::OnListItemChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMLISTVIEW* pNM = (NMLISTVIEW*)pNMHDR;
	if( (pNM->uOldState ^ pNM->uNewState) & LVIS_FOCUSED &&
		pNM->uNewState & LVIS_FOCUSED )
	{
		CShlRoot* pItem = m_aItemData[pNM->iItem];
		m_edMenuName.SetWindowText(pItem->GetDisplayName());
		m_hkMenuKey.SetHotKey(LOBYTE(pItem->GetHotKey()),HIBYTE(pItem->GetHotKey()));
		m_edFileMask.SetWindowText(pItem->GetFileMask());
		m_edNestLevel.SetWindowText( StrPrintf("%d", pItem->GetNestLevel()) );
		m_cbExpandLinks.SetCheck( (CShlRoot::optExpandLinks & pItem->GetOptions()) != 0 );
		m_cbExpandRemovable.SetCheck( (CShlRoot::optExpandRemovable & pItem->GetOptions()) != 0 );
		m_cbFoldersOnTop.SetCheck( (CShlRoot::optFoldersOnTop & pItem->GetOptions()) != 0 );
		m_cbShowHidden.SetCheck( (CShlRoot::optShowHidden & pItem->GetOptions()) != 0 );
		m_cbSortByType.SetCheck( (CShlRoot::optSortByType & pItem->GetOptions()) != 0 );
		m_cbTrayIcon.SetCheck( (CShlRoot::optTrayIcon & pItem->GetOptions()) != 0 );

		m_edMenuName.EnableWindow(pNM->iItem);
		m_hkMenuKey.EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNDEL]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNHKSAVE]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNHKSAVE]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNMOVEDN]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNMOVEUP]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_BTNBROWSE]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_TESTMENU]->EnableWindow(pNM->iItem);
		(*this)[IDC_FLDR_TRAYICON]->EnableWindow(pNM->iItem);
	}
}

void CPPFolders::OnListKeyDown(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_KEYDOWN* pNM = (LV_KEYDOWN*)pNMHDR;
	if( pNM->wVKey == VK_INSERT) OnAddClick(); else
	if( pNM->wVKey == VK_DELETE) OnDelClick();
	*pResult = 0;
}


void CPPFolders::OnListDblClick(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnBrowseClick();
	*pResult = 0;
}

void CPPFolders::OnMenunameChange() 
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	CString sDisplayName;
	m_edMenuName.GetWindowText(sDisplayName);
	m_aItemData[nFocused]->SetDisplayName(sDisplayName);
	UpdateItem(nFocused);
}

void CPPFolders::OnNestLevelChange() 
{
	if( !::IsWindow(m_lvList.m_hWnd) ) return;

	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	CString sText;
	m_edNestLevel.GetWindowText(sText);
	m_aItemData[nFocused]->SetNestLevel( _ttoi(sText) );
	UpdateItem(nFocused);
}

void CPPFolders::OnFilemaskChange() 
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	CString sText;
	m_edFileMask.GetWindowText(sText);
	m_aItemData[nFocused]->SetFileMask( sText );
	UpdateItem(nFocused);
}

void CPPFolders::SetOption(CButton* btn, int nOpt)
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	UINT opt = m_aItemData[nFocused]->GetOptions();
	if( btn->GetCheck() ) opt |= nOpt; else opt &= ~nOpt;
	m_aItemData[nFocused]->SetOptions(opt);
}

void CPPFolders::OnExpandLinksChange() 
{
	SetOption(&m_cbExpandLinks, CShlRoot::optExpandLinks);
}

void CPPFolders::OnExpandRemovableChange() 
{
	SetOption(&m_cbExpandRemovable, CShlRoot::optExpandRemovable);
}

void CPPFolders::OnFoldersOnTopChange() 
{
	SetOption(&m_cbFoldersOnTop, CShlRoot::optFoldersOnTop);
}

void CPPFolders::OnrShowHiddenChange() 
{
	SetOption(&m_cbShowHidden, CShlRoot::optShowHidden);
}

void CPPFolders::OnSortByTypeChange() 
{
	SetOption(&m_cbSortByType, CShlRoot::optSortByType);
}


void CPPFolders::OnTrayIconChange() 
{
	SetOption(&m_cbTrayIcon, CShlRoot::optTrayIcon);
}

static
int CALLBACK BrowseCallbackProc(
    HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	if( uMsg == BFFM_INITIALIZED && lpData) {
		::SendMessage(hwnd,	BFFM_SETSELECTION,
			(WPARAM)FALSE, lpData);
	} else
	if( uMsg == BFFM_SELCHANGED) {
		TCHAR  szPath[MAX_PATH];
		if( !SHGetPathFromIDList((LPITEMIDLIST) lParam, szPath)) szPath[0] = 0;
        ::SendMessage(hwnd, BFFM_SETSTATUSTEXT, 0, (LPARAM)szPath);
	}
	return 0;
}

LPITEMIDLIST CPPFolders::BrowseForFolder(LPITEMIDLIST pidlStart/*=NULL*/)
{
	INIT<BROWSEINFO> b;
	//b.lpszTitle = _T("Browse for folder xx");
	b.hwndOwner = this->GetSafeHwnd();
	b.lpfn = ::BrowseCallbackProc;
	b.lParam = (LPARAM)pidlStart;
	//b.ulFlags = BIF_VALIDATE|BIF_STATUSTEXT;

	return ::SHBrowseForFolder(&b);
}

void CPPFolders::UpdateItem(UINT nItem)
{
	m_lvList.SetItemText(nItem,0,LPSTR_TEXTCALLBACK);
	m_lvList.SetItemText(nItem,1,LPSTR_TEXTCALLBACK);
	m_lvList.SetItemText(nItem,2,LPSTR_TEXTCALLBACK);
}

void CPPFolders::OnTestMenu() 
{
	UINT nFocused = m_lvList.GetNextItem(-1,LVNI_FOCUSED);
	if( nFocused == -1 ) return;

	CWnd *pTestBtn;
	if( !(pTestBtn = GetDlgItem(IDC_FLDR_TESTMENU)) ) return;
	CRect rcBtn; pTestBtn->GetWindowRect(rcBtn);
	try {
		CShlMenu mnu;
		mnu.LoadToolbar(IDR_SHELL);
		mnu.SetRefreshID(ID_SHL_REFRESH);
		mnu.SetOpenFolderString(IDS_OPEN_FOLDER);
		mnu.SetTruncate(MAINWND->m_nTruncate);
		UINT nResult = mnu.TrackPopupMenu(*m_aItemData[nFocused],
			MenuFlags, rcBtn.right, rcBtn.top);
		if( nResult ) {
			CShlItem* item = m_aItemData[nFocused]->GetItemByID(nResult);
			if( item ) item->Invoke();
		}
	} catch( _com_error e ) {
		AfxMessageBox( (LPCTSTR)e.ErrorMessage(), MB_ICONSTOP );
	}
}
