/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
// PPWCat.cpp : implementation file
//

#include "stdafx.h"
#include "GLOBALS.H"
#include "WCAT.H"
#include "PPMain.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void CPPMain::ChkCombo::MeasureItem(LPMEASUREITEMSTRUCT lpm)
{
	if( lpm->itemID == -1 ) return;

	CRect rcText;
	CWindowDC dc(NULL); CFont* pOldFont = dc.SelectObject( GetFont() );

	CString Str; GetLBText(lpm->itemID, Str);
	if( ((CPPMain*)GetParent())->m_HotKey[lpm->itemID] != 0 ) {
		Str += _T(" ");
		Str += ::GetKeyName(((CPPMain*)GetParent())->m_HotKey[lpm->itemID]);
	}
	dc.DrawText(Str, rcText, DT_SINGLELINE|DT_LEFT|DT_VCENTER|DT_CALCRECT);
	if( rcText.Width() > (int)lpm->itemWidth )
		lpm->itemWidth = rcText.Width();
	dc.SelectObject( pOldFont );
}

void CPPMain::ChkCombo::DrawItem(LPDRAWITEMSTRUCT lpd)
{
	if( lpd->itemID == -1 ) return;

	CDC dc; dc.Attach(lpd->hDC); int nSaveID = dc.SaveDC();
	
	if( lpd->itemState & ODS_SELECTED ) {
		dc.SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));
		dc.SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
	} else {
		dc.SetBkColor(::GetSysColor(COLOR_WINDOW));
		dc.SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));
	}
	CRect rc(lpd->rcItem);
	dc.FillRect(rc, &CBrush(dc.GetBkColor()));
	if( lpd->itemState & ODS_SELECTED )
		dc.DrawFocusRect(rc);

	dc.SelectObject(GetFont());
	CString sText; GetLBText(lpd->itemID, sText);
	dc.DrawText(sText, (rc.left++,rc), DT_SINGLELINE|DT_LEFT|DT_VCENTER);
	if( (lpd->itemState & ODS_COMBOBOXEDIT) == 0 &&
		((CPPMain*)GetParent())->m_HotKey[lpd->itemID] != 0 )
	{
		sText = ::GetKeyName(((CPPMain*)GetParent())->m_HotKey[lpd->itemID]);
		if( !sText.IsEmpty() )
			dc.DrawText(sText, (rc.right--,rc), DT_SINGLELINE|DT_RIGHT|DT_VCENTER);
	}
	dc.RestoreDC(nSaveID); dc.Detach();
}

/////////////////////////////////////////////////////////////////////////////
// CPPMain property page
CPPMain::CPPMain():
	CPropertyPage(CPPMain::IDD),
	m_cbCurrentIcon(m_cbCurrentIcon.IconSmall)
{
	m_psp.dwFlags |= PSP_USEICONID;
	m_psp.pszIcon = (char*)MAKEINTRESOURCE(IDR_MAINFRAME);
	//{{AFX_DATA_INIT(CPPMain)
	m_nLanguage = -1;
	m_bEnableIcon = FALSE;
	m_bUseOverlay = FALSE;
	m_bSortWindows = FALSE;
	//}}AFX_DATA_INIT
	ZeroMemory(&m_MouseAction, sizeof(m_MouseAction));

	GetModuleFileName(NULL, m_sIconFile.GetBuffer(_MAX_PATH),_MAX_PATH); m_sIconFile.ReleaseBuffer();
	m_nIconIndex = 0;
}

void CPPMain::GetData(CMainWnd* pWnd)
{
	theApp.SetLanguage(m_nLanguage);
	theApp.SetAutoRun(m_bAutoRun);
	pWnd->m_sIconFile		= m_sIconFile;
	pWnd->m_nIconIndex		= m_nIconIndex;
	pWnd->m_bEnableIcon		= m_bEnableIcon;
	pWnd->m_bUseCompact		= m_bUseCompact;
	pWnd->m_bUseRestore		= m_bUseRestore;
	pWnd->m_nTruncate		= m_nTruncate;
	pWnd->m_bTopOnly		= m_bTopOnly;
	pWnd->m_bShowIcons		= m_bShowIcons;
	pWnd->m_MouseAction		= m_MouseAction;
	pWnd->m_MouseTAction	= m_MouseTAction;
	memcpy( pWnd->m_HotKey, m_HotKey, sizeof(pWnd->m_HotKey));
	pWnd->m_bUseOverlay		= m_bUseOverlay;
	pWnd->m_bSortWindows	= m_bSortWindows;
}

void CPPMain::SetData(CMainWnd* pWnd)
{
	m_nLanguage		= theApp.GetLanguage();
	m_sIconFile		= pWnd->m_sIconFile;
	m_nIconIndex	= pWnd->m_nIconIndex;
	m_bEnableIcon	= pWnd->m_bEnableIcon;
	m_bAutoRun		= theApp.IsAutoRun();
	m_bUseCompact	= pWnd->m_bUseCompact;
	m_bUseRestore	= pWnd->m_bUseRestore;
	m_nTruncate		= pWnd->m_nTruncate;
	m_bTopOnly		= pWnd->m_bTopOnly;
	m_bShowIcons	= pWnd->m_bShowIcons;
	m_MouseAction	= pWnd->m_MouseAction;
	m_MouseTAction	= pWnd->m_MouseTAction;
	memcpy(m_HotKey, pWnd->m_HotKey, sizeof(pWnd->m_HotKey));
	m_bUseOverlay   = pWnd->m_bUseOverlay;
	m_bSortWindows  = pWnd->m_bSortWindows;
}

void CPPMain::DoDataExchange(CDataExchange* pDX)
{
	ASSERT(sizeof(MAINWND->m_HotKey)==sizeof(m_HotKey));
	int i = 0,j;

	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPPMain)
	DDX_Control(pDX, IDC_PPW_MOUSEKEYMOD, m_cbKeyMod);
	DDX_Control(pDX, IDC_PPW_TMOUSEKEYMOD, m_cbTKeyMod);
	DDX_Control(pDX, IDC_PPW_TMOUSE, m_cbTMouse);
	DDX_Control(pDX, IDC_PPW_MOUSE, m_cbMouse);
	DDX_Control(pDX, IDC_PPW_HOTKEY, m_hkHotKey);
	DDX_Control(pDX, IDC_PPW_HOTKEYTYPE, m_cbHotkeyType);
	DDX_Control(pDX, IDC_PPW_ENABLEICON, m_cbEnableIcon);
	DDX_Control(pDX, IDC_PPW_TACTION, m_cbTAction);
	DDX_Control(pDX, IDC_PPW_ACTION, m_Action);
	DDX_Check(pDX, IDC_PPW_USECOMPACT,	m_bUseCompact);
	DDX_Check(pDX, IDC_PPW_USERESTORE,	m_bUseRestore);
	DDX_Text(pDX, IDC_PPW_TRUNCATE,	m_nTruncate);
	DDV_MinMaxUInt(pDX, m_nTruncate,0,99);
	DDX_Check(pDX, IDC_PPW_TOPONLY, m_bTopOnly);
	DDX_Check(pDX, IDC_PPW_SHOWICONS, m_bShowIcons);
	DDX_Check(pDX, IDC_PPW_AUTORUN,	m_bAutoRun);
	DDX_CBIndex(pDX, IDC_PPW_LANGUAGE, m_nLanguage);
	DDX_Check(pDX, IDC_PPW_ENABLEICON, m_bEnableIcon);
	DDX_Check(pDX, IDC_PPW_USEOVERLAY, m_bUseOverlay);
	DDX_Check(pDX, IDC_PPW_SORTWINDOWS, m_bSortWindows);
	//}}AFX_DATA_MAP
	DDX_Control(pDX, IDC_PPW_ICONBTN, m_cbCurrentIcon);
	DDX_PxIconPicker(pDX, IDC_PPW_ICONBTN, m_sIconFile, m_nIconIndex);
	DDX_CBIndex(pDX, IDC_PPW_MOUSEKEYMOD, i=0);
	DDX_CBIndex(pDX, IDC_PPW_MOUSE,  i=0);
	DDX_CBIndex(pDX, IDC_PPW_ACTION, i=m_MouseAction.data[0]);
	DDX_CBIndex(pDX, IDC_PPW_TMOUSEKEYMOD, i=0);
	DDX_CBIndex(pDX, IDC_PPW_TMOUSE,  i=0);
	DDX_CBIndex(pDX, IDC_PPW_TACTION, i=m_MouseTAction.data[0]);
	DDX_CBIndex(pDX, IDC_PPW_HOTKEYTYPE, i=0);
	DDX_SpinMinMax(pDX, IDC_PPW_TRUNCATESPIN, i=0, j=99);

	if( !pDX->m_bSaveAndValidate ) {
		m_hkHotKey.SetHotKey(LOBYTE(m_HotKey[0]),HIBYTE(m_HotKey[0]));
		OnEnableIcon();
	} else {
	//  make sure that WatchCat will not be locked by disabling both hotkey and tray icon
		if( m_bEnableIcon==0 && m_HotKey[hkMenu]==0) {
			AfxMessageBox(IDS_ERR_HOTKEYANDICONDISABLED);
			m_cbEnableIcon.SetFocus();
			pDX->Fail();
		}
	//	make sure that at least one mouse action is assigned
		for(i=0;i<countof(m_MouseAction.data);i++)
			if( m_MouseAction.data[i] == ACTION_SHOWLIST ) break;
		if(i >=countof(m_MouseAction.data))
			m_MouseAction.data[maRightBtnClick]=ACTION_SHOWLIST;
	//	make sure that at least one mouse action is assigned
		for(i=0;i<countof(m_MouseTAction.data);i++)
			if( m_MouseTAction.data[i] != TACTION_NOTHING ) break;
		if(i >=countof(m_MouseTAction.data))
			m_MouseTAction.data[maRightBtnClick]=TACTION_MENU;
	}
}


BEGIN_MESSAGE_MAP(CPPMain, CPropertyPage)
	//{{AFX_MSG_MAP(CPPMain)
	ON_CBN_SELCHANGE(IDC_PPW_MOUSEKEYMOD, OnMouseSelChange)
	ON_CBN_SELCHANGE(IDC_PPW_ACTION, OnActionSelChange)
	ON_CBN_SELCHANGE(IDC_PPW_TACTION, OnTActionSelChange)
	ON_CBN_SELCHANGE(IDC_PPW_TMOUSEKEYMOD, OnTMouseSelChange)
	ON_BN_CLICKED(IDC_PPW_ENABLEICON, OnEnableIcon)
	ON_CBN_SELCHANGE(IDC_PPW_HOTKEYTYPE, OnHotkeyTypeSelChange)
	ON_BN_CLICKED(IDC_PPW_HKSAVE, OnSaveHotKey)
	ON_CBN_SELCHANGE(IDC_PPW_MOUSE, OnMouseSelChange)
	ON_CBN_SELCHANGE(IDC_PPW_TMOUSE, OnTMouseSelChange)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL SetCbImageList(CComboBoxEx& rCb, CImageList& rIl)
{
	rCb.SetImageList(&rIl);

	INIT<COMBOBOXEXITEM> item;
	item.mask = CBEIF_IMAGE | CBEIF_OVERLAY | CBEIF_SELECTEDIMAGE;
	for(int i=rCb.GetCount()-1; i>=0; i--) {
		item.iItem = item.iImage = item.iSelectedImage = item.iOverlay = i;
		rCb.SetItem(&item);
	}
	return TRUE;
}

int AdjustDropWidth(CComboBox* pCb)
{
	if( !pCb ) return 0;
	if( !pCb->GetSafeHwnd() ) return 0;

	int nWidth = pCb->GetDroppedWidth() - GetSystemMetrics(SM_CYVSCROLL) - 6;
	int nOriginalWidth = nWidth;
	for( int i = 0; i<pCb->GetCount();i++ ) {
		CWindowDC dc(NULL);	// screen DC--I won't actually draw on it
		CRect rcText(0,0,0,0);
		CFont* pFont = pCb->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CString Str; pCb->GetLBText(i,Str);
		dc.DrawText(Str, rcText, DT_SINGLELINE|DT_LEFT|DT_VCENTER|DT_CALCRECT);
		if( rcText.Width() > nWidth )
			nWidth = rcText.Width();
		dc.SelectObject( pOldFont );

		//if it's ownerdrawn combo, check owner drawn item size
		if( pCb->GetStyle() & (CBS_OWNERDRAWVARIABLE|CBS_OWNERDRAWFIXED) ) {
			INIT<MEASUREITEMSTRUCT> ms;
			ms.CtlID     = pCb->GetDlgCtrlID();
			ms.CtlType   = ODT_COMBOBOX;
			ms.itemData  = pCb->GetItemData(i);
			ms.itemID    = i;
			ms.itemWidth = nWidth;
			pCb->GetParent()->SendMessage(WM_MEASUREITEM, ms.CtlID, (LPARAM)&ms);
			if( (int)ms.itemWidth > nWidth ) nWidth = ms.itemWidth;
		}
	}
	if( nWidth > nOriginalWidth ) {
		nWidth += 6;
		nWidth += GetSystemMetrics(SM_CYVSCROLL);
		if( pCb->GetRuntimeClass() == RUNTIME_CLASS(CComboBoxEx) ) {
			nWidth += 3;
			CImageList* pImg = ((CComboBoxEx*)pCb)->GetImageList();
			if( pImg ) {
				IMAGEINFO imgInfo;
				pImg->GetImageInfo(0,&imgInfo);
				nWidth += imgInfo.rcImage.right;
			}
		}
		pCb->SetDroppedWidth(nWidth);
	}
	return nWidth;
}


BOOL CPPMain::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	ttm.Install(this);
	ttm.SetTooltipWidth(200);

	m_cbKeyMod.EnableToolTips(TRUE);
	m_cbTKeyMod.EnableToolTips(TRUE);
	m_cbTMouse.EnableToolTips(TRUE);
	m_cbMouse.EnableToolTips(TRUE);
	
	m_ilMouse.Create(IDR_MOUSECLICK,16,0,RGB(255,0,255));
	::SetCbImageList(m_cbMouse,m_ilMouse);
	::SetCbImageList(m_cbTMouse,m_ilMouse);

	m_ilKbdHit.Create(IDR_KBDHIT,16,0,RGB(255,0,255));
	::SetCbImageList(m_cbKeyMod,m_ilKbdHit);
	::SetCbImageList(m_cbTKeyMod,m_ilKbdHit);
	
	AdjustDropWidth(&m_cbKeyMod);
	AdjustDropWidth(&m_cbMouse);
	AdjustDropWidth(&m_Action);
	AdjustDropWidth(&m_cbTKeyMod);
	AdjustDropWidth(&m_cbTMouse);
	AdjustDropWidth(&m_cbTAction);
	AdjustDropWidth((CComboBox*)GetDlgItem(IDC_PPW_LANGUAGE));
	AdjustDropWidth(&m_cbHotkeyType);

	((CButton*)GetDlgItem(IDC_PPW_HKSAVE))->SetBitmap(
		LoadBitmap(AfxGetInstanceHandle(),MAKEINTRESOURCE(IDB_BTNSAVE)));
	GetDlgItem(IDC_PPW_LANGUAGE)->EnableWindow(::IsWinNT());
	GetDlgItem(IDC_PPW_USECOMPACT)->EnableWindow(::IsWinNT());

	return TRUE;
}

void CPPMain::OnMouseSelChange() 
{
	if( m_cbMouse.GetCurSel() == CB_ERR ) return;
	if( m_cbKeyMod.GetCurSel() == CB_ERR ) return;
	int Sel = m_cbMouse.GetCurSel() + maLast * m_cbKeyMod.GetCurSel();
	m_Action.SetCurSel(m_MouseAction.data[Sel]);
}

void CPPMain::OnActionSelChange() 
{
	if( m_cbMouse.GetCurSel() == CB_ERR ) return;
	if( m_cbKeyMod.GetCurSel() == CB_ERR ) return;
	int Sel = m_cbMouse.GetCurSel() + maLast * m_cbKeyMod.GetCurSel();
	m_MouseAction.data[Sel] = m_Action.GetCurSel();
}

void CPPMain::OnTMouseSelChange() 
{
	if( m_cbTMouse.GetCurSel() == CB_ERR ) return;
	if( m_cbTKeyMod.GetCurSel() == CB_ERR ) return;
	int Sel = m_cbTMouse.GetCurSel() + maLast * m_cbTKeyMod.GetCurSel();
	m_cbTAction.SetCurSel(m_MouseTAction.data[Sel]);
}

void CPPMain::OnTActionSelChange() 
{
	if( m_cbTMouse.GetCurSel() == CB_ERR ) return;
	if( m_cbTKeyMod.GetCurSel() == CB_ERR ) return;
	int Sel = m_cbTMouse.GetCurSel() + maLast * m_cbTKeyMod.GetCurSel();
	m_MouseTAction.data[Sel] = m_cbTAction.GetCurSel();
}

void CPPMain::OnHotkeyTypeSelChange() 
{
	int nCurSel = m_cbHotkeyType.GetCurSel();
	if( nCurSel != CB_ERR ) {
		m_hkHotKey.SetHotKey(LOBYTE(m_HotKey[nCurSel]),HIBYTE(m_HotKey[nCurSel]));
	}
}

void CPPMain::OnSaveHotKey() 
{
	int nCurSel = m_cbHotkeyType.GetCurSel();
	if( nCurSel != CB_ERR ) {
		WORD wHotKey = (WORD)m_hkHotKey.GetHotKey();
	//	validate hotkey
		if( wHotKey ) {
			if( !MAINWND->ValidateHotKey((HotKeys)nCurSel, wHotKey) ) {
				AfxMessageBox(IDS_ERR_HOTKEY_USED);
				return;
			}
			for(int i=0; i<hkLast; i++ ) {
				if( i == nCurSel ) continue;
				if( wHotKey == m_HotKey[i] ) {
					AfxMessageBox(IDS_ERR_HOTKEY_USED);
					return;
				}
			}
		}
	//	assign it
		m_HotKey[nCurSel] = wHotKey;
		AdjustDropWidth(&m_cbHotkeyType);
		m_hkHotKey.SetFocus();
	}
}

void CPPMain::OnEnableIcon() 
{
	m_cbCurrentIcon.EnableWindow( m_cbEnableIcon.GetCheck() == 1 );
}

/////////////////////////////////////////////////////////////////////////////
// COptionsSheet
COptionsSheet::COptionsSheet(UINT nIDCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(nIDCaption, pParentWnd, iSelectPage)
{
	AddControlPages();
}

COptionsSheet::COptionsSheet(LPCTSTR pszCaption, CWnd* pParentWnd, UINT iSelectPage)
	:CPropertySheet(pszCaption, pParentWnd, iSelectPage)
{
	AddControlPages();
}

COptionsSheet::~COptionsSheet()
{
}

void COptionsSheet::AddControlPages()
{
	AddPage(&m_ppMain);
	AddPage(&m_ppMisc);
	AddPage(&m_ppRuleSet);
	AddPage(&m_ppFolders);
	AddPage(&m_ppAbout);
}

BEGIN_MESSAGE_MAP(COptionsSheet, CPropertySheet)
	//{{AFX_MSG_MAP(COptionsSheet)
	ON_WM_HELPINFO()
	ON_BN_CLICKED(IDC_OPT_AUTOHELP, OnAutoHelp)
	ON_WM_CONTEXTMENU()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL COptionsSheet::OnInitDialog() 
{
	CPropertySheet::OnInitDialog();
	SetIcon(::CopyIcon(::PxGetWindowIcon(AfxGetMainWnd()->GetSafeHwnd())),   TRUE);
	SetIcon(::CopyIcon(::PxGetWindowIconSm(AfxGetMainWnd()->GetSafeHwnd())), FALSE);
	ModifyStyleEx(NULL, WS_EX_CONTEXTHELP|WS_EX_APPWINDOW);

	m_ttm.Install(this);
	m_ttm.SetTooltipWidth(200);
//	hide "apply" button
	CRect rcApply;   GetDlgItem(ID_APPLY_NOW)->GetWindowRect(rcApply); ScreenToClient(rcApply);
	CRect rcCancel;  GetDlgItem(IDCANCEL)->GetWindowRect(rcCancel); ScreenToClient(rcCancel);
	GetDlgItem(ID_APPLY_NOW)->ShowWindow(SW_HIDE);
	GetDlgItem(IDCANCEL)->MoveWindow(rcApply);
	GetDlgItem(IDOK)->MoveWindow(rcCancel);
//	add "autohelp" check box
	CRect rcOK;  GetDlgItem(IDOK)->GetWindowRect(rcOK); ScreenToClient(rcOK);
	CRect rcTab; GetDlgItem(AFX_IDC_TAB_CONTROL)->GetWindowRect(rcTab); ScreenToClient(rcTab);
	m_cbAutoHelp.Create(::LoadString(IDS_AUTOHELP),
		BS_AUTOCHECKBOX|WS_CHILD|WS_VISIBLE,
		CRect(rcTab.left,rcOK.top,rcOK.left,rcOK.bottom),this,
		IDC_OPT_AUTOHELP);
	m_cbAutoHelp.SetFont(GetFont());
	m_cbAutoHelp.SetCheck(AfxGetApp()->GetProfileInt(_T(""),_T("AutoHelp"),TRUE));
	OnAutoHelp();

	return TRUE;
}

void COptionsSheet::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	HWND hWnd = ::WindowFromPoint(point);
	if( hWnd ) {
		DWORD ctx[4];
		ZeroMemory(&ctx, sizeof(ctx));
		ctx[0] = ::GetDlgCtrlID(hWnd);
		ctx[1] = ::GetWindowContextHelpId(hWnd);
		if( ctx[1] ) //FYI CApp::WinHelp doesn't work here
			::WinHelp(hWnd, theApp.GetHelpFileName(),
				HELP_CONTEXTMENU, (DWORD)&ctx);
	}
}

BOOL COptionsSheet::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	theApp.WinHelp(pHelpInfo->dwContextId, HELP_CONTEXTPOPUP);
	return TRUE;
}

void COptionsSheet::OnAutoHelp()
{
	BOOL bOn = m_cbAutoHelp.GetCheck();
	m_ppMain.ttm.SetHelpTip(bOn);
	m_ppMisc.ttm.SetHelpTip(bOn);
	m_ppFolders.ttm.SetHelpTip(bOn);
	m_ppRuleSet.ttm.SetHelpTip(bOn);
	AfxGetApp()->WriteProfileInt(_T(""),_T("AutoHelp"),bOn);
}

/////////////////////////////////////////////////////////////////////////////
// CPPAbout property page
CPPAbout::CPPAbout():
	CPropertyPage(CPPAbout::IDD)
{
	//{{AFX_DATA_INIT(CPPAbout)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CPPAbout::~CPPAbout()
{
}

BEGIN_MESSAGE_MAP(CPPAbout, CPropertyPage)
	//{{AFX_MSG_MAP(CPPAbout)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CPPAbout::OnInitDialog() 
{
	if( !CPropertyPage::OnInitDialog() )
		return FALSE;

	CPxModuleVersion WCatVer;
	if( WCatVer.GetFileVersionInfo(NULL) ) {
		CString sStr;

		if( GetDlgItem(IDC_ABOUT_CONTACT) ) {
			sStr.Format(_T("mailto:%s?subject=%s%%20%d.%d.%d"),
				WCatVer.GetValue(_T("Contact-EMail")),
				WCatVer.GetValue(_T("ProductName")),
				HIWORD(WCatVer.dwProductVersionMS),
				LOWORD(WCatVer.dwProductVersionMS),
				LOWORD(WCatVer.dwFileVersionLS));
			CPxHyperlink* pLink = new CPxHyperlink(sStr, TRUE);
			pLink->SubclassDlgItem(IDC_ABOUT_CONTACT, this);
		}
		if( GetDlgItem(IDC_ABOUT_WWW) ) {
			CPxHyperlink* pLink = new CPxHyperlink(WCatVer.GetValue(_T("Contact-WebPage")), TRUE);
			pLink->SubclassDlgItem(IDC_ABOUT_WWW, this);
		}
		if( GetDlgItem(IDC_ABOUT_TRANS_CZECH) ) {
			CPxHyperlink* pLink = new CPxHyperlink("mailto:pavelr@iname.com", TRUE);
			pLink->SubclassDlgItem(IDC_ABOUT_TRANS_CZECH, this);
		}
		if( GetDlgItem(IDC_ABOUT_APPINFO) ) {
			sStr.Format(LoadString(IDC_ABOUT_APPINFO),
				WCatVer.GetValue(_T("ProductName")),
				HIWORD(WCatVer.dwProductVersionMS),LOWORD(WCatVer.dwProductVersionMS),LOWORD(WCatVer.dwFileVersionLS),
				WCatVer.GetValue(_T("Comments")),
				WCatVer.GetValue(_T("LegalCopyright")));
			GetDlgItem(IDC_ABOUT_APPINFO)->SetWindowText(sStr);
		}
		if( GetDlgItem(IDC_ABOUT_LICENSE) ) {
			GetDlgItem(IDC_ABOUT_LICENSE)->SetWindowText(LoadString(IDS_LICENSE));
		}
	}
	return TRUE;
}


void COptionsSheet::OnPaint() 
{
	if (IsIconic()) {
		CPaintDC dc(this); // device context for painting
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, ::PxGetWindowIconSm(GetSafeHwnd()));
	} else {
		CPropertySheet::OnPaint();
	}
}
