/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#if !defined(AFX_PPWCAT_H__C00E16E7_EBB4_11D1_A0AA_00A0C941B362__INCLUDED_)
#define AFX_PPWCAT_H__C00E16E7_EBB4_11D1_A0AA_00A0C941B362__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "PPMisc.h"
#include "PPFolders.h"
#include "PPRuleSet.h"

class COptionsSheet;

/////////////////////////////////////////////////////////////////////////////
// CPPMain dialog
class CPPMain : public CPropertyPage
{
	friend class COptionsSheet;
private:
	// owner-drawn combo for hotkey actions 
	// it shows key names beside action
	class ChkCombo:	public CComboBox {
	public:
		ChkCombo() {}
		void MeasureItem(LPMEASUREITEMSTRUCT lpm);
		void DrawItem(LPDRAWITEMSTRUCT lpm);
	};
	friend class CPPMain::ChkCombo;
private:
// Dialog Data
	//{{AFX_DATA(CPPMain)
	enum { IDD = IDD_PPMAIN };
	CComboBoxEx2	m_cbKeyMod;
	CComboBoxEx2	m_cbTKeyMod;
	CComboBoxEx2	m_cbTMouse;
	CComboBoxEx2	m_cbMouse;
	CHotKeyCtrl	m_hkHotKey;
	ChkCombo	m_cbHotkeyType;
	CButton	m_cbEnableIcon;
	CComboBox	m_cbTAction;
	CComboBox	m_Action;
	BOOL		m_bUseCompact;
	BOOL		m_bUseRestore;
	UINT		m_nTruncate; 
	BOOL		m_bTopOnly;
	BOOL		m_bShowIcons;
	BOOL		m_bAutoRun;
	int		m_nLanguage;
	BOOL	m_bEnableIcon;
	BOOL	m_bUseOverlay;
	BOOL	m_bSortWindows;
	//}}AFX_DATA
	CPxIconPicker	m_cbCurrentIcon;
	WORD			m_HotKey[hkLast];
	CMouseAction	m_MouseAction;
	CMouseAction	m_MouseTAction;
	CString		m_sIconFile;
	UINT		m_nIconIndex;
	CPxTooltipManager	ttm;
	CImageList			m_ilMouse;
	CImageList			m_ilKbdHit;
public:
	CPPMain();
	void SetData(CMainWnd* pWnd);
	void GetData(CMainWnd* pWnd);

// Overrides
	// ClassWizard generate virtual function overrides
	//{{AFX_VIRTUAL(CPPMain)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	// Generated message map functions
	//{{AFX_MSG(CPPMain)
	afx_msg void OnMouseSelChange();
	afx_msg void OnActionSelChange();
	virtual BOOL OnInitDialog();
	afx_msg void OnTActionSelChange();
	afx_msg void OnTMouseSelChange();
	afx_msg void OnEnableIcon();
	afx_msg void OnHotkeyTypeSelChange();
	afx_msg void OnSaveHotKey();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////
// CPPAbout dialog
class CPPAbout : public CPropertyPage
{
	friend class COptionsSheet;
// Construction
public:
	CPPAbout();
	~CPPAbout();

// Dialog Data
	//{{AFX_DATA(CPPAbout)
	enum { IDD = IDD_ABOUT };
		// NOTE - ClassWizard will add data members here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_DATA


// Overrides
	// ClassWizard generate virtual function overrides
	//{{AFX_VIRTUAL(CPPAbout)
	//}}AFX_VIRTUAL

// Implementation
protected:
	// Generated message map functions
	//{{AFX_MSG(CPPAbout)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

};

/////////////////////////////////////////////////////////////////////////////
// COptionsSheet
class COptionsSheet : public CPropertySheet
{
public:
	COptionsSheet(UINT nIDCaption, CWnd* pParentWnd = NULL, UINT iSelectPage = 0);
	COptionsSheet(LPCTSTR pszCaption, CWnd* pParentWnd = NULL, UINT iSelectPage = 0);
protected:
	void AddControlPages(void);
public:
public:
	//{{AFX_VIRTUAL(COptionsSheet)
	public:
	virtual BOOL OnInitDialog();
	//}}AFX_VIRTUAL
public:
	CPPAbout	m_ppAbout;
	CPPMain		m_ppMain;
	CppRuleSet	m_ppRuleSet;
	CPPFolders	m_ppFolders;
	CPPMisc		m_ppMisc;
	virtual ~COptionsSheet();
protected:
	CPxTooltipManager	m_ttm;
	CButton		m_cbAutoHelp;
	//{{AFX_MSG(COptionsSheet)
	afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
	afx_msg void OnAutoHelp();
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnPaint();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_PPWCAT_H__C00E16E7_EBB4_11D1_A0AA_00A0C941B362__INCLUDED_)
