/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "shlcache.h"
#include "fnmatch.h"

#ifdef _UNICODE
	#define IShellLinkPtr	IShellLinkWPtr	
	#define IExtractIconPtr	IExtractIconWPtr
#else
	#define IShellLinkPtr	IShellLinkAPtr	
	#define IExtractIconPtr	IExtractIconAPtr
#endif

CString GetKeyName(DWORD wHotKey)
{
	CString str;
	BYTE bModifier = HIBYTE(wHotKey);
	if( bModifier & HOTKEYF_CONTROL )  str += _T("Ctrl+");
	if( bModifier & HOTKEYF_ALT )      str += _T("Alt+");
	if( bModifier & HOTKEYF_SHIFT )    str += _T("Shift+");
	{
		TCHAR keyname[64];
		UINT vkey = MapVirtualKey(LOBYTE(wHotKey), 0)<<16;
		if( !vkey ) return _T("");
		vkey = GetKeyNameText(vkey, keyname, sizeof(keyname));
		if( !vkey ) return _T("");
		str += keyname;
	}
	return str;
}

BOOL GetShellItemName(
	/*in*/  LPCITEMIDLIST pidl,
	/*in*/  DWORD dwFlags,
	/*out*/ CString& sDisplayName)
{
	SHFILEINFO sfi;
	IShellFolderPtr pFolder;
	CPidl pidlRel;
	if( SUCCEEDED(SHBindToParent(pidl, __uuidof(pFolder),
			(void**)&pFolder, pidlRel)) )
	{
		STRRET str;

		if( SUCCEEDED(pFolder->GetDisplayNameOf(pidlRel, dwFlags, &str)) )
			switch (str.uType) {
			case STRRET_WSTR:
				sDisplayName = CString(str.pOleStr);
				break;
			case STRRET_OFFSET:
				sDisplayName = CString((LPSTR)(LPITEMIDLIST)pidlRel + str.uOffset);
				break;
			case STRRET_CSTR:
				sDisplayName = str.cStr;
				break;
		}
	} else
	if( ::SHGetFileInfo((LPCTSTR)pidl, NULL, &sfi, sizeof(sfi),
		SHGFI_PIDL|SHGFI_DISPLAYNAME) )
	{
		sDisplayName = sfi.szDisplayName;
	} else
		return FALSE;
	return TRUE;
}

static inline
CString GetShellLinkPath(
	/*in*/  LPCITEMIDLIST pidl)
{
	CString sPath;
	::GetShellItemName(pidl, SHGDN_NORMAL|SHGDN_FORPARSING, sPath);
	return sPath;
}

static inline
IShellLinkPtr OpenShellLink(
	/*in*/  LPCITEMIDLIST pidl)
{
	IShellLinkPtr pLink(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER);
	if( (bool)pLink ) {
		IPersistFilePtr pFile = pLink;
		if( FAILED(pFile->Load(_bstr_t(::GetShellLinkPath(pidl)), STGM_READ)) )
			pLink = NULL;
	}
	return pLink;
}

static inline
IShellLinkPtr OpenShellLink(
	/*in*/  LPCTSTR pszPath)
{
	IShellLinkPtr pLink(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER);
	if( (bool)pLink ) {
		IPersistFilePtr pFile = pLink;
		if( FAILED(pFile->Load(_bstr_t(pszPath), STGM_READ)) )
			pLink = NULL;
	}
	return pLink;
}

static
BOOL GetShellItemData(
	/*in*/  LPCITEMIDLIST pidl,
	/*out*/ CString* psDisplayName,
	/*out*/ HICON*   phIcon,
	/*out*/ WORD*    pwHotKey,
	/*out*/ CString* psType)
{
	SHFILEINFO sfi;
	if( !pidl )	return FALSE;

	if( psDisplayName ) psDisplayName->Empty();
	if( psType ) psType->Empty();
	if( phIcon && *phIcon ) ::DestroyIcon(*phIcon);
	if( pwHotKey ) *pwHotKey = 0;

	if( psDisplayName ) {
		if( !::GetShellItemName(pidl, SHGDN_INFOLDER, *psDisplayName) )
			return FALSE;
	}

	sfi.dwAttributes = SFGAO_DISPLAYATTRMASK;
	if( !::SHGetFileInfo((LPCTSTR)pidl, NULL, &sfi, sizeof(sfi),
		SHGFI_PIDL|SHGFI_ATTRIBUTES|SHGFI_DISPLAYNAME|
		(psType?SHGFI_TYPENAME:NULL)) )
		return FALSE;
	if( psType ) *psType = sfi.szTypeName;

	BOOL bIsLink = sfi.dwAttributes & SFGAO_LINK;
	if( phIcon &&
		::SHGetFileInfo((LPCTSTR)pidl, NULL, &sfi, sizeof(sfi),
			SHGFI_PIDL|SHGFI_ICON|SHGFI_SMALLICON|
			(bIsLink?SHGFI_LINKOVERLAY:NULL)))
	{
		*phIcon = sfi.hIcon;
	}
	
	if( bIsLink && pwHotKey ) {
		IShellLinkPtr pLink = ::OpenShellLink(pidl);
		if( (bool)pLink ) pLink->GetHotkey(pwHotKey);
	}
	return TRUE;
}

static
BOOL IsFolder(LPITEMIDLIST *pidl, UINT nOptions) 
{
	HRESULT hr;

	if( !*pidl ) {
		TRACE("IsFolder(): NULL pidl passed.\n");
		return FALSE;
	}

	if( CompareItemID(CSIDL_BITBUCKET, *pidl) == 0 ) {
		TRACE("IsFolder(): Idnoring trash can.\n");
		return FALSE; //ignore recycle bin
	}

	SHFILEINFO sfi;
	sfi.dwAttributes = SFGAO_DISPLAYATTRMASK|SFGAO_CONTENTSMASK;
	if( !::SHGetFileInfo((LPCTSTR)*pidl, NULL, &sfi, sizeof(sfi),
		SHGFI_PIDL|SHGFI_ATTRIBUTES|SHGFI_DISPLAYNAME) )
		return FALSE;

	if( (sfi.dwAttributes & SFGAO_LINK) &&
		(nOptions & CShlRoot::optExpandLinks) )
	{
		IShellLinkPtr pLink = ::OpenShellLink(*pidl);
		if( !(bool)pLink ) return FALSE;

		TCHAR szPath[_MAX_PATH];
		CPidl linkPidl;
		WIN32_FIND_DATA FindData;
		if( FAILED(hr = pLink->GetIDList(linkPidl)) ) {
			TRACE("IsFolder(): IShellLink->GetIDList() (hr=%0X) failed for shortcut '%s'\n",
				hr, sfi.szDisplayName);
			if( FAILED(hr = pLink->GetPath(szPath, countof(szPath), &FindData, SLGP_UNCPRIORITY)) )
			{
				TRACE("IsFolder(): IShellLink->GetParh() (hr=%0X) failed for shortcut '%s'\n",
					hr, sfi.szDisplayName);
				return FALSE;
			}
			linkPidl = ::GetItemIDFromPath(szPath);
		}

		if( linkPidl.IsEmpty() ) 
			return FALSE;
		if( ::IsFolder(linkPidl, nOptions) ) {
			GetMalloc()->Free(*pidl);
			*pidl = linkPidl.Detach();
			return TRUE;
		}
	} else
		return
			(sfi.dwAttributes & SFGAO_FOLDER) &&
			((nOptions & CShlRoot::optExpandRemovable) || (sfi.dwAttributes & SFGAO_REMOVABLE) == 0);
	return FALSE;
}

static 
BOOL FnMatch(LPCITEMIDLIST pidl, CArray<CString, CString&>& aFileMask)
{
	if( !pidl )
		return FALSE;

	if( aFileMask.GetSize() == 0 )
		return TRUE; // the mask is empty -> permit all

//	prepare file name
	TCHAR szPath[_MAX_PATH];
	LPCTSTR pszFName, pszMask;
	if( !::SHGetPathFromIDList(pidl, szPath) ) {
	//	filename is not accessible - try to match object name
		CString sName; //::GetShellItemData(pidl, &sName, NULL, NULL, NULL);
		::GetShellItemName(pidl, SHGDN_NORMAL|SHGDN_INCLUDE_NONFILESYS|SHGDN_FORPARSING, sName);
		_tcsncpy(szPath, sName, countof(szPath)); szPath[countof(szPath)-1]=0;
	}
	pszFName = _tcsrchr(szPath, '\\');
	if( !pszFName ) pszFName = szPath; else pszFName++;
	if( pszFName[0] == 0 )	return -1; // no path
//	BOOL bResult = fnmatch(pszMask, pszFName, FNM_PERIOD|FNM_CASEFOLD) == 0;

//	check for exclusion masks
	for(int i=0; i < aFileMask.GetSize(); i++) {
		pszMask = aFileMask[i];
		if( *pszMask++ != '!' )
			continue;
		if( fnmatch(pszMask, pszFName, FNM_PERIOD|FNM_CASEFOLD) == 0 )
			return FALSE;
	}

//	check for inclusion masks
	for(i=0; i < aFileMask.GetSize(); i++) {
		pszMask = aFileMask[i];
		if( *pszMask == '!' )
			continue;
		if( fnmatch(pszMask, pszFName, FNM_PERIOD|FNM_CASEFOLD) == 0 )
			return TRUE;
	}
	return FALSE;
}

static
BOOL InvokeIdList(LPCITEMIDLIST pidl, LPCTSTR pszVerb = NULL)
{
	INIT2<SHELLEXECUTEINFO> sei;
	sei.nShow    = SW_SHOWDEFAULT;
	sei.hwnd     = AfxGetMainWnd()->GetSafeHwnd();
	sei.lpVerb   = pszVerb;
	if( pidl->mkid.cb ) {
		sei.fMask    = SEE_MASK_INVOKEIDLIST;
		sei.lpIDList = (LPVOID)pidl;
	} else {
		TCHAR szPath[_MAX_PATH];
		if( !::SHGetPathFromIDList(pidl, szPath) )
			return FALSE;
		sei.lpFile = szPath;
	}
	return ::ShellExecuteEx(&sei);
}

//=======================================================================
// CShlItem
CShlItem::CShlItem(CShlRoot* pRoot):
	m_pRoot(pRoot),
	m_hIcon(NULL), m_wHotKey(NULL), m_nMenuID(0)
{
}

CShlItem::CShlItem(CShlRoot* pRoot, LPITEMIDLIST pidl):
	CPidl(pidl), 
	m_pRoot(pRoot),
	m_hIcon(NULL), m_wHotKey(NULL), m_nMenuID(0)
{
}

CShlItem::~CShlItem()
{
	if( m_hIcon ) ::DestroyIcon(m_hIcon);
}

void CShlItem::GetData()
{
	if( m_sDisplayName.IsEmpty() || !m_hIcon)
		GetShellItemData(*(CPidl*)this,
			&m_sDisplayName, &m_hIcon, &m_wHotKey, &m_sType);
}

HICON CShlItem::GetIcon() const
{
	((CShlItem*)this)->GetData(); return m_hIcon;
}

WORD CShlItem::GetHotKey() const
{
	((CShlItem*)this)->GetData(); return m_wHotKey;
}

LPCTSTR CShlItem::GetDisplayName() const
{
	((CShlItem*)this)->GetData(); return m_sDisplayName;
}

LPCTSTR CShlItem::GetTypeName() const
{
	((CShlItem*)this)->GetData(); return m_sType;
}

BOOL CShlItem::Invoke(LPCTSTR pszVerb) const
{
	return InvokeIdList(*(CPidl*)this, pszVerb);
}

//=======================================================================
// CShlFolder
CShlFolder::CShlFolder(CShlRoot* pRoot):
	CShlItem(pRoot), m_dwFlags(NULL)
{
}

CShlFolder::CShlFolder(CShlRoot* pRoot, LPITEMIDLIST pidl):
	CShlItem(pRoot), m_dwFlags(NULL)
{
	Attach(pidl);
}

CShlFolder::CShlFolder(CShlRoot* pRoot, int nSpecialFolder):
	CShlItem(pRoot), m_dwFlags(NULL)
{
	Attach(nSpecialFolder);
}

CShlFolder::~CShlFolder()
{
	Flush();
}

void CShlFolder::GetData()
{
	if( m_sDisplayName.IsEmpty() || !m_hIcon )
		GetShellItemData(*(CPidl*)this,
			(m_dwFlags&stickyName) && (!m_sDisplayName.IsEmpty())?NULL:&m_sDisplayName,
			(m_dwFlags&stickyIcon) && m_hIcon?NULL:&m_hIcon,
			(m_dwFlags&stickyKey)  && m_wHotKey?NULL:&m_wHotKey,
			&m_sType);
}

void CShlFolder::SetIcon(HICON hNewIcon)
{
	if( m_hIcon ) ::DestroyIcon(m_hIcon);
	m_hIcon = hNewIcon;
	if( m_hIcon ) m_dwFlags |= stickyIcon;
	else m_dwFlags &= ~stickyIcon;
}

void CShlFolder::SetDisplayName(LPCTSTR pszNewName)
{
	m_sDisplayName = pszNewName;
	if( m_sDisplayName.IsEmpty() ) m_dwFlags &= ~stickyName;
	else m_dwFlags |= stickyName;
}

void CShlFolder::SetHotKey(WORD wHotKey)
{
	m_wHotKey = wHotKey;
	if( m_wHotKey ) m_dwFlags |= stickyKey;
	else m_dwFlags &= ~stickyKey;
}

void CShlFolder::Flush()
{ 
	m_Notify.Destroy();
	
	if( !(m_dwFlags&stickyName) )
		m_sDisplayName.Empty();
	if( !(m_dwFlags&stickyIcon) && m_hIcon ) {
		::DestroyIcon(m_hIcon);
		m_hIcon = NULL;
	}
	if( !(m_dwFlags&stickyKey) )
		m_wHotKey = 0;
	this->RemoveAll();
}

void CShlFolder::Attach(LPITEMIDLIST pidl)
{
	Flush();
	((CPidl*)this)->operator=(pidl);
	m_dwFlags &= ~errNotifyError;

	if( !pidl->mkid.cb ) {
	//  epmty pidl - it's desktop folder
	//	we have problems accessing desktop folder by its pidl
		m_pFolder = GetDesktopFolder();
	} else {
		GetDesktopFolder()->BindToObject(pidl, NULL,
			__uuidof(m_pFolder), (void**)&m_pFolder);
	}
}

void CShlFolder::Attach(int nSpecialFolder)
{
	CPidl pidl(nSpecialFolder);
	if( !pidl.IsEmpty() )
		Attach(pidl.Detach());
}

BOOL CShlFolder::Refresh()
{       
	HRESULT hr;

	if( !(bool)::GetDesktopFolder() )
		return FALSE;

	if( !(bool)::GetMalloc() )
		return FALSE;

	if( !(bool)m_pFolder )
		return FALSE;
		
//	check desktop folder for changes and flush it.
	if( m_Notify.Changed() )
		Flush();

//	Create notify handler if not yet created.
	if( (HANDLE)m_Notify == NULL &&
		(m_dwFlags & errNotifyError) == 0 )
	{
		TCHAR szPath[_MAX_PATH];
		if( ::SHGetPathFromIDList(*(CPidl*)this, szPath) )
			m_Notify.Create(szPath,	m_Notify.FileName|m_Notify.DirName|m_Notify.Size|m_Notify.Write);
		if( (HANDLE)m_Notify == NULL )
			m_dwFlags |= errNotifyError;
	}

//	cache is empty - load it with stuff
	if( !this->GetSize() ) {
		BOOL bExpandFolders = GetRoot()->GetNestLevel() > GetRoot()->GetItemLevel(this);

		IEnumIDListPtr pDeskEnum;
		if( FAILED(hr = m_pFolder->EnumObjects(NULL,
			SHCONTF_NONFOLDERS|SHCONTF_FOLDERS|
			(GetRoot()->GetOptions()&CShlRoot::optShowHidden?SHCONTF_INCLUDEHIDDEN:0),
			&pDeskEnum)) )
		{
			TRACE("CShellItemCache::Refresh(): EnumObjects failed %X\n",hr);
			return FALSE;
		}

		CPidl pidl;
		while( pidl.Free(),
			   SUCCEEDED(hr = pDeskEnum->Next(1, pidl, NULL)) &&
			   hr != S_FALSE)
		{
			pidl = ::MergeItemID( *(CPidl*)this, (LPCITEMIDLIST)pidl, NULL);

			if( GetRoot()->FileMatch(pidl) == FALSE )
				continue; //item mask doesn't match this item

			CShlItem shlItem(GetRoot(), ::CopyItemID(pidl) );
			if( bExpandFolders &&
				::IsFolder(pidl, GetRoot()->GetOptions()) )
			{
				CShlFolder *fldr;
				Add(fldr = new CShlFolder(GetRoot(), pidl.Detach()));
				fldr->SetDisplayName(shlItem.GetDisplayName() );
				fldr->SetIcon( ::CopyIcon( shlItem.GetIcon() ) );
				fldr->SetHotKey( shlItem.GetHotKey() );
				fldr->m_sType = shlItem.m_sType;
				fldr->m_nMenuID = GetRoot()->GetNextMenuID();
			} else
			if( !pidl.IsEmpty() )
			{
				CShlItem *item;
				Add(item = new CShlItem(GetRoot(), pidl.Detach()));
				item->m_nMenuID = GetRoot()->GetNextMenuID();
			}
		}
		this->Sort();
	}
	return this->GetSize() > 0;
}

//=======================================================================
// CShlFolder
UINT CShlRoot::defFirstMenuID=1;
CShlRoot::CShlRoot():
	CShlFolder(NULL)
{
	m_pRoot = this;
	SetFirstMenuID();
	SetOptions();
	SetNestLevel();
}

CShlRoot::CShlRoot(LPITEMIDLIST pidl):
	CShlFolder(NULL, pidl)
{
	m_pRoot = this;
	SetFirstMenuID();
	SetOptions();
	SetNestLevel();
}

CShlRoot::CShlRoot(int nSpecialFolder):
	CShlFolder(NULL, nSpecialFolder)
{
	m_pRoot = this;
	SetFirstMenuID();
	SetOptions();
	SetNestLevel();
}

static
UINT GetLevel(CShlFolder* fldr, CShlItem* item, UINT prevLevel)
{
	if( item == fldr )
		return prevLevel;

	for(int i=0;i<fldr->GetSize();i++) {
		if( (*fldr)[i] == item )
			return prevLevel + 1;
		if( (*fldr)[i]->IsFolder() ) {
			int level = GetLevel((CShlFolder*)(*fldr)[i], item, prevLevel+1);
			if( level ) return level;
		}
	}
	return 0;
}

UINT CShlRoot::GetItemLevel(CShlItem* item)
{
	return GetLevel(this, item, 1);
}

static
CShlItem* GetItemByID(const CShlFolder* fldr, UINT nMenuID)
{
	if( fldr->GetMenuID() == nMenuID )
		return (CShlItem*)fldr;

	for(int i=0;i<fldr->GetSize();i++) {
		if( (*fldr)[i]->GetMenuID() == nMenuID )
			return (*fldr)[i];
		if( (*fldr)[i]->IsFolder() ) {
			CShlItem* item = GetItemByID((CShlFolder*)(*fldr)[i], nMenuID);
			if( item ) return item;
		}
	}
	return NULL;
}

CShlItem* CShlRoot::GetItemByID(UINT nMenuID) const
{
	return ::GetItemByID(this, nMenuID);
}

void CShlRoot::SetFileMask(LPCTSTR pszMask)
{
	Flush();
	m_aFileMask.RemoveAll();
	LPCTSTR pszCur = pszMask, pszNext;
	CString sMask;
	while(1) {
		sMask.Empty();

		while(*pszCur == ' ') pszCur++; //trim leading space
		if( ! *pszCur ) break;
		
		pszNext = pszCur;
		do{//find next space (start of next mask)
			pszNext = _tcschr(pszNext, ' ');
			if( !pszNext )
				break;
			LPCTSTR pszSlash = pszNext-1;
			if( *pszSlash == '\\' ) {
				while( *pszSlash == '\\' && pszSlash >= pszCur )
					pszSlash--; pszSlash++;
				if( ((pszNext-pszSlash)%2) == 0 )
					break;
				sMask += CString(pszCur, pszSlash - pszCur);
				sMask += CString(pszSlash+1,pszNext-pszSlash);
				pszCur = pszNext+1;
			} else
				break;
		} while( pszNext++ );
		if( !pszNext ) pszNext = pszCur + _tcslen(pszCur);
		
		m_aFileMask.Add(sMask + CString(pszCur, pszNext-pszCur));
		pszCur = pszNext;
	}
	m_sFilemask = pszMask;
}

BOOL CShlRoot::FileMatch(LPCITEMIDLIST pidl)
{
	return FnMatch(pidl, m_aFileMask);
}

int CShlRoot::CompareItems(CShlItem* item1, CShlItem* item2)
{
	int nResult;
	if( (GetOptions() & optFoldersOnTop) &&
		item1->IsFolder() != item2->IsFolder() )
		return item1->IsFolder()?-1:1;
	if( GetOptions() & optSortByType ) {
		nResult = _tcsicmp(item1->GetTypeName(), item2->GetTypeName());
		if( nResult ) return nResult;
	}
	return _tcsicmp(item1->GetDisplayName(), item2->GetDisplayName());
}

void CShlRoot::RegSave(LPCTSTR pszSection) const
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);
	HKEY hKey = pApp->GetSectionKey(_T(""));
	pApp->DelRegTree(hKey, pszSection);
	RegCloseKey(hKey);

	pApp->WriteProfileBinary(pszSection, _T("ItemIDList"),
		(LPBYTE)(LPITEMIDLIST)(CPidl)(*this), GetItemIDSize((CPidl)(*this))+sizeof(USHORT));
	pApp->WriteProfileInt(pszSection, _T("Options"), GetOptions());
	pApp->WriteProfileInt(pszSection, _T("NestLevel"), GetNestLevel());
	pApp->WriteProfileString(pszSection, _T("FileMask"), GetFileMask());
	if( m_dwFlags & stickyName ) pApp->WriteProfileString(pszSection, _T("DisplayName"), GetDisplayName());
	if( m_dwFlags & stickyKey ) pApp->WriteProfileInt(pszSection, _T("HotKey"), GetHotKey());
}

BOOL CShlRoot::RegRead(LPCTSTR pszSection)
{
	CWinApp *pApp = AfxGetApp(); ASSERT(pApp);

	LPBYTE pPidl; UINT nSize;
	if( pApp->GetProfileBinary(pszSection, _T("ItemIDList"), &pPidl, &nSize) ) {
		Attach(CopyItemID((LPITEMIDLIST)pPidl)); delete pPidl;
		if( ((CPidl*)this)->IsEmpty() ) return FALSE;
	} else
		return FALSE;
	SetOptions(pApp->GetProfileInt(pszSection, _T("Options"), defOptions));
	SetNestLevel(pApp->GetProfileInt(pszSection, _T("NestLevel"), defNestLevel));
	SetFileMask(pApp->GetProfileString(pszSection, _T("FileMask"), NULL));
	SetDisplayName(pApp->GetProfileString(pszSection, _T("DisplayName"), NULL));
	SetHotKey(pApp->GetProfileInt(pszSection, _T("HotKey"), NULL));
	return TRUE;
}

CShlRoot& CShlRoot::operator =(const CShlRoot& x)
{
	Flush();
	this->Attach( ((CPidl&)x).Copy() );
	if( x.m_dwFlags && stickyName) this->SetDisplayName(x.GetDisplayName());
	if( x.m_dwFlags && stickyKey)  this->SetHotKey(x.GetHotKey());
	if( x.m_dwFlags && stickyIcon) this->SetIcon(::CopyIcon(x.GetIcon()));
	this->SetOptions(x.GetOptions());
	this->SetNestLevel(x.GetNestLevel());
	this->SetFileMask(x.GetFileMask());
	return *this;
}
