/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "globals.h"
#include "shlmenu.h"

/******************************************************************************
CShlMenuCmdTarget
******************************************************************************/
class CShlMenuCmdTarget:
	public CWnd
{
	static LPCTSTR	m_pszWndClassName;
	static BOOL	m_bWndClassRegistered;
public:
	CShlMenuCmdTarget();
	BOOL PreCreateWindow(CREATESTRUCT &cs);
	BOOL Create(CShlMenu* pMenu, CWnd* pParentWnd=NULL);
protected:
	CShlMenu	*m_pMenu;
protected:
	//{{AFX_MSG(CShlMenuCmdTarget)
	afx_msg void OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu);
	//}}AFX_MSG
	afx_msg void OnMenuRButtonUp(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()
};
LPCTSTR	CShlMenuCmdTarget::m_pszWndClassName = _T("CShlMenuCmdTarget-WndClass");
BOOL CShlMenuCmdTarget::m_bWndClassRegistered = FALSE;

BEGIN_MESSAGE_MAP(CShlMenuCmdTarget, CWnd)
	//{{AFX_MSG_MAP(CShlMenuCmdTarget)
	ON_WM_INITMENUPOPUP()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_MENURBUTTONUP, OnMenuRButtonUp)
END_MESSAGE_MAP()

CShlMenuCmdTarget::CShlMenuCmdTarget():
	m_pMenu(NULL)
{
}

BOOL CShlMenuCmdTarget::PreCreateWindow(CREATESTRUCT &cs)
{
	if( !m_bWndClassRegistered ) {
		INIT<WNDCLASS> cls;
		cls.lpszClassName = m_pszWndClassName;
		cls.lpfnWndProc = AfxWndProc;
		cls.hInstance = ::AfxGetInstanceHandle();
		cls.hbrBackground = (HBRUSH)COLOR_WINDOW;
		cls.hIcon = ::PxLoadIcon(NULL, IDI_APPLICATION);
		cls.hCursor = ::LoadCursor(NULL, IDC_ARROW);
		if( !(m_bWndClassRegistered=AfxRegisterClass(&cls)) )
			return FALSE;
	}
	cs.lpszClass = m_pszWndClassName;
	return TRUE;
}

BOOL CShlMenuCmdTarget::Create(CShlMenu* pMenu, CWnd* pParentWnd/*=NULL*/)
{
	BOOL bResult = CWnd::CreateEx(NULL,m_pszWndClassName,NULL,
		NULL, CRect(), pParentWnd?pParentWnd:AfxGetMainWnd(), NULL);
	if( bResult ) m_pMenu = pMenu;
	return TRUE;
}

void CShlMenuCmdTarget::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, BOOL bSysMenu) 
{
	if( bSysMenu ||
		m_pMenu == NULL ||
		pPopupMenu  == NULL ||
		!::IsMenu(pPopupMenu->GetSafeHmenu()))
		return;

	UINT nMenuID = pPopupMenu->GetMenuItemID(0);
	if( pPopupMenu->GetMenuItemCount() == 1 && nMenuID > 0)
	{
		CString str;
		pPopupMenu->GetMenuString(0, str, MF_BYPOSITION);
		if( str != "*" ) return;
		pPopupMenu->DeleteMenu(0, MF_BYPOSITION);
		CShlFolder *pFldr = (CShlFolder*)m_pMenu->GetItemByID(nMenuID);
		if( pFldr ) m_pMenu->BuildSubMenu(*pFldr, (CMenu2&)*pPopupMenu);
	}
}

void CShlMenuCmdTarget::OnMenuRButtonUp(WPARAM wParam, LPARAM lParam)
{
/*	CMenu2 *pSrcMenu = (CMenu2*)CMenu2::FromHandle((HMENU)lParam);
	CMenu2 pMenu;
	pMenu.CreatePopupMenu();
	pMenu.AppendMenu(MF_STRING, 1, "One");
	pMenu.AppendMenu(MF_STRING, 2, "Two");
	pMenu.AppendMenu(MF_STRING, 3, "Three");
	CPoint p; GetCursorPos(&p);
	::TrackPopupMenuEx(pMenu.GetSafeHmenu(), MenuFlags|TPM_RECURSE|TPM_NONOTIFY, p.x, p.y,
		GetSafeHwnd(), NULL);
*/}

/******************************************************************************
CMenu2
******************************************************************************/
CString CMenu2::GetMenuString(UINT nIDItem, BOOL bByPosition/*=FALSE*/)
{
	CString str;
	CMenu::GetMenuString(nIDItem, str, bByPosition?MF_BYPOSITION:MF_BYCOMMAND);
	return str;
}

BOOL CMenu2::SetMenuString(UINT nIDItem, LPCTSTR pszStr, BOOL bByPosition/*=FALSE*/)
{
	INIT2<MENUITEMINFO> mii;
	mii.fMask = MIIM_TYPE;
	if( !GetMenuItemInfo(nIDItem, &mii, bByPosition) )
		return FALSE;
	if( !mii.fType & MFT_STRING )
		return FALSE;
	mii.dwTypeData = (LPTSTR)pszStr;
	return ::SetMenuItemInfo(GetSafeHmenu(), nIDItem, bByPosition, &mii);
}

BOOL CMenu2::AppendHotKey(UINT nIDItem, DWORD wHotKey, BOOL bByPosition/*=FALSE*/)
{
	int pos;
	CString str;
	if( !wHotKey )
		return FALSE;
	if( !CMenu::GetMenuString(nIDItem, str, bByPosition) )
		return FALSE;
	if( (pos = str.Find('\t')) >= 0 )
		str.Delete(pos);
	str += '\t' + ::GetKeyName( wHotKey );
	if( str[str.GetLength()-1] == '\t') 
		return FALSE;
	return SetMenuString(nIDItem, str, bByPosition);
}

CMenu2* CMenu2::AppendSubMenu(LPCTSTR pszName, LPCTSTR pszRsrcMenu)
{
	HMENU hSubMenu = ::LoadMenu(::AfxGetInstanceHandle(), pszRsrcMenu);
	if( hSubMenu ) {
		this->AppendMenu(MF_STRING|MF_POPUP,(UINT)hSubMenu, pszName);
		return (CMenu2*)CMenu::FromHandle(hSubMenu);
	}
	return NULL;
}

BOOL CMenu2::SetMenuItemID(int nPos, WORD wID)
{
	MENUITEMINFO info;
	info.cbSize = sizeof(info);
	info.fMask  = MIIM_ID;
	info.wID    = wID;
	return ::SetMenuItemInfo(this->GetSafeHmenu(), nPos, TRUE, &info);
}

static
BOOL AddAmpersand(CString& in, CString& used)
{
	int pos, hasalpha;
	if( (pos = in.Find('&')) >= 0 ) {
	// no need to add ampersand here
		if( ++pos >= in.GetLength() )
			return FALSE;
	//	add amersand char to "used" string
		if( _istalnum( in[pos] ) &&
			used.Find(_totupper(in[pos])) < 0 )
		{
			used += _totupper(in[pos]);
		}
		return FALSE;
	}

	pos = -1; hasalpha = 0;
	for(int i=0; i < in.GetLength() && in[i] != '\t'; i++)
	{
		TCHAR ch = _totupper(in[i]);
		if( !_istalnum(ch) ) 
			continue;
		hasalpha=1;
		if( used.Find(ch) < 0 ) {
			pos = i;
			break;
		}
	}
	if( pos >= 0 ) {
		used += _totupper(in[pos]);
		in.Insert(pos, '&');
	} else
	if( hasalpha ) {
	//  all the chars are used up. start new cycle
		used.Empty();
		return AddAmpersand(in, used);
	}
	return TRUE;
}

void CMenu2::AddAmpersands()
{
	CString sUsed;
	for(UINT i=0; i<this->GetMenuItemCount(); i++ )
	{
		CString str;
		if( !CMenu::GetMenuString(i, str, MF_BYPOSITION) )
			continue;
		if( AddAmpersand(str, sUsed) )
			SetMenuString(i, str, TRUE);
	}
}

/******************************************************************************
CShlMenuCmdTarget
******************************************************************************/
CShlMenu::CShlMenu():
	m_nTruncate(25),
	m_idRefresh(0x10000),
	m_sOpenFolder(_T("Open %s")),
	m_pShlRoot(NULL)
{
}

UINT CShlMenu::TrackPopupMenu(CShlRoot& rShlCache, UINT nFlags, int x, int y, BuildCallback cb/*=NULL*/)
{
	UINT nResult=0;
	CShlMenuCmdTarget pTarget;
	m_pBuildCallback = cb;
	if( !pTarget.Create(this) ) return 0;
	this->Install(&pTarget);
	this->m_pShlRoot = &rShlCache;

TRACKAGAIN:
	if( this->GetSafeHmenu() ) this->DestroyMenu();
	if( !this->CreatePopupMenu() ) return 0;
	this->AppendMenu(MF_STRING|MF_GRAYED,rShlCache.GetMenuID(), "*");
	this->SetItemImage(rShlCache.GetMenuID(),rShlCache.GetIcon());

	nResult = CMenu2::TrackPopupMenu(nFlags, x, y, &pTarget);
	int err = GetLastError();
	if( m_idRefresh && nResult == m_idRefresh ) {
		this->Refresh();
		rShlCache.Flush();
		goto TRACKAGAIN;
	}
	
	m_pShlRoot = NULL;
	return nResult;
}

CString CShlMenu::PrepareMenuString(CShlItem* shlItem)
{
	CString sItemName = shlItem->GetDisplayName();
	if( m_nTruncate && sItemName.GetLength() > m_nTruncate )
		sItemName = sItemName.Mid(0, m_nTruncate) + "...";
	if( !shlItem->IsFolder() && shlItem->GetHotKey() )
		sItemName += '\t' + GetKeyName(shlItem->GetHotKey());
	return sItemName;
}

void CShlMenu::BuildSubMenu(CShlFolder& rFolder, CMenu2& rSubMenu)
{
	CString sItemName;
	CWaitCursor wait;
	rFolder.Refresh();
	if( !m_sOpenFolder.IsEmpty() ) {
		sItemName = StrPrintf(m_sOpenFolder, rFolder.GetDisplayName());
		if( rFolder.GetHotKey() ) sItemName += '\t' + GetKeyName(rFolder.GetHotKey());
		rSubMenu.AppendMenu(MF_STRING, rFolder.GetMenuID(), sItemName);
	}
	if( m_idRefresh && (HANDLE)rFolder.GetNotify() == NULL ) {
		sItemName = ::LoadString(m_idRefresh);
		if( sItemName.IsEmpty() ) sItemName = _T("&Refresh");
		rSubMenu.AppendMenu(MF_STRING, m_idRefresh, sItemName);
	}
	if( m_pBuildCallback ) m_pBuildCallback(*this, rFolder, rSubMenu);
	if( rSubMenu.GetMenuItemCount() && rFolder.GetSize() )
		rSubMenu.AppendMenu(MF_SEPARATOR);

	// compute how many columns we need in menu
	UINT nMenuItemHeight = max(::GetSystemMetrics(SM_CYMENU), ::GetSystemMetrics(SM_CYSMICON)+4);
	UINT nMenuCount = rFolder.GetSize()+rSubMenu.GetMenuItemCount();
	UINT nStep = (nMenuItemHeight*nMenuCount) /
		((UINT)::GetSystemMetrics(SM_CYSCREEN)-4) + 1;
	if( nMenuCount % nStep ) nStep = nMenuCount / nStep + 1;
	else nStep = nMenuCount / nStep;

 	// stuff menu with data
	for(int i=0,n=rSubMenu.GetMenuItemCount(); i < rFolder.GetSize(); i++,n++) {
		UINT fFlags = 0;
		
		if( nStep > 1 && n > 0 && (n % nStep) == 0 )
			fFlags |= MF_MENUBARBREAK;

		sItemName = PrepareMenuString(rFolder[i]);
		if( rFolder[i]->IsFolder() ) {
			CMenu2 submnu;
			if( !submnu.CreatePopupMenu() )
				continue;
			submnu.AppendMenu(MF_STRING|MF_GRAYED,rFolder[i]->GetMenuID(), "*");
			rSubMenu.AppendMenu(MF_POPUP|fFlags,
				(UINT)submnu.Detach(),
				sItemName);
			rSubMenu.SetMenuItemID(rSubMenu.GetMenuItemCount()-1, rFolder[i]->GetMenuID());
		} else
			rSubMenu.AppendMenu(MF_STRING|fFlags,
				rFolder[i]->GetMenuID(), sItemName);
		this->SetItemImage(rFolder[i]->GetMenuID(), rFolder[i]->GetIcon());
	}
	rSubMenu.AddAmpersands();
}
