/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __SHLMENU_H_INCLUDED__
#define __SHLMENU_H_INCLUDED__
#include "shlcache.h"

class CMenu2: 
	public CMenu
{
public:
	CMenu2(): CMenu()
		{}
public:
	CString GetMenuString(UINT nIDItem, BOOL bByPosition=FALSE);
	BOOL SetMenuString(UINT nIDItem, LPCTSTR pszStr, BOOL bByPosition=FALSE);
	BOOL AppendHotKey(UINT nIDItem, DWORD wHotKey, BOOL bByPosition=FALSE);
	CMenu2* AppendSubMenu(LPCTSTR pszName, LPCTSTR pszRsrcMenu);
	CMenu2* AppendSubMenu(LPCTSTR pszName, UINT nRsrcMenu)
		{ return AppendSubMenu(pszName, MAKEINTRESOURCE(nRsrcMenu));}
	CMenu2* AppendSubMenu(UINT nRsrcMenu)
		{ return AppendSubMenu(LoadString(nRsrcMenu), nRsrcMenu);}
	BOOL SetMenuItemID(int nPos, WORD wID);
	void AddAmpersands();
};

class CShlMenu:
	public CMenu2,
	public CPxMenuManager
{public:
	friend class CShlMenuCmdTarget;
	typedef void (CALLBACK *BuildCallback)(CShlMenu& rRootMenu, CShlFolder& rFolder, CMenu2& rSubMenu);
protected:
	BuildCallback	m_pBuildCallback;
	int				m_nTruncate;	//truncate menu item names to position
	UINT			m_idRefresh;	//id of "REFRESH" menu item
	CString			m_sOpenFolder;  //menu string for "Open folder" menu item
	CShlRoot*		m_pShlRoot;

	void BuildSubMenu(CShlFolder& rFolder, CMenu2& rSubMenu);
	CString PrepareMenuString(CShlItem* shlItem);
public:
	CShlMenu();
    void* operator new(size_t nSize)
        { return CMenu::operator new(nSize); }
    void operator delete(void* p)
        { CMenu::operator delete(p); }
public:
	UINT TrackPopupMenu(CShlRoot& rShlRoot, UINT nFlags, int x, int y, BuildCallback cb=NULL);
	CShlItem* GetItemByID(UINT nMenuID)
		{ return m_pShlRoot?m_pShlRoot->GetItemByID(nMenuID):0; }
//	sets ID of "Refresh" menu item
//	it's neccessary to have string resource for new ID
	void SetRefreshID(UINT nID)
		{ m_idRefresh = nID; }
	void SetTruncate(int n)
		{ m_nTruncate = n; }
	void SetOpenFolderString(LPCTSTR pszString)
		{ m_sOpenFolder = pszString; }
	void SetOpenFolderString(UINT nResourceID)
		{ m_sOpenFolder.LoadString(nResourceID); }
};

#endif //__SHLMENU_H_INCLUDED__
