/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "stdafx.h"
#include "GLOBALS.H"
#include "RULE.H"
#include "WNDLIST.H"
#include "WCAT.H"
#include "PPMAIN.H"
#include "PASSWORD.H"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWCatApp
CWCatApp theApp;
BEGIN_MESSAGE_MAP(CWCatApp, CWinApp)
	//{{AFX_MSG_MAP(CWCatApp)
	//}}AFX_MSG
END_MESSAGE_MAP()

static LCID langs[] = {
	LOCALE_USER_DEFAULT,
	MAKELCID(MAKELANGID(LANG_ENGLISH,SUBLANG_ENGLISH_US),SORT_DEFAULT),
	MAKELCID(MAKELANGID(LANG_RUSSIAN,SUBLANG_DEFAULT),SORT_DEFAULT),
	MAKELCID(MAKELANGID(LANG_CZECH,SUBLANG_DEFAULT),SORT_DEFAULT)};

void CWCatApp::SetLanguage(UINT nLang)
{
	if( nLang > countof(langs) )
		return;
	SetThreadLocale(langs[nLang]);
	VOID (WINAPI *IML)(LANGID) = 
		(VOID (WINAPI *)(LANGID))::GetProcAddress(
			::GetModuleHandle(_T("COMCTL32.DLL")), "InitMUILanguage");
	if( IML ) IML(LANGIDFROMLCID(langs[nLang]));
}

UINT CWCatApp::GetLanguage()
{
	LCID locale = GetThreadLocale();
	for(int i=0; i<countof(langs); i++)
		if( locale == langs[i] )
			return i;
	return 0;
}

BOOL CWCatApp::InitInstance()
{
#ifdef _DEBUG
	SetRegistryKey(_T("FREEWARE-DEBUG"));
#else  //_DEBUG
	SetRegistryKey(_T("FREEWARE"));
#endif //not _DEBUG
	m_pszMainWndClass = AfxRegisterWndClass(0);
	CShlRoot::defFirstMenuID = ID_SHELL_FIRST;
#ifndef _DEBUG
//  check that we have only one WatchCat instance running
	HWND hWnd = ::FindWindow(m_pszMainWndClass, AfxGetAppName());
	if( hWnd ) {
		SendMessage(hWnd,WM_RESTOREICON,0,0);
		return FALSE;
	}
#endif //_DEBUG
	m_pMainWnd = new CMainWnd();
	if( !m_pMainWnd->CreateEx(WS_EX_TOOLWINDOW,
		m_pszMainWndClass, AfxGetAppName(),
		0, CRect(0,0,0,0), NULL, 0))
	{
		delete m_pMainWnd;
		return FALSE;
	}
	if( GetRegisterService() )
		m_bServiceRegistered = RegisterServiceProcess(NULL,1);
	return TRUE;
}

int CWCatApp::ExitInstance() 
{
	UnregisterService(); //-- unregister application 
	return CWinApp::ExitInstance();
}

BOOL CWCatApp::IsAutoRun()
{
	HKEY hkRun;
	LONG nResult = RegOpenKeyEx(HKEY_CURRENT_USER,
		_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"),
		NULL, KEY_READ, &hkRun);
	if( nResult != ERROR_SUCCESS ) 
		return FALSE;
	nResult = RegQueryValueEx(hkRun, m_pszAppName,
		NULL, NULL, NULL, NULL);
	RegCloseKey(hkRun);
	return nResult == ERROR_SUCCESS;
}

void CWCatApp::SetAutoRun(BOOL bRun)
{
	if( bRun == IsAutoRun() )
		return;

	HKEY hkRun;
	LONG nResult = RegOpenKeyEx(HKEY_CURRENT_USER,
		_T("Software\\Microsoft\\Windows\\CurrentVersion\\Run"),
		NULL, KEY_ALL_ACCESS, &hkRun);
	if( nResult != ERROR_SUCCESS ) 
		return;
	if( bRun ) {
		CString sExeName = GetExeFileName();
		::RegSetValueEx(hkRun,m_pszAppName, NULL, REG_SZ,
			(BYTE*)(LPCTSTR)sExeName, sExeName.GetLength()+sizeof(TCHAR));
	} else {
		::RegDeleteValue(hkRun,m_pszAppName);
	}
	::RegCloseKey(hkRun);
}

CString CWCatApp::GetHelpFileName()
{
	static LPCTSTR langsuff[] = {_T(""),_T(""),_T("RU"),_T("CZ")};
	TCHAR szHelpFName[_MAX_PATH], *pszDot;
	::GetModuleFileName(NULL,szHelpFName,countof(szHelpFName));
	if( !(pszDot = _tcsrchr(szHelpFName,'.')) )
		pszDot = szHelpFName + _tcslen(szHelpFName);
	_tcscpy(pszDot, langsuff[GetLanguage()]);
	_tcscat(pszDot, _T(".HLP"));
	if( _taccess(szHelpFName, 04) == -1 )
		_tcscpy(pszDot,_T(".HLP"));
	if( _taccess(szHelpFName, 04) == -1 )
		return (LPCTSTR)NULL;
	return szHelpFName;
}

CString CWCatApp::GetExeFileName()
{
	CString sExeName;
	GetModuleFileName(NULL, sExeName.GetBuffer(_MAX_PATH),_MAX_PATH);
	sExeName.ReleaseBuffer();
	return sExeName;
}

void CWCatApp::WinHelp(DWORD dwData, UINT nCmd) 
{
	CString& sHelpFile = GetHelpFileName();
	if( sHelpFile.IsEmpty() ) return;
	if( m_pszHelpFilePath ) free( (void *)m_pszHelpFilePath );
	m_pszHelpFilePath = _tcsdup(sHelpFile);
	CWinApp::WinHelp(dwData, nCmd);
}

BOOL CWCatApp::OnIdle(LONG lCount) 
{
	if( m_pMainWnd ) {
		// remove pasword verification flag in idle time
		if( MAINWND->m_nLockUI == 0 )
			MAINWND->m_bPasswordVerified = FALSE;
		// clear "just hidden window" variable
		MAINWND->m_hJustHiddenWnd = NULL;
	}
	return CWinApp::OnIdle(lCount);
}

/////////////////////////////////////////////////////////////////////////////
// MainWnd
CMainWnd::CMainWnd():
	m_TrayIcon(IDR_MAINFRAME)
{
	m_aWindows.RuleSet = &m_aRuleSet;
	m_MouseAction.data[maLeftBtnClick]      = ACTION_SHOWLIST;
	m_MouseAction.data[maRightBtnClick]     = ACTION_SHOWLIST;
	m_MouseAction.data[maLeftBtnDblClick]   = ACTION_SHOWALL;
	m_MouseAction.data[maRightBtnDblClick]  = ACTION_SHOWPROP;

	m_MouseTAction.data[maLeftBtnClick]     = TACTION_RESTORE;
	m_MouseTAction.data[maRightBtnClick]    = TACTION_MENU;
	m_MouseTAction.data[maLeftBtnDblClick]  = TACTION_NOTHING;
	m_MouseTAction.data[maRightBtnDblClick] = TACTION_RESTOREMAX;

	m_sIconFile = theApp.GetExeFileName();
	m_nIconIndex = 0;

	m_nLockUI  = 0;
	m_bTimerLocked = 0;

	m_CwdMenuTemplate.SetDisplayName(_T("Current Directory"));
}

CMainWnd::~CMainWnd()
{
}

BEGIN_MESSAGE_MAP(CMainWnd, CWnd)
	//{{AFX_MSG_MAP(CMainWnd)
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_HELPINFO()
	ON_WM_ENDSESSION()
	//}}AFX_MSG_MAP
	ON_MESSAGE( WM_HOTKEY,			OnHotKey )
	ON_MESSAGE( WM_TRAYICON,		OnTrayIcon )
	ON_MESSAGE( WM_RESTOREICON,		OnRestoreIcon )
	ON_MESSAGE( WM_WNDMINIMIZE,		OnWndMinimize )
	ON_MESSAGE( WM_WNDTRAYICON,		OnWndTrayIcon)
	ON_MESSAGE( WM_SHOWPROPERTIES,	OnProperties)
	ON_MESSAGE( WM_SHELLMENU,		OnShellMenu)
	ON_MESSAGE( WM_FIXDSKTEXT,		OnFixDskText)
	ON_MESSAGE( WM_WNDACTIVATE,		OnWndActivate)
	ON_MESSAGE( WM_SHELLRUN,		OnShellRun)
	ON_MESSAGE( WM_SHELLTRAYICON,	OnShellTrayIcon)
	ON_MESSAGE( WM_CREATETRAYICON,	OnCreateTrayIcon)
	ON_MESSAGE( WM_VERIFYPASSWORD,	OnVerifyPassword)
	ON_MESSAGE( WM_SETJUSTHIDDEN,	OnSetJustHidden)	
	ON_MESSAGE(	WM_GETCONFIG,		OnGetConfig)
END_MESSAGE_MAP()

BOOL CALLBACK RestoreWndTrayIcons(HWND hWnd, LPARAM lParam)
{
	CMainWnd* pMainWnd = (CMainWnd*)lParam;
	try {
		CWndData& wndData = pMainWnd->m_aWindows[hWnd];
		if( !wndData.IsVisible() && 
			wndData.property[wndData.propTray] )
		{
			pMainWnd->OnCreateTrayIcon(NULL, (LPARAM)&wndData);
		}
	} catch( CUserException*) {/*do nothing*/}
	return TRUE;
}

void SOUND(LPCTSTR snd)
{
	PlaySound(NULL,NULL,NULL);
	PlaySound(snd, NULL, SND_ALIAS|SND_ASYNC|SND_NODEFAULT);
}

void WARNSOUND(void)
{
	MessageBeep(MB_ICONWARNING);
}

DWORD RegisterServiceProcess(DWORD dwProcessId, DWORD dwType)
{
	typedef BOOL (FAR WINAPI *REGFUNC)(DWORD, DWORD);
	HINSTANCE hLib = LoadLibrary(_T("KERNEL32.DLL"));
	DWORD dwResult = 0;
	if( hLib ) {
		REGFUNC f = (REGFUNC)GetProcAddress(hLib, "RegisterServiceProcess");
		if(f) dwResult = f(dwProcessId, dwType);
		FreeLibrary(hLib);
	}
	return dwResult;
}

DWORD GetPlatform()
{
	OSVERSIONINFO ov;
	ov.dwOSVersionInfoSize = sizeof(ov);
	GetVersionEx(&ov);
	return ov.dwPlatformId;
}

BOOL IsNewWin()
{
	OSVERSIONINFO ov;
	ov.dwOSVersionInfoSize = sizeof(ov);
	GetVersionEx(&ov);
	return
	//	Windows 98
		(ov.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS &&
		 ov.dwMinorVersion > 0) ||
	//	Windows 2000
		(ov.dwPlatformId == VER_PLATFORM_WIN32_NT &&
		 ov.dwMajorVersion >= 5);
}

void CShlRootArray::Save(LPCTSTR pszSection)
{
	HKEY hKey;
	if( hKey = theApp.GetSectionKey("") ) { 
		theApp.DelRegTree(hKey, pszSection);
		::RegCloseKey(hKey);
	}
	for(int i=0; i<GetSize(); i++)
		(*this)[i]->RegSave(::StrPrintf("%s\\Folder-%d",pszSection,i+1));
	theApp.WriteProfileInt(pszSection, _T("FolderCount"), GetSize());
}

void CShlRootArray::Restore(LPCTSTR pszSection)
{
	RemoveAll();
	for(int i=1; ; i++) {
		CShlRoot* newRoot;
		Add(newRoot = new CShlRoot() );
		if( !newRoot ||
			!newRoot->RegRead(::StrPrintf("%s\\Folder-%d",pszSection,i)) ) 
		{
			RemoveAt(GetSize()-1);
			break;
		}
	}
	if( GetSize() == 0 &&
		theApp.GetProfileInt(pszSection, _T("FolderCount"), 0xDEADBEEF) == 0xDEADBEEF)
	{
	//  it looks like first run - stuff predefined folders
		Add(new CShlRoot(CSIDL_DESKTOP));
		Add(new CShlRoot(CSIDL_DRIVES)); 
		Add(new CShlRoot(CSIDL_NETWORK)); 
		Add(new CShlRoot(CSIDL_CONTROLS));
	}
}

int	 CShlRootArray::FindPidl(LPCITEMIDLIST pidl)
{
	for(int i=0; i<GetSize(); i++) {
		CPidl *t = (CPidl*)((*this)[i]);
		if( CompareItemID(*t, pidl) == 0 )
			return i;
	}
	return -1;
}

ModifierKeys GetModifierKey()
{
	BOOL bShift = GetAsyncKeyState(VK_SHIFT)  & 0x8000;
	BOOL bAlt   = GetAsyncKeyState(VK_MENU)   & 0x8000;
	BOOL bCtrl  = GetAsyncKeyState(VK_CONTROL)& 0x8000;
	if( bCtrl && bAlt && bShift )return kmCtrlAltShift;	else
	if( bAlt && bShift )		return kmAltShift;		else
	if( bCtrl && bShift )		return kmCtrlShift;		else
	if( bCtrl && bAlt )			return kmCtrlAlt;		else
	if( bCtrl)					return kmCtrl;			else
	if( bAlt )					return kmAlt;			else
	if( bShift )				return kmShift;			else
	return kmNone;
}

int CMainWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1) return -1;
#ifdef _DEBUG
	SetWindowText(_T("Debug WatchCat"));
	SetIcon(LoadIcon(0, IDI_APPLICATION), TRUE);
#ifdef _TEST_PROPERTIES
//	for properties window debugging 
	ReadConfig();
	SendMessage(WM_SHOWPROPERTIES);	
	return -1;
#endif //_TEST_PROPERTIES
#else  //_DEBUG
	SetIcon(PxLoadIcon(IDR_MAINFRAME), TRUE);
	SetIcon(PxLoadIconSm(IDR_MAINFRAME), FALSE);
#endif //not _DEBUG
	ReadConfig();
	m_Manager.Install(this);
	m_Manager.LoadToolbar(IDR_OVERLAY);
	m_Manager.LoadToolbar(IDR_MAINFRAME);
	m_Manager.LoadToolbar(IDR_SHELL);
	
	m_TrayIcon.SetNotificationWnd(this, WM_TRAYICON);
#ifndef _DEBUG
//  usually debug version of WatchCat runs concurrently
//	with release version, so don't restore tray icons
//	for windows hidden to tray (otherwise they will duplicate)
	::EnumWindows(::RestoreWndTrayIcons, (LPARAM)this);
#endif	//_DEBUG
	PostMessage(WM_RESTOREICON,NULL,NULL);
	PostMessage(WM_FIXDSKTEXT,NULL,NULL);

	SetTimer(1,TIMESLICE,NULL);
	m_Hook.SetHook(GetSafeHwnd());
	return 0;
}

void CMainWnd::OnEndSession(BOOL bEnding) 
{// Windows is going down...
	CWnd::OnEndSession(bEnding);
	if( bEnding ) {
	//	unregister service in order to prevent WatchCat to survive logoff
		theApp.UnregisterService();
	//	eject CD if user wants so.
		if( m_bEjectCDAtExit ) {
			TCHAR szRet[255];
			mciSendString(_T("status cdaudio ready wait"),
				szRet, countof(szRet), NULL);
			if( _tcsicmp(szRet, "TRUE") == 0 )
				mciSendString(_T("set cdaudio door open"),
					szRet, countof(szRet), NULL);
		}
	}
}

void CMainWnd::OnDestroy() 
{
//-- kill WatchCat timer
	KillTimer(1);
//-- unbind all hot keys
	SetHotKeys(FALSE);
//-- tell help file that we don't need it any more
	theApp.WinHelp(0, HELP_QUIT);
//-- restore taskbar - because without it life becomes really silly
	if( !CWndData::GetTaskbarWnd().IsVisible() )
		CWndData::GetTaskbarWnd().Show(TRUE);
	CWnd::OnDestroy();
}

void CMainWnd::PostNcDestroy() 
{
	if( m_bUseRestore ) m_aWindows.ShowAll();
	delete this;
}

void CMainWnd::OnTrayIcon(WPARAM wParam, LPARAM lParam)
{
	if( wParam != IDR_MAINFRAME ) return;
	if(!((UINT)lParam == WM_LBUTTONUP ||
	     (UINT)lParam == WM_LBUTTONDBLCLK ||
	     (UINT)lParam == WM_RBUTTONUP ||
	     (UINT)lParam == WM_RBUTTONDBLCLK ||
	     (UINT)lParam == WM_MBUTTONUP ||
	     (UINT)lParam == WM_MBUTTONDBLCLK) )
		 return;

	if( m_Hook.IsHookSet() != GetSafeHwnd())
		m_Hook.SetHook(GetSafeHwnd());

	CPoint p; GetCursorPos(&p);
	switch( m_MouseAction.GetAction((UINT)lParam, ::GetModifierKey()) ) {
	case ACTION_SHOWLIST:
		TrackWndMenu(p);
		break;
	case ACTION_SHOWPROP:
		PostMessage(WM_CANCELMODE,NULL,NULL);
		PostMessage(WM_NULL,NULL,NULL);
		PostMessage(WM_SHOWPROPERTIES,NULL,NULL);
		break;
	case ACTION_SHOWALL:
		m_aWindows.ShowAll();
		break;
	case ACTION_HIDEALL:
		m_aWindows.HideAll();
		break;
	case ACTION_TOGGLEICONS: {
		CWndData& wndData = CWndData::GetDesktopWnd();
		wndData.Show( !wndData.IsVisible() ); }
	default:
		break;
	}
	CWndData::Compact(m_hWnd);
}

void CMainWnd::OnRestoreIcon(WPARAM wParam, LPARAM lParam)
{
	int i;

	m_TrayIcon.SetIcon(NULL);
	if( m_bEnableIcon ) {
		BOOL bResult = m_TrayIcon.SetIconFromFile(
			m_sIconFile, m_nIconIndex, AfxGetAppName());
	//	if icon cannot be set from file, set default WatchCat icon.
		if( !bResult ) m_TrayIcon.SetIcon();
	}
//--restore icons for shell folders
	if( m_aShellTrayIcons.GetSize() == 0 )
		for( i=0; i < m_aShellMenus.GetSize(); i++ )
		{
			if( !(m_aShellMenus[i]->GetOptions() & CShlRoot::optTrayIcon) )
				continue;

			CPxTrayIcon *pTmpIcon = new CPxTrayIcon( (UINT)i );
			pTmpIcon->SetNotificationWnd(this, WM_SHELLTRAYICON);
			HICON hIcon = m_aShellMenus[i]->GetIcon();
			CString sTip = m_aShellMenus[i]->GetDisplayName();
			if( m_aShellMenus[i]->GetHotKey() )
				sTip += _T(" (") + ::GetKeyName(m_aShellMenus[i]->GetHotKey()) + ')';
			if( hIcon && pTmpIcon->SetIcon(hIcon, sTip) )
				m_aShellTrayIcons.Add(pTmpIcon);
			else
				delete pTmpIcon;
		}
	else
		for( i=0; i<m_aShellTrayIcons.GetSize(); i++ )
			 m_aShellTrayIcons[i]->Refresh();
//--restore icons for windows hidden to tray
	for( i=0; i<m_aWndTrayIcons.GetSize(); i++ )
		 m_aWndTrayIcons[i]->Refresh();
}

CPoint CMainWnd::GetTrayCoords()
{
	CPoint p;
	HWND hWnd = ::FindWindowEx(CWndData::GetTaskbarWnd(),
		NULL, "TrayNotifyWnd", NULL);
	if( hWnd ) {
		CRect rc; ::GetWindowRect(hWnd, rc);
		p.x = min(GetSystemMetrics(SM_CXSCREEN), max(rc.right,0));
		p.y = min(GetSystemMetrics(SM_CYSCREEN), max(rc.bottom,0));
	} else {
		p.x = GetSystemMetrics(SM_CXSCREEN);
		p.y = GetSystemMetrics(SM_CYSCREEN);
	}
	return p;
}

void CMainWnd::OnHotKey(WPARAM wParam, LPARAM lParam)
{
	if( m_Hook.IsHookSet() != GetSafeHwnd())
		m_Hook.SetHook(GetSafeHwnd());

	CPoint p = GetTrayCoords();

	if( wParam == hkMenu ) {
		TrackWndMenu(p);
	} else
//	hide current || hide current ot tray
	if( wParam == hkHide ||
		wParam == hkHideToTray )
	{
	//go through owner chain and hide first window we can hide
		HWND hWnd = ::GetForegroundWindow();
		do{
			try {
				CWndData& wndData = m_aWindows[hWnd];
				BOOL bToTray = wndData.IsTrayable();
				if( wParam == hkHideToTray )
					bToTray = !bToTray;
				wndData.Show(FALSE, bToTray);
				::SetForegroundWindow(m_aWindows.GetVisibleWnd());
				CWndData::Compact(m_hWnd);
				return;
			} catch(CUserException*){}
		} while( hWnd = ::GetWindow(hWnd, GW_OWNER) );
		WARNSOUND();
	} else
//	minimize current
	if( wParam == hkMinimize ) {
		::PostMessage(::GetForegroundWindow(),
			WM_SYSCOMMAND,SC_MINIMIZE, NULL);
	} else
//	hide all
	if( wParam == hkHideAll ) {
		m_aWindows.HideAll();
	} else
//	show all
	if( wParam == hkShowAll ) {
		m_aWindows.ShowAll();
	} else
//	toggle desktop icons
	if( wParam == hkToggleDesktop ) {
		ToggleDesktopIcons();
	} else
//	toggle taskbar icons
	if( wParam == hkToggleTaskbar ) {
		ToggleTaskbar();
	} else
//	hide everything
	if( wParam == hkHideEverything ) {
		m_aWindows.HideAll();
		ToggleDesktopIcons(FALSE);
		//hide taskbar only if either "toggle taskbar"
		//or "menu" hotkeys are assigned
		if( m_HotKey[hkToggleTaskbar] || m_HotKey[hkMenu] )	ToggleTaskbar(FALSE);
	} else
//	show everything
	if( wParam == hkShowEverything ) {
		m_aWindows.ShowAll();
		ToggleDesktopIcons(TRUE);
		ToggleTaskbar(TRUE);
	} else
//	activate screensaver
	if( wParam == hkScreensaver )
		PostMessage(WM_SYSCOMMAND,SC_SCREENSAVE,0);
	else
//	shell run dialog
	if( wParam == hkRunDialog )
		PostMessage(WM_SHELLRUN);
//	open/close CD-ROM
	if( wParam == hkCDROM ) {
		TCHAR szRet[255];
		mciSendString(_T("status cdaudio ready wait"), szRet, countof(szRet), NULL);
		mciSendString(
			StrPrintf(_T("set cdaudio door %s"), _tcsicmp(szRet, "TRUE") == 0 ? "open":"closed"),
			szRet, countof(szRet), NULL);
	} else
//	current dir menu
	if( wParam == hkCurrentDir )
	{
		TCHAR szPath[MAX_PATH];
		if( m_Hook.GetWndCWD(::GetForegroundWindow(), szPath, countof(szPath)) &&
			*szPath != 0 )
		do {
			CPidl pidl(szPath);
			if( pidl.IsEmpty() ) {
				WARNSOUND(); break;
			}
			int nPos = m_aShellTemp.FindPidl(pidl);
			if( nPos < 0 ) {
				CShlRoot *newMenu = new CShlRoot();
				*newMenu = m_CwdMenuTemplate;
				newMenu->Attach(pidl.Detach());
				m_aShellTemp.InsertAt(0, newMenu);
				if( m_aShellTemp.GetSize() > 10 )
					m_aShellTemp.RemoveAt(m_aShellTemp.GetSize()-1);
			} else
			if( nPos > 0 ) {
				CShlRoot *newMenu = m_aShellTemp[nPos]; 
				m_aShellTemp.GetData()[nPos] = NULL; 
				m_aShellTemp.RemoveAt(nPos);
				m_aShellTemp.InsertAt(0, newMenu);
			}
			PostMessage(WM_SHELLMENU, smfMenuAtMousePos|smfTempMenu, 0);
		} while(0);
	} else
//	shell menu
	if( wParam >= (UINT)ID_SHELL_FIRST &&
		wParam <  (UINT)(ID_SHELL_FIRST + m_aShellMenus.GetSize()) )
	{
		PostMessage(WM_SHELLMENU,FALSE,wParam-ID_SHELL_FIRST);
	}
	CWndData::Compact(m_hWnd);
}

void CMainWnd::OnShellRun(WPARAM wParam, LPARAM lParam)
{
	RunDialog();
}

void CMainWnd::OnShellTrayIcon(WPARAM wParam, LPARAM lParam)
{
	if(!( (UINT)lParam == WM_LBUTTONUP ||
	      (UINT)lParam == WM_RBUTTONUP ||
	      (UINT)lParam == WM_MBUTTONUP) )
		 return;
	PostMessage(WM_SHELLMENU, (WPARAM)TRUE, (LPARAM)wParam);
}

void CALLBACK
ShlMenuBuildCallback(CShlMenu& rRootMenu, CShlFolder& rFolder, CMenu2& rSubMenu)
{
	if( rFolder.GetRoot() == &rFolder ) {
		rSubMenu.AppendMenu(MF_STRING, ID_SHL_CDUP,	LoadString(ID_SHL_CDUP));
	} else {
		rSubMenu.AppendMenu(MF_STRING, ID_SHELL_LAST+rFolder.GetMenuID(), LoadString(ID_SHL_CDHERE));
		rRootMenu.SetItemImage(ID_SHELL_LAST+rFolder.GetMenuID(), ID_SHL_CDHERE);
	}
}

void CMainWnd::OnShellMenu(WPARAM wParam, LPARAM lParam)
{
//	wParam:
//		TRUE  -> menu at mouse cursor
//		FALSE -> menu at system tray
//	lParam:
//		the index of shell cache

	CShlRootArray *paMenus = &m_aShellMenus;
	if( wParam & smfTempMenu ) {
		paMenus = &m_aShellTemp;
		(*paMenus)[lParam]->SetDisplayName(LoadString(ID_SHL_CWD));
	}
	if( lParam >= paMenus->GetSize() )
		return;

	try {
		CPoint p;
		if( wParam ) ::GetCursorPos(&p);
		else p = GetTrayCoords();

		CShlMenu mnu;
		UINT nShlResult;
		mnu.LoadToolbar(IDR_SHELL);
		mnu.SetTruncate(m_nTruncate);
		mnu.SetRefreshID(ID_SHL_REFRESH);
		mnu.SetOpenFolderString(IDS_OPEN_FOLDER);
		
		HWND hFWnd = ::GetForegroundWindow();
		this->SetForegroundWindow();
		LockUI();
		try {
			nShlResult = mnu.TrackPopupMenu(
				*((*paMenus)[lParam]), MenuFlags, p.x, p.y,
				(wParam&smfTempMenu)?ShlMenuBuildCallback:NULL);
		} catch(...) {
			UnlockUI();
			throw;
		}
		UnlockUI();
		PostMessage(WM_NULL, 0, 0);
		
		if( (wParam & smfTempMenu) &&
			nShlResult > ID_SHELL_LAST )
		{
			CShlItem* item;
			TCHAR szPath[MAX_PATH];
			if( nShlResult == ID_SHL_CDUP )
				item = (*paMenus)[lParam];
			else
				item = ((*paMenus)[lParam])->GetItemByID(nShlResult-ID_SHELL_LAST);
			if( !item || !SHGetPathFromIDList(*item, szPath) || *szPath == 0 )
				goto QUIT;
			if( nShlResult == ID_SHL_CDUP ) {
				LPTSTR pszSlash = _tcsrchr(szPath, '\\');
				if( !pszSlash ) goto QUIT; else *(pszSlash+1) = 0;
				PostMessage(WM_HOTKEY, (WPARAM)hkCurrentDir);
			}
			paMenus->RemoveAt(lParam);
			m_Hook.SetWndCWD(hFWnd, szPath);
			goto QUIT;
		} else
		if( nShlResult ) {
			CShlItem* item = ((*paMenus)[lParam])->GetItemByID(nShlResult);
			if( item ) item->Invoke();
		} else 
QUIT:	if( ::GetForegroundWindow() == m_hWnd &&
			::IsWindow(hFWnd) )
		{
			::SetForegroundWindow(hFWnd);
		}
	} catch( _com_error e ) {
		AfxMessageBox( (LPCTSTR)e.ErrorMessage(), MB_ICONSTOP );
	}
	CWndData::Compact(m_hWnd);
}

void RemoveMenuItems(CMenu& m, CMenuIDList& l)
{
	POSITION p = l.GetHeadPosition();
	while( p ) m.DeleteMenu(l.GetNext(p), MF_BYCOMMAND);

	INIT2<MENUITEMINFO> mii;
	mii.fMask = MIIM_TYPE;
	while( m.GetMenuItemInfo(0, &mii, TRUE) &&
		   (mii.fType & MFT_SEPARATOR) ) 
	{
		m.DeleteMenu(0, MF_BYPOSITION);
		mii.clear(); mii.fMask = MIIM_TYPE;
	}

	mii.clear(); mii.fMask = MIIM_TYPE;
	while( m.GetMenuItemInfo(m.GetMenuItemCount()-1, &mii, TRUE) &&
		  (mii.fType & MFT_SEPARATOR) ) 
	{
		m.DeleteMenu(m.GetMenuItemCount()-1, MF_BYPOSITION);
		mii.clear(); mii.fMask = MIIM_TYPE;
	}
}

void CMainWnd::TrackWndMenu(CPoint& p)
{
	if( m_nLockUI ) {
		this->SetForegroundWindow();
		WARNSOUND();
		return;
	}

	if( m_bPasswordMenu ) {
		if( !OnVerifyPassword(NULL,NULL) ) return;
	}

	HWND hFWnd = ::GetForegroundWindow();
	CMenu2 theMenu, *subMenu;

	m_Manager.Refresh();
	if( !theMenu.CreatePopupMenu() ) return;
	if( subMenu = theMenu.AppendSubMenu(AfxGetAppName(),IDR_MAINFRAME) ) {
		if( !(HWND)CWndData::GetDesktopWnd() )
			subMenu->EnableMenuItem(ID_DESKTOPICONS, MF_BYCOMMAND|MF_GRAYED);
		else if( CWndData::GetDesktopWnd().IsVisible() ) 
			subMenu->CheckMenuItem(ID_DESKTOPICONS, MF_BYCOMMAND|MF_CHECKED);

		if( !(HWND)CWndData::GetTaskbarWnd() )
			subMenu->EnableMenuItem(ID_TRAYWND, MF_BYCOMMAND|MF_GRAYED);
		else if( CWndData::GetTaskbarWnd().IsVisible() ) 
			subMenu->CheckMenuItem(ID_TRAYWND, MF_BYCOMMAND|MF_CHECKED);

		if( m_bTimerLocked == 0 ) 
			subMenu->CheckMenuItem(ID_TIMEROP, MF_BYCOMMAND|MF_CHECKED);

		if( theMenu.SetMenuItemID(theMenu.GetMenuItemCount()-1, IDR_MAINFRAME) )
			m_Manager.SetItemImage(IDR_MAINFRAME, PxGetWindowIconSm(this->GetSafeHwnd()));
		RemoveMenuItems(*subMenu, m_lsDisabledMenus);
	} else 
		theMenu.AppendMenu(MF_STRING, ID_APP_EXIT, _T("E&xit"));
//	Fill submenu "Shell" with data
	if( subMenu = theMenu.AppendSubMenu(IDR_SHELL) ) {
		theMenu.SetMenuItemID(theMenu.GetMenuItemCount()-1, IDR_SHELL);
		if( m_aShellMenus.GetSize() )
			subMenu->AppendMenu(MF_SEPARATOR);
		for(int i=0;i<m_aShellMenus.GetSize();i++) {
			subMenu->AppendMenu(MF_STRING,ID_SHELL_FIRST+i,m_aShellMenus[i]->GetDisplayName());
			subMenu->AppendHotKey(ID_SHELL_FIRST+i,m_aShellMenus[i]->GetHotKey());
			m_Manager.SetItemImage(ID_SHELL_FIRST+i, m_aShellMenus[i]->GetIcon());
		}
		subMenu->AddAmpersands();
		RemoveMenuItems(*subMenu, m_lsDisabledMenus);
		if( !subMenu->GetMenuItemCount() ) //remove shell submenu if it's empty
			theMenu.DeleteMenu(theMenu.GetMenuItemCount()-1, MF_BYPOSITION);
	}

	theMenu.AppendHotKey(ID_SCREENSAVE,      m_HotKey[hkScreensaver]);
	theMenu.AppendHotKey(ID_HIDEALL,	     m_HotKey[hkHideAll]);
	theMenu.AppendHotKey(ID_SHOWALL,	     m_HotKey[hkShowAll]);
	theMenu.AppendHotKey(ID_DESKTOPICONS,    m_HotKey[hkToggleDesktop]);
	theMenu.AppendHotKey(ID_TRAYWND,	     m_HotKey[hkToggleTaskbar]);
	theMenu.AppendHotKey(IDR_MAINFRAME,      m_HotKey[hkMenu]);
	theMenu.AppendHotKey(ID_SHL_RUN,		 m_HotKey[hkRunDialog]);
	theMenu.AppendMenu(MF_SEPARATOR);
	BuildWndMenu(theMenu);

	this->SetForegroundWindow();  
	LockUI();
	int nResult = theMenu.TrackPopupMenu(MenuFlags, p.x, p.y, this);
	UnlockUI();
	PostMessage(WM_NULL, 0, 0);
	
	switch( nResult ) {
	case 0:	break;
	case ID_APP_EXIT:			DestroyWindow(); return;
	case ID_SHL_RUN:			PostMessage(WM_SHELLRUN); break;
	case ID_SCREENSAVE:			PostMessage(WM_SYSCOMMAND,SC_SCREENSAVE,0);	break;
	case ID_SHOWALL:			m_aWindows.ShowAll(); break;
	case ID_HIDEALL:			m_aWindows.HideAll(); break;
	case ID_DESKTOPICONS:		ToggleDesktopIcons(); break;
	case ID_TRAYWND:			ToggleTaskbar(); break;	
	case ID_OPTIONS:			PostMessage(WM_SHOWPROPERTIES,NULL,NULL); break;
	case ID_TIMEROP:			LockTimer(!m_bTimerLocked); SaveConfig(); break;
	case ID_HELP:				theApp.WinHelp(1); break;
	default:
		if( nResult >= ID_SHELL_FIRST &&
			nResult < (ID_SHELL_FIRST + m_aShellMenus.GetSize()))
		{// process shell menus
			PostMessage(WM_SHELLMENU,TRUE,nResult-ID_SHELL_FIRST);
		} else
		if( nResult >= ID_WND_FIRST &&
			nResult < (ID_WND_FIRST + m_aWindows.GetSize()) )
		{// show|hide windows
			nResult -= ID_WND_FIRST;
			CWndData& wndData = m_aWindows[nResult];
			wndData.Show( !wndData.IsVisible() );
			if( wndData.IsVisible() ) hFWnd = wndData;
		}
	}
	m_Manager.Refresh();
//	restore foreground window
	if( !hFWnd || !CWndData::IsVisible(hFWnd) )
		hFWnd = m_aWindows.GetVisibleWnd();
	if( ::GetForegroundWindow() == m_hWnd &&
		::IsWindow(hFWnd) )
	{
		::SetForegroundWindow(hFWnd);
	}
	CWndData::Compact(m_hWnd);
}

BOOL CMainWnd::IsScreensaverRunning()
{
	BOOL bResult = FALSE;
	if( IsNewWin() ) {
	//--for Win98 & Win2k
		if( !::SystemParametersInfo(SPI_SCREENSAVERRUNNING,
			NULL, &bResult, FALSE))
			return FALSE;
		return bResult;
	} else 
	if( IsWinNT() ) {
	//--for WinNT 4.0
		HINSTANCE hLib = ::GetModuleHandle(_T("USER32.DLL"));
		if( hLib ) {
			HDESK (WINAPI *fnOpen)(LPCSTR,DWORD,BOOL,DWORD) = (HDESK (WINAPI *)(LPCSTR,DWORD,BOOL,DWORD))::GetProcAddress(hLib,"OpenDesktopA");
			BOOL (WINAPI *fnClose)(HDESK) = (BOOL (WINAPI *)(HDESK))::GetProcAddress(hLib,"CloseDesktop");
			
			if( fnOpen && fnClose ) {
				HDESK hDesk = fnOpen("screen-saver",NULL,FALSE,DESKTOP_READOBJECTS);
				bResult = (hDesk != NULL);
				if( hDesk )	fnClose(hDesk);
			}
		}
	}
	//--Win95 users - sorry but on your OS this stuff is _really_ slow
	return bResult;
}

BOOL CMainWnd::IsWndInTray(HWND hWnd)
{
	for( int j=0; j<m_aWndTrayIcons.GetSize(); j++ )
		 if( hWnd == (HWND)m_aWndTrayIcons[j]->GetID() ) 
			 return TRUE;
	return FALSE;
}

CString CMainWnd::TruncateWndName(LPCTSTR szName)
{
	CString sName(szName);
	if( !m_nTruncate ) return sName;
	LPTSTR pszBuf = sName.GetBuffer(255);

	if( _tcslen(pszBuf) > m_nTruncate ) {
		for(UINT i = m_nTruncate; i > 0; i--)
			if( _tcschr(_T(" /\\.,-!+:"), pszBuf[i]) ) {
				i++; break;
			}
		if( i < (m_nTruncate/2) )
			i = m_nTruncate - 3;

		for(UINT j=i; j<(i+3); j++)
			pszBuf[j] = '.';
		pszBuf[j] = 0;
	}

	sName.ReleaseBuffer();
	return sName;
}

void CMainWnd::BuildWndMenu(CMenu2& wndMenu)
{
	m_Manager.ResetOverlay();
	if( !wndMenu.GetSafeHmenu() ) wndMenu.CreatePopupMenu();
	CWndList::ByName aByName = m_aWindows.GetWndByName();

	int nID, nPos, nStartPos, nVisPos, nHidTrayPos, nHidPos;
	nStartPos = nVisPos = nHidTrayPos = nHidPos = wndMenu.GetMenuItemCount();
	for(int i = 0; i < aByName.GetSize(); i++ ) {
		CWndData& wndData = *aByName[i];

		nID = ID_WND_FIRST + m_aWindows.Lookup(wndData);
		if( wndData.IsVisible() ) {
			nPos = nVisPos;
			nVisPos++; nHidTrayPos++; nHidPos++;
		} else {
			if( IsWndInTray(wndData) ) {
				if( m_bUseOverlay )	m_Manager.SetItemOverlay(nID, 2);
				nPos = nHidTrayPos;
				nHidTrayPos++; nHidPos++;
			} else {
				if( m_bUseOverlay )	m_Manager.SetItemOverlay(nID, 1);
				nPos = nHidPos;
				nHidPos++;
			}
		}
		if( !m_bSortWindows )
			nPos = wndMenu.GetMenuItemCount();

		wndMenu.InsertMenu(nPos, MF_BYPOSITION|MF_STRING,
			nID, TruncateWndName(wndData.GetWindowName()) );
		wndMenu.AppendHotKey(nID, wndData.GetHotKey());
		if( wndData.IsVisible() )
			wndMenu.CheckMenuItem(nID, MF_BYCOMMAND|MF_CHECKED);
		if( m_bShowIcons ) {
			m_Manager.SetItemImage(nID, wndData.GetIcon() );
		}
	}
	if( m_bSortWindows ) {
		if( nStartPos != nVisPos && 
			(nVisPos != nHidTrayPos || nVisPos != nHidPos) )
		{
			nPos = nVisPos; nVisPos++; nHidTrayPos++; nHidPos++;
			wndMenu.InsertMenu(nPos, MF_BYPOSITION|MF_SEPARATOR);
		}
		if( nVisPos != nHidTrayPos && nHidTrayPos != nHidPos )
			wndMenu.InsertMenu(nHidTrayPos, MF_BYPOSITION|MF_SEPARATOR);
	}
	wndMenu.AddAmpersands();
}

void CMainWnd::ReadConfig()
{
	SetHotKeys(FALSE);
	if( theApp.GetProfileInt(_T("RuleSet"), _T("Count"), 0xDEAD) == 0xDEAD )
	{
		CRule rule;
		rule.name = _T("Hide E-Mail client to tray");
		rule.expr = _T("(w/Outlook( Express)?$/r & (c/Outlook/ | c/rctrl_renwnd32/ci)) | (w/^Inbox/r & e/netscape.exe$/ri)");
		rule.stmt.rsDisable    = 1;
		rule.stmt.rsTrayWindow = 1;
		rule.stmt.rsHideWindow = 1;
		rule.hide.wTimeout	   = 60;
		m_aRuleSet.Add(rule);

		rule = CRule();
		rule.name = _T("Hide WinAmp to tray");
		rule.expr = _T("w/Winamp/ & c/Winamp v1.x/ci");
		rule.stmt.rsDisable    = 1;
		rule.stmt.rsTrayWindow = 1;
		rule.stmt.rsHideWindow = 1;
		rule.hide.wTimeout	   = 5;
		m_aRuleSet.Add(rule);

		rule = CRule();
		rule.name = _T("Minimize all windows to tray");
		rule.expr = _T("w/.*/r");
		rule.stmt.rsDisable    = 1;
		rule.stmt.rsTrayWindow = 1;
		m_aRuleSet.Add(rule);

		rule = CRule();
		rule.name = _T("Protect all windows with password");
		rule.expr = _T("w/.*/r");
		rule.stmt.rsDisable = 1;
		rule.stmt.rsPasswordWindow = 1;
		m_aRuleSet.Add(rule);
		m_aRuleSet.RegSave(_T("RuleSet"));
	}

	theApp.SetLanguage( theApp.GetProfileInt(_T(""),_T("Language"),0) );
	m_sIconFile   = theApp.GetProfileString(_T(""),  _T("IconFile"), m_sIconFile);
	m_nIconIndex  = theApp.GetProfileInt(_T(""), _T("IconIndex"),  0);
	m_bEnableIcon = theApp.GetProfileInt(_T(""), _T("EnableIcon"), TRUE);
	m_bUseRestore = theApp.GetProfileInt(_T(""), _T("UseRestore"), TRUE);
	m_bUseCompact = theApp.GetProfileInt(_T(""), _T("UseCompact"), FALSE);
	m_bUseOverlay = theApp.GetProfileInt(_T(""), _T("UseOverlay"), TRUE);
	m_bSortWindows =theApp.GetProfileInt(_T(""), _T("SortWindows"), TRUE);
	m_bPasswordMenu=theApp.GetProfileInt(_T(""), _T("PasswordMenu"), FALSE);
	m_bEjectCDAtExit=theApp.GetProfileInt(_T(""), _T("EjectCDAtExit"), TRUE);
	m_bTopOnly    = theApp.GetProfileInt(_T(""), _T("TopOnly"), TRUE);
	m_bShowIcons  = theApp.GetProfileInt(_T(""), _T("ShowIcons"), TRUE);
	m_nTruncate   = theApp.GetProfileInt(_T(""), _T("Truncate"), 25);
	m_bTimerLocked= theApp.GetProfileInt(_T(""), _T("TimerLocked"), FALSE);
	m_bFixDskText = theApp.GetProfileInt(_T(""), _T("FixDesktopText"), TRUE);
	m_crDskText   = theApp.GetProfileInt(_T(""), _T("DesktopColor"), ::GetSysColor(COLOR_CAPTIONTEXT));
	CPassword::m_nPasswordTimeout = theApp.GetProfileInt(_T(""), _T("PasswordTimeout"), 0);
	m_MouseAction.Restore(_T("MainIcon"));
	m_MouseTAction.Restore(_T("WndIcon"));
	m_lsDisabledMenus.Restore(_T("DisabledMenus"));
	m_aRuleSet.RegRead(_T("RuleSet"));
	m_aShellMenus.Restore(_T("ShellMenu"));
	m_CwdMenuTemplate.RegRead(_T("ShellMenu\\CWD"));
//	read hotkeys
	ZeroMemory(m_HotKey, sizeof(m_HotKey));
	for(int i=0; i<hkLast; i++) m_HotKey[i] = theApp.GetProfileInt(_T(""), StrPrintf(_T("Hotkey%d"),i), NULL);

//	set visibility of taskbar & desktop icons
	i = theApp.GetProfileInt(_T(""), _T("TaskbarVisible"), 0xEEEEE); if(i != 0xEEEEE) ToggleTaskbar(i);
	i = theApp.GetProfileInt(_T(""), _T("DesktopIconsVisible"), 0xEEEEE); if(i != 0xEEEEE) ToggleDesktopIcons(i);
	SetHotKeys(TRUE);
}

void CMainWnd::SaveConfig()
{
	theApp.WriteProfileInt(_T(""), _T("Language"),	  theApp.GetLanguage());
	theApp.WriteProfileString(_T(""),_T("IconFile"),  m_sIconFile);
	theApp.WriteProfileInt(_T(""),  _T("IconIndex"),  m_nIconIndex);
	theApp.WriteProfileInt(_T(""),  _T("EnableIcon"), m_bEnableIcon);
	theApp.WriteProfileInt(_T(""), _T("UseRestore"),  m_bUseRestore);
	theApp.WriteProfileInt(_T(""), _T("UseCompact"),  m_bUseCompact);
	theApp.WriteProfileInt(_T(""), _T("UseOverlay"),  m_bUseOverlay);
	theApp.WriteProfileInt(_T(""), _T("SortWindows"), m_bSortWindows);
	theApp.WriteProfileInt(_T(""), _T("PasswordMenu"),m_bPasswordMenu);
	theApp.WriteProfileInt(_T(""), _T("EjectCDAtExit"),m_bEjectCDAtExit);
	theApp.WriteProfileInt(_T(""), _T("TopOnly"),     m_bTopOnly);
	theApp.WriteProfileInt(_T(""), _T("ShowIcons"),   m_bShowIcons);
	theApp.WriteProfileInt(_T(""), _T("Truncate"),    m_nTruncate);
	theApp.WriteProfileInt(_T(""), _T("TimerLocked"), m_bTimerLocked);
	theApp.WriteProfileInt(_T(""), _T("FixDesktopText"), m_bFixDskText);
	theApp.WriteProfileInt(_T(""), _T("DesktopColor"),	 m_crDskText);
	theApp.WriteProfileInt(_T(""), _T("PasswordTimeout"),CPassword::m_nPasswordTimeout);
	m_MouseAction.Save(_T("MainIcon"));
	m_MouseTAction.Save(_T("WndIcon"));
	m_lsDisabledMenus.Save(_T("DisabledMenus"));
	m_aRuleSet.RegSave(_T("RuleSet"));
	m_aShellMenus.Save(_T("ShellMenu"));
	m_CwdMenuTemplate.RegSave(_T("ShellMenu\\CWD"));

//	write hotkeys
	for(int i=0;i<hkLast;i++ )
		theApp.WriteProfileInt(_T(""), StrPrintf(_T("Hotkey%d"),i), m_HotKey[i]);
}

void CMainWnd::OnProperties(WPARAM, LPARAM)
{
	if( m_nLockUI != 0 ) {
		SetForegroundWindow();
		WARNSOUND();
		return;
	}

	if( !OnVerifyPassword(NULL,NULL) )
		return;

	LockUI();
	if( !PxCheckModuleVersion(_T("COMCTL32.DLL"), 4, 71, IDS_ERR_OLDCOMCTRL) )
	{
		UnlockUI();
		return;
	}
	
	COptionsSheet dlgProp(::StrPrintf(IDS_OPTIONS, ::AfxGetAppName()));
	dlgProp.m_ppMain.SetData(this);
	dlgProp.m_ppMisc.SetData(this);
	dlgProp.m_ppFolders.SetData(this);
	dlgProp.m_ppRuleSet.SetData(this);

	SetHotKeys(FALSE);// clear hotkeys before entering in modal loop
	if( dlgProp.DoModal() == IDOK ) {
		dlgProp.m_ppMain.GetData(this);
		dlgProp.m_ppMisc.GetData(this);
		dlgProp.m_ppFolders.GetData(this);
		dlgProp.m_ppRuleSet.GetData(this);
		m_aShellTemp.RemoveAll();
		SendMessage(WM_RESTOREICON,0,0);
		SaveConfig();
	};

	SetHotKeys(TRUE);// set hotkeys back
	UnlockUI();
}

void CMainWnd::OnTimer(UINT nIDEvent) 
{
	if( m_nLockUI )
		return; //user interface is locked - suspend timer too

	if( IsScreensaverRunning() )
		return; //don't do anything while screensaver is up

	static int nDskUpdateCount = 1;
	if( (--nDskUpdateCount ) == 0 ) {
		nDskUpdateCount = SEC2TIMESLICE(5);
		OnFixDskText(0,0);
	}

	static int nTrayUpdateCount = 1;
	if( (--nTrayUpdateCount ) == 0 ) {
		nTrayUpdateCount = SEC2TIMESLICE(1);
		CWndData::RefreshProcessCache();
		UpdateTrayIcons();
	}

//	process auto actions
	if( m_bTimerLocked || m_aRuleSet.GetSize() == 0 )
		return;
	m_aWindows.Refresh();
	for(int i = 0; i < m_aWindows.GetSize(); i++ ) {
		CWndData& wndData = m_aWindows[i];
		wndData.AutoMinimize();
		wndData.AutoHide();
		wndData.AutoClose();
	}
}

BOOL RegisterHotKey(HWND hWnd, int iId, WORD wHotKey)
{
	UINT uMod = 0;
	if( HIBYTE(wHotKey) & HOTKEYF_SHIFT )   uMod |= MOD_SHIFT;
	if( HIBYTE(wHotKey) & HOTKEYF_CONTROL ) uMod |= MOD_CONTROL;
	if( HIBYTE(wHotKey) & HOTKEYF_ALT )     uMod |= MOD_ALT;
	if( HIBYTE(wHotKey) & HOTKEYF_EXT )     uMod |= MOD_WIN;
	return ::RegisterHotKey(hWnd, iId, uMod, LOBYTE(wHotKey));
}

void CMainWnd::SetHotKeys(BOOL bSet/*=TRUE*/)
{
	if( bSet ) {
	//	set hot keys
		for( int i = 0; i < hkLast; i++ )
			SetHotKey((HotKeys)i, m_HotKey[i]);
		for( i = 0; i < m_aShellMenus.GetSize(); i++ )
			if( m_aShellMenus[i]->GetHotKey() )
				::RegisterHotKey(GetSafeHwnd(), ID_SHELL_FIRST+i,
					m_aShellMenus[i]->GetHotKey());
	} else {
	//	reset hot keys
		for( int i = 0; i < hkLast; i++ ) {
			WORD wSaveHotKey = m_HotKey[i];
			SetHotKey((HotKeys)i,NULL);
			m_HotKey[(HotKeys)i]=wSaveHotKey;
		}
		for( i = 0; i < m_aShellMenus.GetSize(); i++ )
			if( m_aShellMenus[i]->GetHotKey() )
				::UnregisterHotKey(GetSafeHwnd(), ID_SHELL_FIRST+i);
	}
}

BOOL CMainWnd::SetHotKey(HotKeys index, WORD wHotKey)
{
	BOOL bResult = TRUE;
	
	if( wHotKey ) bResult = ::RegisterHotKey(GetSafeHwnd(), index, wHotKey);
	else bResult = ::UnregisterHotKey(GetSafeHwnd(), index);
	if( bResult ) m_HotKey[index] = wHotKey;
	return bResult;
}


BOOL CMainWnd::ValidateHotKey(HotKeys hkIndex, WORD wHotKey)
{
	ASSERT( hkIndex < hkLast );

	if( wHotKey == NULL ) return TRUE;
	if( m_HotKey[hkIndex] == wHotKey ) return TRUE;

	BOOL bResult = TRUE;
	bResult = ::RegisterHotKey(GetSafeHwnd(), hkIndex, (WORD)wHotKey);
	if( bResult ) UnregisterHotKey(GetSafeHwnd(), hkLast);
	return bResult;
}

BOOL CMainWnd::OnHelpInfo(HELPINFO* pHelpInfo) 
{
	if( pHelpInfo->iContextType == HELPINFO_MENUITEM ) {
		int iContext = (int)pHelpInfo->iCtrlId;
		if( iContext >= ID_WND_FIRST && iContext <= ID_WND_LAST )
		{
			iContext = ::GetMenuState((HMENU)pHelpInfo->hItemHandle,iContext,MF_BYCOMMAND)&MF_CHECKED?
				ID_VISIBLEWND:ID_HIDDENWND;
		}
		theApp.WinHelp(iContext,HELP_CONTEXTPOPUP);
		return TRUE;
	}
	return FALSE;
}

void CMainWnd::OnWndMinimize(WPARAM wParam, LPARAM lParam)
// wParam: CLICKDATA struct
// lParam: HWND clicked
{
	
	CLICKDATA click;
	click.dwData = wParam;
/*	SetForegroundWindow();
	AfxMessageBox(StrPrintf("Mouse click on HWND %X\n\tHitTest: %s\n\tButton: %s\n\tLongClick: %s",
		lParam,
		click.nHitTestCode==HTREDUCE?"Minimize":
		click.nHitTestCode==HTZOOM?"Maximize":
		click.nHitTestCode==HTCLOSE?"Close":"Hell knows",
		click.nMouseButton==1?"Right":
		click.nMouseButton==2?"Middle":"Hell knows",
		click.bLongClick==1?"Yes":"No"));
*/
	if( click.nHitTestCode != HTREDUCE )
		return;
	HWND hWnd = ::GetForegroundWindow();
	do{
		try {
			CWndData& wndData = m_aWindows[hWnd];
			BOOL bToTray = wndData.IsTrayable();
			if( click.bLongClick )
				bToTray = !bToTray;
			wndData.Show(FALSE, bToTray);
			// Do not call ::SetForegroundWindow(m_aWindows.GetVisibleWnd()) here !
			// on Win98/Win2K it won't make any good...
			CWndData::Compact(m_hWnd);
			return;
		} catch(CUserException*){}
	} while( hWnd = ::GetWindow(hWnd, GW_OWNER) );
	WARNSOUND();
	CWndData::Compact(m_hWnd);
}

void CMainWnd::OnWndActivate(WPARAM wParam, LPARAM lParam)
{
	HWND hWnd = (HWND)lParam;
	do{
		if( hWnd == m_hJustHiddenWnd )
			continue;
		try {
			CWndData& wndData = m_aWindows[hWnd];
			if( wndData.IsChanged() )
			{
				if( (HWND)wndData == m_hJustHiddenWnd ||
					wndData == CWndData::GetTaskbarWnd() ||
					wndData == CWndData::GetDesktopWnd() )
				{
					continue;
				}
				m_aWindows[hWnd].Show(TRUE);
				CWndData::Compact(m_hWnd);
				return;
			}
		} catch(CUserException*){/*do nothing*/}
	} while( hWnd = ::GetWindow(hWnd, GW_OWNER) );
}

void CMainWnd::OnWndTrayIcon(WPARAM wParam, LPARAM lParam)
{
	CWndData& wndData = m_aWindows[(HWND)wParam];
	if( !::IsWindow(wndData) || wndData.IsVisible() )
	{
		INIT2<NOTIFYICONDATA> nid;
		nid.uID	 = (UINT)wParam;
		nid.hWnd = GetSafeHwnd();
		::Shell_NotifyIcon(NIM_DELETE,&nid);
		return;
	}

	HWND hFWnd = ::GetForegroundWindow();
	switch( lParam = m_MouseTAction.GetAction((UINT)lParam, ::GetModifierKey()) ) {
	case TACTION_RESTORE:
	case TACTION_RESTOREMAX: {
		wndData.Show(TRUE);
		hFWnd = wndData;
		if( wndData.IsVisible() && lParam == TACTION_RESTOREMAX )
			::ShowWindow(wndData, SW_MAXIMIZE);
		break;}//TACTION_RESTORE

	case TACTION_CLOSE: {
		CPoint p; GetCursorPos(&p);
		::PostMessage(wndData, WM_SYSCOMMAND, SC_CLOSE, ((UINT)p.y << 16) + p.x);
		break;}//TACTION_CLOSE

	case TACTION_MENU: {
		HMENU hMenu = ::GetSystemMenu(wndData, FALSE);
		if( hMenu && IsMenu(hMenu) ) {
			::EnableMenuItem(hMenu,SC_RESTORE,MF_BYCOMMAND|MF_ENABLED);
			::EnableMenuItem(hMenu,SC_MAXIMIZE,MF_BYCOMMAND|
				((::GetWindowLong(wndData,GWL_STYLE)&WS_MAXIMIZE)?MF_GRAYED:MF_ENABLED));
			::EnableMenuItem(hMenu,SC_MINIMIZE,MF_BYCOMMAND|
				((::GetWindowLong(wndData,GWL_STYLE)&WS_MINIMIZE)?MF_GRAYED:MF_ENABLED));
			::EnableMenuItem(hMenu,SC_SIZE,MF_BYCOMMAND|MF_GRAYED);
			::EnableMenuItem(hMenu,SC_MOVE,MF_BYCOMMAND|MF_GRAYED);
			::SetMenuDefaultItem(hMenu,SC_RESTORE,FALSE);

			this->SetForegroundWindow();
			CPoint p; GetCursorPos(&p);
			int nResult = ::TrackPopupMenu(hMenu,
				MenuFlags|TPM_NONOTIFY,
				p.x, p.y, 0, this->GetSafeHwnd(), NULL);
			switch( nResult ) {
			case 0: break;
			case SC_RESTORE:
			case SC_MAXIMIZE:
			case SC_MINIMIZE: {
				wndData.Show(TRUE); 
				if( wndData.IsVisible() )
					::PostMessage(hFWnd = wndData, WM_SYSCOMMAND, nResult, 0);
				break;}
			default:
				::PostMessage(wndData, WM_SYSCOMMAND, nResult, 0);
			}
			if( ::GetForegroundWindow() != m_hWnd )
				hFWnd = NULL;
		}
		break;}//TACTION_MENU
	}	
	if( ::IsWindow(hFWnd) )
		::SetForegroundWindow(hFWnd);
	CWndData::Compact(m_hWnd);
}

void CMainWnd::UpdateTrayIcons()
{
	static BOOL bLocked = FALSE;
	if( bLocked ) return;
	bLocked = TRUE;

	for( int i = 0; i<m_aWndTrayIcons.GetSize(); i++ )
	try {
		CWndData& wndData = m_aWindows[(HWND)m_aWndTrayIcons[i]->GetID()];
		if( wndData.IsVisible() ) {
			wndData.property[wndData.propTray] = NULL;
			AfxThrowUserException();
		}
	//	update tooltip if window title has changed
		CPxTrayIcon* pIcon = m_aWndTrayIcons[i];
		LPCTSTR pszTip = wndData.GetWindowName();
		if( _tcscmp(pszTip, pIcon->GetTip()) )
			pIcon->SetIcon(pIcon->GetIcon(), pszTip);
	} catch( CUserException* ) {
		m_aWndTrayIcons.RemoveAt(i--);
	}
	bLocked = FALSE;
}

LRESULT CMainWnd::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	// restore tray icons after taskbar was restarted
	static UINT s_uTaskbarRestart = NULL;

	if( message == WM_CREATE ) {
		s_uTaskbarRestart = RegisterWindowMessage(_T("TaskbarCreated"));
	} else
	if( message == s_uTaskbarRestart && 
		s_uTaskbarRestart )
	{
	//	restore tray icons
		PostMessage(WM_RESTOREICON);
	//	fix desktop icon titles
		PostMessage(WM_FIXDSKTEXT);
	}
	return CWnd::WindowProc(message, wParam, lParam);
}

void CMainWnd::RunDialog()
{
	try {
		IShellDispatchPtr pShellDisp( CLSID_Shell );
		pShellDisp->FileRun();
	} catch ( _com_error e ) {
		AfxMessageBox( (LPCTSTR)e.ErrorMessage(), MB_ICONSTOP );
	}
}

void CMainWnd::ToggleTaskbar(BOOL bOn)
{
	CWndData& hTaskbar = CWndData::GetTaskbarWnd();
	if( !(HWND)hTaskbar ) return;
	if( bOn == -1 ) bOn = !hTaskbar.IsVisible();
	if( bOn == 0 && 
		m_HotKey[hkMenu] == NULL &&
		m_HotKey[hkToggleTaskbar] == NULL)
	{
		AfxMessageBox(IDS_ERR_HIDETRAY_NOHOTKEY);
		return;
	}
	hTaskbar.Show(bOn);
	theApp.WriteProfileInt(_T(""), _T("TaskbarVisible"), hTaskbar.IsVisible());
}

void CMainWnd::ToggleDesktopIcons(BOOL bOn)
{
	CWndData& hDesktop = CWndData::GetDesktopWnd();
	if( bOn == -1 ) bOn = !hDesktop.IsVisible();
	hDesktop.Show(bOn);
	theApp.WriteProfileInt(_T(""), _T("DesktopIconsVisible"), hDesktop.IsVisible());
}

void CMainWnd::OnFixDskText(WPARAM wParam, LPARAM lParam)
{
#ifndef _DEBUG
//	do not fix desktop text in debug version - because
//	usuall debug version runs concurrently with release
//	version it'll cause desktop text flicker
	if( !m_bFixDskText ) return;
	CWndData& hDeskList = CWndData::GetDesktopListCtrl();
	if( !hDeskList ) return;
	if( ListView_GetTextBkColor(hDeskList) != -1 ||
		ListView_GetTextColor(hDeskList) != m_crDskText)
	{
		ListView_SetTextBkColor(hDeskList, -1);
		ListView_SetTextColor(hDeskList, m_crDskText);
		::InvalidateRect(hDeskList, NULL, TRUE);
	}
#endif //_DEBUG
}

LPARAM CMainWnd::OnCreateTrayIcon(WPARAM wParam, LPARAM lParam)
// wParam: not used
// lParam: pointer to CWndData object representing the window
{
	CWndData& wndData = *(CWndData*)lParam;

	for( int i = 0; i<m_aWndTrayIcons.GetSize(); i++ )
		 if( m_aWndTrayIcons[i]->GetID() == (UINT)(HWND)wndData )
		 {
			 m_aWndTrayIcons.RemoveAt(i);
			 break;
		 }
	CPxTrayIcon *pTmpIcon = new CPxTrayIcon( (UINT)(HWND)wndData  );
	pTmpIcon->SetNotificationWnd(this, WM_WNDTRAYICON);
	HICON hIcon = wndData.GetIcon();
	if( hIcon &&
		pTmpIcon->SetIcon(hIcon, wndData.GetWindowName()) )
	{
		m_aWndTrayIcons.Add(pTmpIcon);
		wndData.property[wndData.propTray] = TRUE;
		return TRUE;
	} else {
		wndData.property[wndData.propTray] = FALSE;
		delete pTmpIcon;
		return FALSE;
	}
}

LPARAM CMainWnd::OnVerifyPassword(WPARAM wParam, LPARAM lParam)
// wParam: not used
// lParam: not used
{
	if( m_nLockUI != 0 ) {
		this->SetForegroundWindow();
		WARNSOUND();
		return FALSE;
	}

	if( m_bPasswordVerified == TRUE )
		return TRUE;
	if( m_bPasswordVerified == -1 )
		return FALSE; // password verification was denied on previous step

	HWND hFWnd = ::GetForegroundWindow();
	this->SetForegroundWindow();
	LockUI();
	
	int nResult = CPassword::AskPassword();
	if( nResult == IDOK ) {
		m_bPasswordVerified = TRUE;
	} else 
	if( nResult == IDCANCEL ) {
		m_bPasswordVerified = FALSE; // password verification failed
	} else
		m_bPasswordVerified = -1;	 // password verification denied
	UnlockUI();
	if( hFWnd ) ::SetForegroundWindow(hFWnd);

	return (m_bPasswordVerified == TRUE);
}

void CMainWnd::OnSetJustHidden(WPARAM wParam, LPARAM lParam)
// wParam: not used
// lParam: window handle
{
	m_hJustHiddenWnd = (HWND)lParam;
}

LPARAM CMainWnd::OnGetConfig(WPARAM wParam, LPARAM lParam)
// wParam: not used
// lParam: config value name
{
	if( _tcscmp((LPCTSTR)lParam, "TopOnly") == 0 )
		return m_bTopOnly;
	else
	if( _tcscmp((LPCTSTR)lParam, "Compact") == 0 )
		return m_bUseCompact;
	return FALSE;
}
