/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef __WCAT_H_INCLUDED__
#define __WCAT_H_INCLUDED__
#pragma once

#include "wcatres.h"		// main symbols
#include "wcathk.h"
#include "shlcache.h"
#include "shlmenu.h"
#include "rule.h"
#include "wndlist.h"

#define MAINWND	((CMainWnd*)theApp.m_pMainWnd)
extern class CWCatApp theApp;

// functions defined in WCAT.CPP
void SOUND(LPCTSTR snd);
void WARNSOUND(void);
DWORD RegisterServiceProcess(DWORD dwProcessId, DWORD dwType);

// functions defined in PPMAIN.CPP
BOOL SetCbImageList(CComboBoxEx& rCb, CImageList& rIl);
int AdjustDropWidth(CComboBox* pCb);

#define ACTION_NOTHING				0	// Do Nothing
#define ACTION_SHOWLIST				1	// Show window list
#define ACTION_SHOWPROP				2	// Show properties
#define ACTION_HIDEALL				3	// Hide all windows
#define ACTION_SHOWALL				4	// Show all windows
#define ACTION_TOGGLEICONS			5	// Toggle desktop icons

#define TACTION_NOTHING		0	// Do Nothing
#define TACTION_MENU		1	// Show menu
#define TACTION_RESTORE		2	// Restore
#define TACTION_RESTOREMAX	3	// Restore maximized
#define TACTION_CLOSE		4	// Close

typedef enum {
	hkFirst = 0,
	hkMenu = 0,				//Show menu
	hkHide,					//Hide current window
	hkMinimize,				//Minimize current window
	hkHideToTray,			//Minimize current window to tray
	hkHideAll,				//Hide all windows
	hkShowAll,				//Show all windows
	hkToggleDesktop,		//Toggle desktop icons
	hkToggleTaskbar,		//Toggle taskbar
	hkHideEverything,		//Hide everything
	hkShowEverything,		//Show everything
	hkScreensaver,			//Activate Screensaver
	hkRunDialog,			//Open "Start/Run" dialog
	hkCDROM,				//Open/Close CD-ROM
	hkCurrentDir,			//Current Directory menu
	hkLast
} HotKeys;

typedef enum {
	maLeftBtnClick = 0,
	maLeftBtnDblClick,
	maRightBtnClick,
	maRightBtnDblClick,
	maMiddleBtnClick,
	maMiddleBtnDblClick,
	maLast
} MouseActions;

typedef enum {
	kmNone = 0,
	kmCtrl,
	kmAlt,
	kmShift,
	kmCtrlAlt,
	kmCtrlShift,
	kmAltShift,
	kmCtrlAltShift,
	kmLast
} ModifierKeys;

class CWCatHook {
protected:
	HMODULE	hHookLib;
	BOOL LoadLib() 
		{
			if( !hHookLib ) hHookLib = LoadLibrary(_T("WCATHK.DLL"));
			return hHookLib != NULL;
		}
public:
	CWCatHook():
		hHookLib(NULL)
		{
		}
	~CWCatHook()
		{
			if( hHookLib ) { 
				RemoveHook();
				FreeLibrary(hHookLib);
			}
		}
	operator HMODULE()
		{
			return hHookLib;
		}
	HWND IsHookSet()
		{
			if( hHookLib ) {
				lpfnIsHookSet proc = (lpfnIsHookSet)GetProcAddress(hHookLib,"IsHookSet");
				if( proc ) return proc();
			}
			return NULL;
		}
	BOOL SetHook(HWND hWnd)
		{
			if( LoadLib() ) {
				lpfnSetHook proc = (lpfnSetHook)GetProcAddress(hHookLib,"SetHook");
				if( proc ) return proc(hHookLib, hWnd);
			}
			return FALSE;
		}
	VOID RemoveHook()
		{
			if( hHookLib ) {
				lpfnRemoveHook proc = (lpfnRemoveHook)GetProcAddress(hHookLib,"RemoveHook");
				if( proc ) proc();
			}
		}
	BOOL GetWndCWD(HWND hWnd, LPTSTR pszCwd, DWORD cbCwdMaxLen)
		{
			if( hHookLib ) {
				lpfnGetWndCWD proc = (lpfnGetWndCWD)GetProcAddress(hHookLib,"GetWndCWD");
				if( proc ) return proc(hWnd, pszCwd, cbCwdMaxLen);
			}
			return FALSE;
		}

	BOOL SetWndCWD(HWND hWnd, LPTSTR pszCwd)
		{
			if( hHookLib ) {
				lpfnSetWndCWD proc = (lpfnSetWndCWD)GetProcAddress(hHookLib,"SetWndCWD");
				if( proc ) return proc(hWnd, pszCwd);
			}
			return FALSE;
		}
};

class CComboBoxEx2: public CComboBoxEx {
public:
	CComboBoxEx2() {}
protected:
	BOOL OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) {
		NMHDR* pNM = (NMHDR*)lParam;
		if( pNM->code == TTN_NEEDTEXTA ||
			pNM->code == TTN_NEEDTEXTW ||
			pNM->code == TTN_SHOW ||
			pNM->code == TTN_POP)
		{
			*pResult = GetParent()->SendMessage(WM_NOTIFY, wParam, lParam);
			return TRUE;
		} else
			return CComboBoxEx::OnNotify(wParam, lParam, pResult);
	}
};
 
struct CMouseAction {
	char data[maLast * kmLast];
	CMouseAction()
		{ ZeroMemory(data, sizeof(data)); }
	CMouseAction& operator =(const CMouseAction& other)
		{ CopyMemory(data, other.data, sizeof(data)); return *this; }
	int GetAction(UINT nMouseClick, ModifierKeys eModKey)
	{
		int action;
		switch( nMouseClick ) {
		case WM_LBUTTONUP:		action = maLeftBtnClick;	break;
		case WM_LBUTTONDBLCLK:	action = maLeftBtnDblClick;	break;
		case WM_RBUTTONUP:		action = maRightBtnClick;	break;
		case WM_RBUTTONDBLCLK:	action = maRightBtnDblClick;break;
		case WM_MBUTTONUP:		action = maMiddleBtnClick;	break;
		case WM_MBUTTONDBLCLK:	action = maMiddleBtnDblClick;break;
		default: return 0;}
		action += maLast * eModKey;
		if( action < 0 || action >= sizeof(data) )
			return 0;
		return data[action];
	}
	void Save(LPCTSTR pszRegSection) 
	{
		HKEY hKey = AfxGetApp()->GetSectionKey(_T(""));
		AfxGetApp()->DelRegTree(hKey, pszRegSection);
		RegCloseKey(hKey);

		for(int i=0;i<countof(data);i++)
			AfxGetApp()->WriteProfileInt(
				pszRegSection, StrPrintf(_T("%d"),i), data[i]);
	}
	void Restore(LPCTSTR pszRegSection) 
	{
		for(int i=0;i<countof(data);i++)
			data[i] = AfxGetApp()->GetProfileInt(
				pszRegSection, StrPrintf(_T("%d"),i), data[i]);
	}
};

class CMenuIDList:
	public CList<WORD,WORD>
{
public:
	CMenuIDList() {}
	CMenuIDList& operator = (CMenuIDList& other)
		{ this->RemoveAll(); this->AddHead(&other); return *this; }
	void Save(LPCTSTR pszRegSection) {
		HKEY hKey = AfxGetApp()->GetSectionKey(_T(""));
		AfxGetApp()->DelRegTree(hKey, pszRegSection);
		RegCloseKey(hKey);
		
		int i = 0; POSITION p = this->GetHeadPosition();
		while(p) AfxGetApp()->WriteProfileInt(pszRegSection, StrPrintf(_T("%d"),i++), GetNext(p));
	}
	void Restore(LPCTSTR pszRegSection) {
		int data, i = 0;
		while( data = AfxGetApp()->GetProfileInt(pszRegSection, StrPrintf(_T("%d"),i++), 0) )
			AddTail(data);
	}
};

/////////////////////////////////////////////////////////////////////////////
// CWCatApp:
class CWCatApp : public CWinApp
{
	struct InitCOM {
		InitCOM()  {CoInitialize(NULL);}
		~InitCOM() {CoUninitialize();}
	};
	InitCOM _g_initCom;
	LPCTSTR	m_pszMainWndClass;
	BOOL	m_bServiceRegistered;
public:
	CString GetHelpFileName();
	CString GetExeFileName();

	void SetAutoRun(BOOL bRun=TRUE);
	BOOL IsAutoRun();

	void SetLanguage(UINT nLang);
	UINT GetLanguage();

	void UnregisterService() {	if( IsServiceRegistered() )	::RegisterServiceProcess(NULL,0); }
	BOOL IsServiceRegistered() { return m_bServiceRegistered; }
	BOOL GetRegisterService() { return GetProfileInt(_T(""),_T("RunAsService"),FALSE); }
	void SetRegisterService(BOOL bOn) { WriteProfileInt(_T(""),_T("RunAsService"),bOn); }

	CWCatApp():
		m_pszMainWndClass(FALSE),
		m_bServiceRegistered(FALSE)
		{}

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CWCatApp)
	public:
	virtual BOOL InitInstance();
	virtual void WinHelp(DWORD dwData, UINT nCmd = HELP_CONTEXT);
	virtual BOOL OnIdle(LONG lCount);
	virtual int ExitInstance();
	//}}AFX_VIRTUAL

// Implementation

	//{{AFX_MSG(CWCatApp)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

class CShlRootArray:
	public CArray<CShlRoot*,CShlRoot*>
{
public:
	CShlRootArray() {}
	void Save(LPCTSTR pszSection);
	void Restore(LPCTSTR pszSection);
	int	 FindPidl(LPCITEMIDLIST pidl);
};

inline 
void AFXAPI DestructElements(CPxTrayIcon** pItems, int nCount)
	{ for( ;nCount>0;nCount-- ) delete *pItems++; }

inline
void AFXAPI DestructElements(CShlRoot** pItems, int nCount)
	{ for( ;nCount>0;nCount-- ) delete *pItems++; }

/////////////////////////////////////////////////////////////////////////////
// MainWnd window
class CMainWnd : public CWnd
{
	friend class CPPMisc;
	friend BOOL CALLBACK RestoreWndTrayIcons(HWND,LPARAM);
public:
	CArray<CPxTrayIcon*, CPxTrayIcon*> m_aShellTrayIcons;
	CArray<CPxTrayIcon*, CPxTrayIcon*> m_aWndTrayIcons;
	CWndList		m_aWindows;
	CRuleSet		m_aRuleSet;
	CShlRoot		m_CwdMenuTemplate;
	CShlRootArray	m_aShellMenus;
	CShlRootArray	m_aShellTemp;
	CPxMenuManager	m_Manager;
	CPxTrayIcon		m_TrayIcon;
	CWCatHook		m_Hook;
public:
//	options
	CMouseAction	m_MouseAction;
	CMouseAction	m_MouseTAction;
	BOOL	 m_bUseCompact;
	BOOL	 m_bUseRestore;
	BOOL	 m_bUseOverlay;		   // mark hidden windows with overlay images	
	BOOL	 m_bSortWindows;       // sort windows by groups in WatchCat menu
	BOOL	 m_bTopOnly;		   // show application windows only
	BOOL	 m_bShowIcons;		   // show icons in window menu
	UINT	 m_nTruncate;		   // truncate window names 
	CString  m_sIconFile;		   // WatchCat icon file
	UINT	 m_nIconIndex;		   // Watchcat icon index
	BOOL	 m_bEnableIcon;		   // show WatchCat icon in tray 
	BOOL	 m_bFixDskText;		   // make desktop icon titles transparent
	COLORREF m_crDskText;
	BOOL	 m_bRegisterService;   // call "RegisterServiceProcess()" or no
	CMenuIDList m_lsDisabledMenus; // list of disabled menu items
	BOOL	 m_bPasswordMenu;	   // protect WatchCat menu with password
	BOOL	 m_bEjectCDAtExit;
//	misc
	UINT	 m_nLockUI;
	UINT	 m_bTimerLocked;
	BOOL	 m_bPasswordVerified; // password verification flag - used to prevent multiple password requests.
	HWND	 m_hJustHiddenWnd;	  // temporary storage for the of window just has been hidden.
//	HotKeys
	WORD	 m_HotKey[hkLast];
public:
	void ToggleDesktopIcons(BOOL bOn = -1); //if bOn == -1 toggle it, otherwise set it on or off
	void ToggleTaskbar(BOOL bOn = -1);		//if bOn == -1 toggle it, otherwise set it on or off
	void SetHotKeys(BOOL bSet=TRUE);
	BOOL SetHotKey(HotKeys index, WORD wHotKey);
	BOOL ValidateHotKey(HotKeys hkIndex, WORD wHotKey);
	void SaveConfig();
	void ReadConfig();
	void BuildWndMenu(CMenu2& wndMenu);
	BOOL IsScreensaverRunning();
	BOOL IsWndInTray(HWND hWnd);
	void TrackWndMenu(CPoint& p);
	CString TruncateWndName(LPCTSTR szName);
	CPoint GetTrayCoords();
	void LockUI() { m_nLockUI++; };
	void UnlockUI() { if( m_nLockUI ) m_nLockUI--; };
	void LockTimer(BOOL bLock) { m_bTimerLocked = bLock; }
	void UpdateTrayIcons();

	CMainWnd();
	virtual ~CMainWnd();
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMainWnd)
	protected:
	virtual void PostNcDestroy();
	virtual LRESULT WindowProc(UINT message, WPARAM wParam, LPARAM lParam);
	//}}AFX_VIRTUAL
protected:
	void RunDialog();
	//{{AFX_MSG(CMainWnd)
	afx_msg int  OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	afx_msg BOOL OnHelpInfo(HELPINFO* pHelpInfo);
	afx_msg void OnEndSession(BOOL bEnding);
	//}}AFX_MSG
	afx_msg	void OnTrayIcon(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnRestoreIcon(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnHotKey(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnProperties(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnShellMenu(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnFixDskText(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnShellRun(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnShellTrayIcon(WPARAM wParam, LPARAM lParam);
//	hook callbacks
	afx_msg	void OnWndMinimize(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnWndActivate(WPARAM wParam, LPARAM lParam);
//	window tray icon management
	afx_msg	void OnWndTrayIcon(WPARAM wParam, LPARAM lParam);
	afx_msg	LPARAM OnCreateTrayIcon(WPARAM wParam, LPARAM lParam);
	afx_msg	LPARAM OnVerifyPassword(WPARAM wParam, LPARAM lParam);
	afx_msg	void OnSetJustHidden(WPARAM wParam, LPARAM lParam);
	afx_msg	LPARAM OnGetConfig(WPARAM wParam, LPARAM lParam);
	DECLARE_MESSAGE_MAP()
};


//{{AFX_INSERT_LOCATION}}
//
#endif //__WCAT_H_INCLUDED__
