/*
 * Copyright (c) 1998,1999 Vassili Bourdo
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice in the documentation and/or other materials provided with 
 *    the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN 
 * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include <windows.h>
#undef min
#undef max
#include "WCATHK.H"
#include "globals.h"

#pragma data_seg(".sdata")
// variables shared between all instances of this DLL
HWND			hAppWnd     = NULL;
HHOOK			hMouseHook  = NULL;
HHOOK			hCwpHook    = NULL;
// "smartclick" vars
struct {
	HWND	  hWnd;
	DWORD	  dwTime;
	CLICKDATA data;
} saveClick = { NULL };
// getting various data not accessible directly from foreign process
UINT			WM_GETPROCESSGUTS		= NULL;
TCHAR			szCwdBuffer[MAX_PATH]	= {0};
#pragma data_seg()

inline static
void* memcpy(void* _dest, const void* _src, size_t size)
{
	char* src=(char*)_src;
	char* dest=(char*)_dest;
	while( size-- > 0 ) *dest++ = *src++;
	return _dest;
}

inline static
void* memset(void* _dest, int ch, size_t size)
{
	char* dest=(char*)_dest;
	while( size-- > 0 ) *dest++ = ch;
	return _dest;
}

template<class T> T min(T a, T b)
{
	return (a < b)?a:b;
}

#ifdef _DEBUG
#define DBGPRINTF(args)	DbgPrintf args
VOID DbgPrintf(LPCTSTR fmt,...)
{
    va_list marker;
    TCHAR szBuf[256];

    va_start(marker, fmt);
    wvsprintf(szBuf, fmt, marker);
    va_end(marker);

    OutputDebugString(szBuf);
}
#else
#define DBGPRINTF(args)
#endif

#define MHPS ((MOUSEHOOKSTRUCT*)lParam)
#define CWPS ((CWPSTRUCT*)lParam)

static
void ResetClick()
{
	memset(&saveClick, 0, sizeof(saveClick));
}

LRESULT CALLBACK 
MouseProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if( nCode >= 0 ) {
		switch( wParam ) {
		case WM_NCRBUTTONDOWN:
		case WM_NCMBUTTONDOWN:
			if( MHPS->wHitTestCode == HTREDUCE ||
				MHPS->wHitTestCode == HTZOOM   || 
				MHPS->wHitTestCode == HTCLOSE )
			{
				saveClick.hWnd				 = MHPS->hwnd;
				saveClick.dwTime			 = ::GetTickCount();
				saveClick.data.nHitTestCode  = MHPS->wHitTestCode;
				saveClick.data.nMouseButton  = wParam == WM_NCRBUTTONDOWN?1:2;
			} else {
				ResetClick();
			}
			break;
		case WM_LBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_RBUTTONDOWN:
		case WM_RBUTTONUP:
		case WM_MBUTTONDOWN:
		case WM_MBUTTONUP:
			ResetClick();
			break;
		case WM_NCRBUTTONUP: 
		case WM_NCMBUTTONUP: {
			BOOL bIntercept = FALSE;
			if( (MHPS->wHitTestCode == HTREDUCE || 
				 MHPS->wHitTestCode == HTZOOM || 
				 MHPS->wHitTestCode == HTCLOSE) &&
				(saveClick.hWnd == NULL || MHPS->hwnd == saveClick.hWnd) &&
				(saveClick.data.nHitTestCode == 0 || MHPS->wHitTestCode == saveClick.data.nHitTestCode) )
			{
				CLICKDATA click;
				click.dwData = 0;
				click.nMouseButton = (wParam==WM_NCRBUTTONUP?1:2);
				click.nHitTestCode = MHPS->wHitTestCode;
				click.bLongClick =
					(MHPS->hwnd == saveClick.hWnd) &&
					(click.nMouseButton == saveClick.data.nMouseButton) &&
					(MHPS->wHitTestCode == saveClick.data.nHitTestCode) &&
					(::GetTickCount() - saveClick.dwTime) >= ::GetDoubleClickTime();
				PostMessage(hAppWnd, WM_WNDMINIMIZE, (WPARAM)click.dwData, (LPARAM)MHPS->hwnd);
				bIntercept = TRUE;
			}
			ResetClick();
			if( bIntercept ) return 1;
			}break;
		default:;
        }
	}
	return CallNextHookEx(hMouseHook, nCode, wParam, lParam);
}

LRESULT CALLBACK 
CallWndProc(int nCode, WPARAM wParam, LPARAM lParam)
{
	if( nCode >= 0 ) {
		if( WM_GETPROCESSGUTS &&
			CWPS->message == WM_GETPROCESSGUTS )
		{
			if( CWPS->wParam == 1 ) {
				DBGPRINTF(("WCATHK.DLL: WM_GETPROCESSGUTS for hwnd=0x%X\n", CWPS->hwnd));
				GetCurrentDirectory(sizeof(szCwdBuffer)/sizeof(szCwdBuffer[0]), 
					szCwdBuffer);
				return 0;
			} else 
			if( CWPS->wParam == 2 ) {
				DBGPRINTF(("WCATHK.DLL: WM_GETPROCESSGUTS for hwnd=0x%X\n", CWPS->hwnd));
				SetCurrentDirectory(szCwdBuffer);
				return 0;
			}
		} else
		if( CWPS->message == WM_ACTIVATE &&
			(BOOL)CWPS->wParam != WA_INACTIVE)
		{
			DBGPRINTF(("WCATHK.DLL: WM_ACTIVATE for hwnd=0x%X\n", CWPS->hwnd));
			PostMessage(hAppWnd, WM_WNDACTIVATE, 0, (LPARAM)CWPS->hwnd);
		}
	}
	return CallNextHookEx(hCwpHook, nCode, wParam, lParam);
}

BOOL FAR WINAPI SetHook(HMODULE hThisDll, HWND hWnd)
{
	if( hWnd == hAppWnd && hMouseHook && hCwpHook)
		return TRUE;
	
	RemoveHook();
	hAppWnd = hWnd;
	hMouseHook = SetWindowsHookEx(WH_MOUSE, (HOOKPROC)MouseProc, hThisDll, 0);
	hCwpHook   = SetWindowsHookEx(WH_CALLWNDPROC, (HOOKPROC)CallWndProc, hThisDll, 0);
	if( hMouseHook == NULL || hCwpHook == NULL) {
		RemoveHook();
		return FALSE;
	}
	return TRUE;
}

VOID FAR WINAPI RemoveHook(void)
{
	if( hMouseHook ) { UnhookWindowsHookEx(hMouseHook);	hMouseHook = NULL; }
	if( hCwpHook ) { UnhookWindowsHookEx(hCwpHook); hCwpHook = NULL; }
	hAppWnd = NULL;
}

HWND FAR WINAPI IsHookSet(void)
{
	return (hMouseHook && hCwpHook)?hAppWnd:NULL;
}

LPCTSTR NAME_GETPROCESSGUTS = "WCATHK-GetProcessGuts";
LPCTSTR NAME_CWDMUTEX		= "WCATHK-CwdBuffer";

BOOL FAR WINAPI GetWndCWD(HWND hWnd, LPTSTR pszCwd, DWORD cbCwdMaxLen)
{
	DBGPRINTF(("WCATHK::GetWndCWD(%x) - Start",hWnd));
	if( !IsWindow(hWnd) ) {
		DBGPRINTF(("WCATHK::GetWndCWD(%x) - Wrong HWND",hWnd));
		return FALSE;
	}
	if( WM_GETPROCESSGUTS == 0 ) {
		WM_GETPROCESSGUTS = RegisterWindowMessage(NAME_GETPROCESSGUTS);
		if( WM_GETPROCESSGUTS == 0 ) {
			DBGPRINTF(("WCATHK::GetWndCWD() - RegisterWindowMessage failed"));
			return FALSE;
		}
	}
	HANDLE hMutex = CreateMutex(NULL,FALSE, NAME_CWDMUTEX);
	if( !hMutex ) {
		DBGPRINTF(("WCATHK::GetWndCWD() - The CreateMutex failed"));
		return FALSE;
	}
	if( WaitForSingleObject(hMutex, 100) == WAIT_OBJECT_0 )
	{
		szCwdBuffer[0] = 0;
		SendMessage(hWnd, WM_GETPROCESSGUTS, 1, 0);
		memcpy(pszCwd, szCwdBuffer,
			min((int)cbCwdMaxLen * sizeof(TCHAR), sizeof(szCwdBuffer)));
		ReleaseMutex(hMutex);
		CloseHandle(hMutex);
		DBGPRINTF(("WCATHK::GetWndCWD(%x) - Success",hWnd));
		return TRUE;
	} else {
		DBGPRINTF(("WCATHK::GetWndCWD(%x) - Cannot get mutex ownreship",hWnd));
		CloseHandle(hMutex);
		return FALSE;
	}
}

BOOL FAR WINAPI SetWndCWD(HWND hWnd, LPTSTR pszCwd)
{
	DBGPRINTF(("WCATHK::SetWndCWD(%x) - Start",hWnd));
	if( !IsWindow(hWnd) ) {
		DBGPRINTF(("WCATHK::SetWndCWD(%x) - Wrong HWND",hWnd));
		return FALSE;
	}
	if( WM_GETPROCESSGUTS == 0 ) {
		WM_GETPROCESSGUTS = RegisterWindowMessage(NAME_GETPROCESSGUTS);
		if( WM_GETPROCESSGUTS == 0 ) {
			DBGPRINTF(("WCATHK::SetWndCWD() - RegisterWindowMessage failed"));
			return FALSE;
		}
	}
	HANDLE hMutex = CreateMutex(NULL,FALSE, NAME_CWDMUTEX);
	if( !hMutex ) {
		DBGPRINTF(("WCATHK::SetWndCWD() - The CreateMutex failed"));
		return FALSE;
	}
	if( WaitForSingleObject(hMutex, 100) == WAIT_OBJECT_0 )
	{
		szCwdBuffer[0] = 0;
		lstrcpy(szCwdBuffer, pszCwd);
		SendMessage(hWnd, WM_GETPROCESSGUTS, 2, 0);
		ReleaseMutex(hMutex);
		CloseHandle(hMutex);
		DBGPRINTF(("WCATHK::SetWndCWD(%x) - Success",hWnd));
		return TRUE;
	} else {
		DBGPRINTF(("WCATHK::SetWndCWD(%x) - Cannot get mutex ownreship",hWnd));
		CloseHandle(hMutex);
		return FALSE;
	}
}

BOOL FAR WINAPI DllMain(HINSTANCE hinstDLL,DWORD fdwReason,LPVOID lpvReserved)
{
	return TRUE;
}
